@@ -5,6 +5,8 @@
  *
  * Author: Cecilie Fritzvold <cecilihf@linpro.no>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -52,7 +54,6 @@
 #include <math.h>
 #include <fcntl.h>
 #include <poll.h>
-#include <signal.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
@@ -74,6 +75,7 @@
 
 
 static const double timeout = 5;	// XXX should be settable by arg ?
+static int p_arg = 0;
 
 static void
 cli_write(int sock, const char *s)
@@ -154,30 +156,56 @@ cli_sock(const char *T_arg, const char *S_arg)
 	return (sock);
 }
 
-static void
+static unsigned
+pass_answer(int fd)
+{
+	unsigned u, status;
+	char *answer = NULL;
+
+	u = VCLI_ReadResult(fd, &status, &answer, timeout);
+	if (u) {
+		if (status == CLIS_COMMS)
+			RL_EXIT(0);
+		if (answer)
+			fprintf(stderr, "%s\n", answer);
+		RL_EXIT(1);
+	}
+
+	if (p_arg && answer != NULL) {
+		printf("%-3u %-8zu\n%s", status, strlen(answer), answer);
+	} else if (p_arg) {
+		printf("%-3u %-8u\n", status, 0U);
+	} else {
+		printf("%u\n", status);
+		if (answer != NULL)
+			printf("%s\n", answer);
+		if (status == CLIS_TRUNCATED)
+			printf("[response was truncated]\n");
+	}
+	free(answer);
+	(void)fflush(stdout);
+	return (status);
+}
+
+static void v_noreturn_
 do_args(int sock, int argc, char * const *argv)
 {
 	int i;
 	unsigned status;
-	char *answer = NULL;
 
-	for (i=0; i<argc; i++) {
+	for (i = 0; i < argc; i++) {
 		/* XXX: We should really CLI-quote these */
 		if (i > 0)
 			cli_write(sock, " ");
 		cli_write(sock, argv[i]);
 	}
 	cli_write(sock, "\n");
-
-	(void)VCLI_ReadResult(sock, &status, &answer, 2000);
-
-	/* XXX: AZ() ? */
-	(void)close(sock);
-
-	printf("%s\n", answer);
+	status = pass_answer(sock);
+	closefd(&sock);
 	if (status == CLIS_OK || status == CLIS_TRUNCATED)
 		exit(0);
-	fprintf(stderr, "Command failed with error code %u\n", status);
+	if (!p_arg)
+		fprintf(stderr, "Command failed with error code %u\n", status);
 	exit(1);
 }
 
@@ -234,34 +262,12 @@ varnishadm_completion (const char *text, int start, int end)
 	return (matches);
 }
 
-static void
-pass_answer(int fd)
-{
-	unsigned u, status;
-	char *answer = NULL;
-
-	u = VCLI_ReadResult(fd, &status, &answer, timeout);
-	if (u) {
-		if (status == CLIS_COMMS)
-			RL_EXIT(0);
-		if (answer)
-			fprintf(stderr, "%s\n", answer);
-		RL_EXIT(1);
-	}
-
-	printf("%u\n", status);
-	if (answer) {
-		printf("%s\n", answer);
-		free(answer);
-	}
-	(void)fflush(stdout);
-}
 
 /*
  * No arguments given, simply pass bytes on stdin/stdout and CLI socket
  * Send a "banner" to varnish, to provoke a welcome message.
  */
-static void
+static void v_noreturn_
 interactive(int sock)
 {
 	struct pollfd fds[2];
@@ -316,7 +322,7 @@ interactive(int sock)
 		if (fds[0].revents & POLLIN) {
 			/* Get rid of the prompt, kinda hackish */
 			printf("\r           \r");
-			pass_answer(fds[0].fd);
+			(void)pass_answer(fds[0].fd);
 			rl_forced_update_display();
 		}
 		if (fds[1].revents & POLLIN) {
@@ -328,7 +334,7 @@ interactive(int sock)
 /*
  * No arguments given, simply pass bytes on stdin/stdout and CLI socket
  */
-static void
+static void v_noreturn_
 pass(int sock)
 {
 	struct pollfd fds[2];
@@ -347,7 +353,7 @@ pass(int sock)
 		}
 		assert(i > 0);
 		if (fds[0].revents & POLLIN)
-			pass_answer(fds[0].fd);
+			(void)pass_answer(fds[0].fd);
 		if (fds[1].revents & POLLIN || fds[1].revents & POLLHUP) {
 			n = read(fds[1].fd, buf, sizeof buf - 1);
 			if (n == 0) {
@@ -417,6 +423,8 @@ n_arg_sock(const char *n_arg, const char *t_arg)
 	return (sock);
 }
 
+#define OPTARG "hn:pS:T:t:"
+
 int
 main(int argc, char * const *argv)
 {
@@ -426,6 +434,10 @@ main(int argc, char * const *argv)
 	const char *t_arg = NULL;
 	int opt, sock;
 
+	if (argc == 2 && !strcmp(argv[1], "--optstring")) {
+		printf(OPTARG "\n");
+		exit(0);
+	}
 	/*
 	 * By default linux::getopt(3) mangles the argv order, such that
 	 *	varnishadm -n bla param.set foo -bar
@@ -434,7 +446,7 @@ main(int argc, char * const *argv)
 	 * The '+' stops that from happening
 	 * See #1496
 	 */
-	while ((opt = getopt(argc, argv, "+hn:S:T:t:")) != -1) {
+	while ((opt = getopt(argc, argv, "+" OPTARG)) != -1) {
 		switch (opt) {
 		case 'h':
 			/* Usage help */
@@ -442,6 +454,9 @@ main(int argc, char * const *argv)
 		case 'n':
 			n_arg = optarg;
 			break;
+		case 'p':
+			p_arg = 1;
+			break;
 		case 'S':
 			S_arg = optarg;
 			break;
@@ -459,9 +474,6 @@ main(int argc, char * const *argv)
 	argc -= optind;
 	argv += optind;
 
-	VSIG_Arm_int();
-	VSIG_Arm_term();
-
 	if (T_arg != NULL) {
 		if (n_arg != NULL)
 			usage(1);
@@ -474,11 +486,16 @@ main(int argc, char * const *argv)
 	if (sock < 0)
 		exit(2);
 
-	if (argc > 0)
+	if (argc > 0) {
+		VSIG_Arm_int();
+		VSIG_Arm_term();
 		do_args(sock, argc, argv);
-	else if (isatty(0))
+		NEEDLESS(exit(0));
+	}
+
+	if (isatty(0) && !p_arg)
 		interactive(sock);
 	else
 		pass(sock);
-	exit(0);
+	NEEDLESS(exit(0));
 }
