@@ -41,7 +41,6 @@
 
 #include <poll.h>
 #include <stdio.h>
-#include <string.h>
 #include <stdlib.h>
 
 #include "cache_varnishd.h"
@@ -53,7 +52,7 @@
 #include "vtim.h"
 
 #include "cache_backend.h"
-#include "cache_tcp_pool.h"
+#include "cache_conn_pool.h"
 
 /* Default averaging rate, we want something pretty responsive */
 #define AVG_RATE			4
@@ -65,7 +64,7 @@ struct vbp_target {
 	VRT_BACKEND_PROBE_FIELDS()
 
 	struct backend			*backend;
-	struct tcp_pool			*tcp_pool;
+	struct conn_pool		*conn_pool;
 
 	char				*req;
 	int				req_len;
@@ -104,7 +103,7 @@ vbp_delete(struct vbp_target *vt)
 #define DN(x)	/**/
 	VRT_BACKEND_PROBE_HANDLE();
 #undef DN
-	VTP_Rel(&vt->tcp_pool);
+	VCP_Rel(&vt->conn_pool);
 	free(vt->req);
 	FREE_OBJ(vt);
 }
@@ -229,6 +228,7 @@ vbp_write(struct vbp_target *vt, int *sock, const void *buf, size_t len)
 	int i;
 
 	i = write(*sock, buf, len);
+	VTCP_Assert(i);
 	if (i != len) {
 		if (i < 0) {
 			vt->err_xmit |= 1;
@@ -251,21 +251,24 @@ vbp_write_proxy_v1(struct vbp_target *vt, int *sock)
 	char buf[105]; /* maximum size for a TCP6 PROXY line with null char */
 	char addr[VTCP_ADDRBUFSIZE];
 	char port[VTCP_PORTBUFSIZE];
-	struct sockaddr_storage ss;
+	char vsabuf[vsa_suckaddr_len];
+	struct suckaddr *sua;
+	int proto;
 	struct vsb vsb;
-	socklen_t l;
 
-	VTCP_myname(*sock, addr, sizeof addr, port, sizeof port);
+	sua = VSA_getsockname(*sock, vsabuf, sizeof vsabuf);
+	AN(sua);
 	AN(VSB_init(&vsb, buf, sizeof buf));
 
-	l = sizeof ss;
-	AZ(getsockname(*sock, (void *)&ss, &l));
-	if (ss.ss_family == AF_INET || ss.ss_family == AF_INET6) {
+	proto = VSA_Get_Proto(sua);
+	if (proto == AF_INET || proto == AF_INET6) {
+		VTCP_name(sua, addr, sizeof addr, port, sizeof port);
 		VSB_printf(&vsb, "PROXY %s %s %s %s %s\r\n",
-		    ss.ss_family == AF_INET ? "TCP4" : "TCP6",
+		    proto == AF_INET ? "TCP4" : "TCP6",
 		    addr, addr, port, port);
-	} else
+	} else {
 		VSB_cat(&vsb, "PROXY UNKNOWN\r\n");
+	}
 	AZ(VSB_finish(&vsb));
 
 	VSB_fini(&vsb);
@@ -285,7 +288,7 @@ vbp_poke(struct vbp_target *vt)
 	t_start = t_now = VTIM_real();
 	t_end = t_start + vt->timeout;
 
-	s = VTP_Open(vt->tcp_pool, t_end - t_now, (const void **)&sa, &err);
+	s = VCP_Open(vt->conn_pool, t_end - t_now, &sa, &err);
 	if (s < 0) {
 		bprintf(vt->resp_buf, "Open error %d (%s)", err, vstrerror(err));
 		Lck_Lock(&vbp_mtx);
@@ -371,6 +374,7 @@ vbp_poke(struct vbp_target *vt)
 			    sizeof vt->resp_buf - rlen);
 		else
 			i = read(s, buf, sizeof buf);
+		VTCP_Assert(i);
 		if (i <= 0) {
 			if (i < 0)
 				bprintf(vt->resp_buf, "Read error %d (%s)",
@@ -485,6 +489,7 @@ vbp_thread(struct worker *wrk, void *priv)
 		} else {
 			VBH_delete(vbp_heap, vt->heap_idx);
 			vt->due = now + vt->interval;
+			VBH_insert(vbp_heap, vt);
 			if (!vt->running) {
 				vt->running = 1;
 				vt->task->func = vbp_task;
@@ -495,7 +500,6 @@ vbp_thread(struct worker *wrk, void *priv)
 				if (r)
 					vt->running = 0;
 			}
-			VBH_insert(vbp_heap, vt);
 		}
 	}
 	NEEDLESS(Lck_Unlock(&vbp_mtx));
@@ -672,7 +676,7 @@ VBP_Control(const struct backend *be, int enable)
 
 void
 VBP_Insert(struct backend *b, const struct vrt_backend_probe *vp,
-    struct tcp_pool *tp)
+    struct conn_pool *tp)
 {
 	struct vbp_target *vt;
 
@@ -684,8 +688,8 @@ VBP_Insert(struct backend *b, const struct vrt_backend_probe *vp,
 	ALLOC_OBJ(vt, VBP_TARGET_MAGIC);
 	XXXAN(vt);
 
-	vt->tcp_pool = tp;
-	VTP_AddRef(vt->tcp_pool);
+	vt->conn_pool = tp;
+	VCP_AddRef(vt->conn_pool);
 	vt->backend = b;
 	b->probe = vt;
 
@@ -693,7 +697,6 @@ VBP_Insert(struct backend *b, const struct vrt_backend_probe *vp,
 	vbp_build_req(vt, vp, b);
 
 	vbp_reset(vt);
-	VBP_Update_Backend(vt);
 }
 
 void
