@@ -206,8 +206,8 @@ VDI_Http1Pipe(struct req *req, struct busyobj *bo)
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 	CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
 	INIT_OBJ(ctx, VRT_CTX_MAGIC);
-	VCL_Req2Ctx(ctx, req);
 	VCL_Bo2Ctx(ctx, bo);
+	VCL_Req2Ctx(ctx, req);
 
 	d = VDI_Resolve(ctx);
 	if (d == NULL || d->vdir->methods->http1pipe == NULL) {
@@ -354,7 +354,7 @@ do_list(struct cli *cli, struct director *d, void *priv)
 	if (la->p && d->vdir->methods->list != NULL)
 		d->vdir->methods->list(ctx, d, la->vsb, la->p, 0);
 
-	VSB_printf(la->vsb, "\n");
+	VSB_cat(la->vsb, "\n");
 	AZ(VCL_Rel_CliCtx(&ctx));
 	AZ(ctx);
 
