@@ -139,6 +139,7 @@ ved_include(struct req *preq, const char *src, const char *host,
 
 	VSLb_ts_req(req, "Start", W_TIM_real(wrk));
 
+	wrk->stats->esi_req++;
 	req->esi_level = preq->esi_level + 1;
 
 	memset(req->top, 0, sizeof *req->top);
@@ -181,16 +182,16 @@ ved_include(struct req *preq, const char *src, const char *host,
 		req->vcl = preq->vcl;
 	VCL_Ref(req->vcl);
 
-	req->req_step = R_STP_TRANSPORT;
+	assert(req->req_step == R_STP_TRANSPORT);
 	req->t_req = preq->t_req;
 
 	req->transport = &VED_transport;
 	req->transport_priv = ecx;
 
-	CNT_Embark(wrk, req);
 	VCL_TaskEnter(req->privs);
 
 	while (1) {
+		CNT_Embark(wrk, req);
 		ecx->woken = 0;
 		s = CNT_Request(req);
 		if (s == REQ_FSM_DONE)
@@ -204,7 +205,6 @@ ved_include(struct req *preq, const char *src, const char *host,
 			    &ecx->preq->wrk->cond, &sp->mtx, 0);
 		Lck_Unlock(&sp->mtx);
 		AZ(req->wrk);
-		CNT_Embark(wrk, req);
 	}
 
 	VCL_Rel(&req->vcl);
@@ -222,7 +222,7 @@ ved_include(struct req *preq, const char *src, const char *host,
 #define Debug(fmt, ...) /**/
 
 static ssize_t
-ved_decode_len(struct req *req, const uint8_t **pp)
+ved_decode_len(struct vsl_log *vsl, const uint8_t **pp)
 {
 	const uint8_t *p;
 	ssize_t l;
@@ -242,7 +242,7 @@ ved_decode_len(struct req *req, const uint8_t **pp)
 		p += 9;
 		break;
 	default:
-		VSLb(req->vsl, SLT_Error,
+		VSLb(vsl, SLT_Error,
 		    "ESI-corruption: Illegal Length %d %d\n", *p, (*p & 15));
 		WRONG("ESI-codes: illegal length");
 	}
@@ -255,17 +255,21 @@ ved_decode_len(struct req *req, const uint8_t **pp)
  */
 
 static int v_matchproto_(vdp_init_f)
-ved_vdp_esi_init(struct req *req, void **priv)
+ved_vdp_esi_init(struct vdp_ctx *vdc, void **priv, struct objcore *oc)
 {
 	struct ecx *ecx;
+	struct req *req;
 
+	CHECK_OBJ_NOTNULL(vdc, VDP_CTX_MAGIC);
+	CHECK_OBJ_ORNULL(oc, OBJCORE_MAGIC);
+	if (oc == NULL || !ObjHasAttr(vdc->wrk, oc, OA_ESIDATA))
+		return (1);
+
+	req = vdc->req;
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 	AN(priv);
 	AZ(*priv);
 
-	if (!ObjHasAttr(req->wrk, req->objcore, OA_ESIDATA))
-		return (1);
-
 	ALLOC_OBJ(ecx, ECX_MAGIC);
 	AN(ecx);
 	assert(sizeof gzip_hdr == 10);
@@ -287,11 +291,11 @@ ved_vdp_esi_init(struct req *req, void **priv)
 }
 
 static int v_matchproto_(vdp_fini_f)
-ved_vdp_esi_fini(struct req *req, void **priv)
+ved_vdp_esi_fini(struct vdp_ctx *vdc, void **priv)
 {
 	struct ecx *ecx;
 
-	(void)req;
+	(void)vdc;
 	AN(priv);
 	CAST_OBJ_NOTNULL(ecx, *priv, ECX_MAGIC);
 	FREE_OBJ(ecx);
@@ -300,7 +304,7 @@ ved_vdp_esi_fini(struct req *req, void **priv)
 }
 
 static int v_matchproto_(vdp_bytes_f)
-ved_vdp_esi_bytes(struct req *req, enum vdp_action act, void **priv,
+ved_vdp_esi_bytes(struct vdp_ctx *vdx, enum vdp_action act, void **priv,
     const void *ptr, ssize_t len)
 {
 	uint8_t *q, *r;
@@ -311,14 +315,18 @@ ved_vdp_esi_bytes(struct req *req, enum vdp_action act, void **priv,
 	struct ecx *ecx;
 	int retval = 0;
 
+	if (act == VDP_END)
+		act = VDP_FLUSH;
+
 	AN(priv);
+	CHECK_OBJ_NOTNULL(vdx, VDP_CTX_MAGIC);
 	CAST_OBJ_NOTNULL(ecx, *priv, ECX_MAGIC);
 	pp = ptr;
 
 	while (1) {
 		switch (ecx->state) {
 		case 0:
-			ecx->p = ObjGetAttr(req->wrk, req->objcore,
+			ecx->p = ObjGetAttr(vdx->wrk, ecx->preq->objcore,
 			    OA_ESIDATA, &l);
 			AN(ecx->p);
 			assert(l > 0);
@@ -326,7 +334,7 @@ ved_vdp_esi_bytes(struct req *req, enum vdp_action act, void **priv,
 
 			if (*ecx->p == VEC_GZ) {
 				if (ecx->pecx == NULL)
-					retval = VDP_bytes(req, VDP_NULL,
+					retval = VDP_bytes(vdx, VDP_NULL,
 					    gzip_hdr, 10);
 				ecx->l_crc = 0;
 				ecx->crc = crc32(0L, Z_NULL, 0);
@@ -344,14 +352,14 @@ ved_vdp_esi_bytes(struct req *req, enum vdp_action act, void **priv,
 			case VEC_V1:
 			case VEC_V2:
 			case VEC_V8:
-				ecx->l = ved_decode_len(req, &ecx->p);
+				ecx->l = ved_decode_len(vdx->vsl, &ecx->p);
 				if (ecx->l < 0)
 					return (-1);
 				if (ecx->isgzip) {
 					assert(*ecx->p == VEC_C1 ||
 					    *ecx->p == VEC_C2 ||
 					    *ecx->p == VEC_C8);
-					l = ved_decode_len(req, &ecx->p);
+					l = ved_decode_len(vdx->vsl, &ecx->p);
 					if (l < 0)
 						return (-1);
 					icrc = vbe32dec(ecx->p);
@@ -365,7 +373,7 @@ ved_vdp_esi_bytes(struct req *req, enum vdp_action act, void **priv,
 			case VEC_S1:
 			case VEC_S2:
 			case VEC_S8:
-				ecx->l = ved_decode_len(req, &ecx->p);
+				ecx->l = ved_decode_len(vdx->vsl, &ecx->p);
 				if (ecx->l < 0)
 					return (-1);
 				Debug("SKIP1(%d)\n", (int)ecx->l);
@@ -378,24 +386,26 @@ ved_vdp_esi_bytes(struct req *req, enum vdp_action act, void **priv,
 				q++;
 				r = (void*)strchr((const char*)q, '\0');
 				AN(r);
-				if (VDP_bytes(req, VDP_FLUSH, NULL, 0)) {
+				if (VDP_bytes(vdx, VDP_FLUSH, NULL, 0)) {
 					ecx->p = ecx->e;
 					break;
 				}
 				Debug("INCL [%s][%s] BEGIN\n", q, ecx->p);
-				ved_include(req,
+				ved_include(ecx->preq,
 				    (const char*)q, (const char*)ecx->p, ecx);
 				Debug("INCL [%s][%s] END\n", q, ecx->p);
 				ecx->p = r + 1;
 				break;
 			default:
-				VSLb(req->vsl, SLT_Error,
+				VSLb(vdx->vsl, SLT_Error,
 				    "ESI corruption line %d 0x%02x [%s]\n",
 				    __LINE__, *ecx->p, ecx->p);
 				WRONG("ESI-codes: Illegal code");
 			}
 			break;
 		case 2:
+			ptr = NULL;
+			len = 0;
 			if (ecx->isgzip && ecx->pecx == NULL) {
 				/*
 				 * We are bytealigned here, so simply emit
@@ -413,20 +423,18 @@ ved_vdp_esi_bytes(struct req *req, enum vdp_action act, void **priv,
 				/* MOD(2^32) length */
 				vle32enc(tailbuf + 9, ecx->l_crc);
 
-				retval = VDP_bytes(req, VDP_END, tailbuf, 13);
+				ptr = tailbuf;
+				len = 13;
 			} else if (ecx->pecx != NULL) {
 				ecx->pecx->crc = crc32_combine(ecx->pecx->crc,
 				    ecx->crc, ecx->l_crc);
 				ecx->pecx->l_crc += ecx->l_crc;
-				retval = VDP_bytes(req, VDP_FLUSH, NULL, 0);
-			} else {
-				retval = VDP_bytes(req, VDP_END, NULL, 0);
 			}
+			retval = VDP_bytes(vdx, VDP_END, ptr, len);
 			ecx->state = 99;
 			return (retval);
 		case 3:
 		case 4:
-			assert(act != VDP_END);
 			/*
 			 * There is no guarantee that the 'l' bytes are all
 			 * in the same storage segment, so loop over storage
@@ -434,7 +442,7 @@ ved_vdp_esi_bytes(struct req *req, enum vdp_action act, void **priv,
 			 */
 			if (ecx->l <= len) {
 				if (ecx->state == 3)
-					retval = VDP_bytes(req, act,
+					retval = VDP_bytes(vdx, act,
 					    pp, ecx->l);
 				len -= ecx->l;
 				pp += ecx->l;
@@ -442,7 +450,7 @@ ved_vdp_esi_bytes(struct req *req, enum vdp_action act, void **priv,
 				break;
 			}
 			if (ecx->state == 3 && len > 0)
-				retval = VDP_bytes(req, act, pp, len);
+				retval = VDP_bytes(vdx, act, pp, len);
 			ecx->l -= len;
 			return (retval);
 		case 99:
@@ -472,13 +480,12 @@ const struct vdp VDP_esi = {
  * Push bytes to preq
  */
 static inline int
-ved_bytes(struct req *req, struct ecx *ecx, enum vdp_action act,
+ved_bytes(struct ecx *ecx, enum vdp_action act,
     const void *ptr, ssize_t len)
 {
-	req->acct.resp_bodybytes += len;
 	if (act == VDP_END)
 		act = VDP_FLUSH;
-	return (VDP_bytes(ecx->preq, act, ptr, len));
+	return (VDP_bytes(ecx->preq->vdc, act, ptr, len));
 }
 
 /*---------------------------------------------------------------------
@@ -500,15 +507,15 @@ ved_bytes(struct req *req, struct ecx *ecx, enum vdp_action act,
  */
 
 static int v_matchproto_(vdp_fini_f)
-ved_pretend_gzip_fini(struct req *req, void **priv)
+ved_pretend_gzip_fini(struct vdp_ctx *vdc, void **priv)
 {
-	(void)req;
+	(void)vdc;
 	*priv = NULL;
 	return (0);
 }
 
 static int v_matchproto_(vdp_bytes_f)
-ved_pretend_gzip_bytes(struct req *req, enum vdp_action act, void **priv,
+ved_pretend_gzip_bytes(struct vdp_ctx *vdx, enum vdp_action act, void **priv,
     const void *pv, ssize_t l)
 {
 	uint8_t buf1[5], buf2[5];
@@ -516,12 +523,12 @@ ved_pretend_gzip_bytes(struct req *req, enum vdp_action act, void **priv,
 	uint16_t lx;
 	struct ecx *ecx;
 
-	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+	CHECK_OBJ_NOTNULL(vdx, VDP_CTX_MAGIC);
 	CAST_OBJ_NOTNULL(ecx, *priv, ECX_MAGIC);
 
 	(void)priv;
 	if (l == 0)
-		return (ved_bytes(req, ecx, act, pv, l));
+		return (ved_bytes(ecx, act, pv, l));
 
 	p = pv;
 
@@ -537,23 +544,23 @@ ved_pretend_gzip_bytes(struct req *req, enum vdp_action act, void **priv,
 	while (l > 0) {
 		if (l >= 65535) {
 			lx = 65535;
-			if (ved_bytes(req, ecx, VDP_NULL, buf1, sizeof buf1))
+			if (ved_bytes(ecx, VDP_NULL, buf1, sizeof buf1))
 				return (-1);
 		} else {
 			lx = (uint16_t)l;
 			buf2[0] = 0;
 			vle16enc(buf2 + 1, lx);
 			vle16enc(buf2 + 3, ~lx);
-			if (ved_bytes(req, ecx, VDP_NULL, buf2, sizeof buf2))
+			if (ved_bytes(ecx, VDP_NULL, buf2, sizeof buf2))
 				return (-1);
 		}
-		if (ved_bytes(req, ecx, VDP_NULL, p, lx))
+		if (ved_bytes(ecx, VDP_NULL, p, lx))
 			return (-1);
 		l -= lx;
 		p += lx;
 	}
 	/* buf1 & buf2 is local, have to flush */
-	return (ved_bytes(req, ecx, VDP_FLUSH, NULL, 0));
+	return (ved_bytes(ecx, VDP_FLUSH, NULL, 0));
 }
 
 static const struct vdp ved_pretend_gz = {
@@ -575,6 +582,7 @@ struct ved_foo {
 	unsigned		magic;
 #define VED_FOO_MAGIC		0x6a5a262d
 	struct ecx		*ecx;
+	struct objcore		*objcore;
 	uint64_t		start, last, stop, lpad;
 	ssize_t			ll;
 	uint64_t		olen;
@@ -583,26 +591,31 @@ struct ved_foo {
 };
 
 static int v_matchproto_(vdp_fini_f)
-ved_gzgz_init(struct req *req, void **priv)
+ved_gzgz_init(struct vdp_ctx *vdc, void **priv, struct objcore *oc)
 {
 	ssize_t l;
 	const char *p;
 	struct ved_foo *foo;
+	struct req *req;
 
+	CHECK_OBJ_NOTNULL(vdc, VDP_CTX_MAGIC);
+	(void)oc;
+	req = vdc->req;
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 	CAST_OBJ_NOTNULL(foo, *priv, VED_FOO_MAGIC);
+	CHECK_OBJ_NOTNULL(foo->objcore, OBJCORE_MAGIC);
 
 	memset(foo->tailbuf, 0xdd, sizeof foo->tailbuf);
 
-	AN(ObjCheckFlag(req->wrk, req->objcore, OF_GZIPED));
+	AN(ObjCheckFlag(vdc->wrk, foo->objcore, OF_GZIPED));
 
-	p = ObjGetAttr(req->wrk, req->objcore, OA_GZIPBITS, &l);
+	p = ObjGetAttr(vdc->wrk, foo->objcore, OA_GZIPBITS, &l);
 	AN(p);
 	assert(l == 32);
 	foo->start = vbe64dec(p);
 	foo->last = vbe64dec(p + 8);
 	foo->stop = vbe64dec(p + 16);
-	foo->olen = ObjGetLen(req->wrk, req->objcore);
+	foo->olen = ObjGetLen(vdc->wrk, foo->objcore);
 	assert(foo->start > 0 && foo->start < foo->olen * 8);
 	assert(foo->last > 0 && foo->last < foo->olen * 8);
 	assert(foo->stop > 0 && foo->stop < foo->olen * 8);
@@ -614,8 +627,18 @@ ved_gzgz_init(struct req *req, void **priv)
 	return (0);
 }
 
+/*
+ * XXX: for act == VDP_END || act == VDP_FLUSH, we send a flush more often than
+ * we need. The VDP_END case would trip our "at most one VDP_END call" assertion
+ * in VDP_bytes(), but ved_bytes() covers it.
+ *
+ * To avoid unnecessary chunks downstream, it would be nice to re-structure the
+ * code to intendify the last block, send VDP_END/VDP_FLUSH for that one and
+ * VDP_NULL for anything before it.
+ */
+
 static int v_matchproto_(vdp_bytes_f)
-ved_gzgz_bytes(struct req *req, enum vdp_action act, void **priv,
+ved_gzgz_bytes(struct vdp_ctx *vdx, enum vdp_action act, void **priv,
     const void *ptr, ssize_t len)
 {
 	struct ved_foo *foo;
@@ -623,6 +646,7 @@ ved_gzgz_bytes(struct req *req, enum vdp_action act, void **priv,
 	ssize_t dl;
 	ssize_t l;
 
+	(void)vdx;
 	CAST_OBJ_NOTNULL(foo, *priv, VED_FOO_MAGIC);
 	pp = ptr;
 	if (len > 0) {
@@ -643,8 +667,8 @@ ved_gzgz_bytes(struct req *req, enum vdp_action act, void **priv,
 		if (dl > 0) {
 			if (dl > len)
 				dl = len;
-			if (ved_bytes(req, foo->ecx, act, pp, dl))
-				return(-1);
+			if (ved_bytes(foo->ecx, act, pp, dl))
+				return (-1);
 			foo->ll += dl;
 			len -= dl;
 			pp += dl;
@@ -654,7 +678,7 @@ ved_gzgz_bytes(struct req *req, enum vdp_action act, void **priv,
 		/* Remove the "LAST" bit */
 		foo->dbits[0] = *pp;
 		foo->dbits[0] &= ~(1U << (foo->last & 7));
-		if (ved_bytes(req, foo->ecx, act, foo->dbits, 1))
+		if (ved_bytes(foo->ecx, act, foo->dbits, 1))
 			return (-1);
 		foo->ll++;
 		len--;
@@ -666,7 +690,7 @@ ved_gzgz_bytes(struct req *req, enum vdp_action act, void **priv,
 		if (dl > 0) {
 			if (dl > len)
 				dl = len;
-			if (ved_bytes(req, foo->ecx, act, pp, dl))
+			if (ved_bytes(foo->ecx, act, pp, dl))
 				return (-1);
 			foo->ll += dl;
 			len -= dl;
@@ -728,7 +752,7 @@ ved_gzgz_bytes(struct req *req, enum vdp_action act, void **priv,
 		default:
 			WRONG("compiler must be broken");
 		}
-		if (ved_bytes(req, foo->ecx, act, foo->dbits + 1, foo->lpad))
+		if (ved_bytes(foo->ecx, act, foo->dbits + 1, foo->lpad))
 			return (-1);
 	}
 	if (len > 0) {
@@ -753,12 +777,13 @@ ved_gzgz_bytes(struct req *req, enum vdp_action act, void **priv,
 }
 
 static int v_matchproto_(vdp_fini_f)
-ved_gzgz_fini(struct req *req, void **priv)
+ved_gzgz_fini(struct vdp_ctx *vdc, void **priv)
 {
 	uint32_t icrc;
 	uint32_t ilen;
 	struct ved_foo *foo;
 
+	(void)vdc;
 	CAST_OBJ_NOTNULL(foo, *priv, VED_FOO_MAGIC);
 	*priv = NULL;
 
@@ -768,7 +793,7 @@ ved_gzgz_fini(struct req *req, void **priv)
 	 *
 	 * Could rewrite use VDP_END
 	 */
-	(void)ved_bytes(req, foo->ecx, VDP_FLUSH, NULL, 0);
+	(void)ved_bytes(foo->ecx, VDP_FLUSH, NULL, 0);
 
 	icrc = vle32dec(foo->tailbuf);
 	ilen = vle32dec(foo->tailbuf + 4);
@@ -790,22 +815,22 @@ static const struct vdp ved_gzgz = {
  */
 
 static int v_matchproto_(vdp_fini_f)
-ved_vdp_fini(struct req *req, void **priv)
+ved_vdp_fini(struct vdp_ctx *vdc, void **priv)
 {
-	(void)req;
+	(void)vdc;
 	*priv = NULL;
 	return (0);
 }
 
 static int v_matchproto_(vdp_bytes_f)
-ved_vdp_bytes(struct req *req, enum vdp_action act, void **priv,
+ved_vdp_bytes(struct vdp_ctx *vdx, enum vdp_action act, void **priv,
     const void *ptr, ssize_t len)
 {
 	struct ecx *ecx;
 
-	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+	(void)vdx;
 	CAST_OBJ_NOTNULL(ecx, *priv, ECX_MAGIC);
-	return (ved_bytes(req, ecx, act, ptr, len));
+	return (ved_bytes(ecx, act, ptr, len));
 }
 
 static const struct vdp ved_ved = {
@@ -819,7 +844,8 @@ static const struct vdp ved_ved = {
 static void v_matchproto_(vtr_deliver_f)
 ved_deliver(struct req *req, struct boc *boc, int wantbody)
 {
-	int i;
+	int i = 0;
+	const char *p;
 	struct ecx *ecx;
 	struct ved_foo foo[1];
 
@@ -835,7 +861,11 @@ ved_deliver(struct req *req, struct boc *boc, int wantbody)
 	if (boc == NULL && ObjGetLen(req->wrk, req->objcore) == 0)
 		return;
 
-	i = ObjCheckFlag(req->wrk, req->objcore, OF_GZIPED);
+	if (http_GetHdr(req->resp, H_Content_Encoding, &p))
+		i = !strcasecmp(p, "gzip");
+	if (i)
+		i = ObjCheckFlag(req->wrk, req->objcore, OF_GZIPED);
+
 	if (ecx->isgzip && i && !(req->res_mode & RES_ESI)) {
 		/* A gzip'ed include which is not ESI processed */
 
@@ -851,18 +881,19 @@ ved_deliver(struct req *req, struct boc *boc, int wantbody)
 
 		INIT_OBJ(foo, VED_FOO_MAGIC);
 		foo->ecx = ecx;
-		i = VDP_Push(req, &ved_gzgz, foo);
+		foo->objcore = req->objcore;
+		i = VDP_Push(req->vdc, req->ws, &ved_gzgz, foo);
 
 	} else if (ecx->isgzip && !i) {
 		/* Non-Gzip'ed include in gzip'ed parent */
-		i = VDP_Push(req, &ved_pretend_gz, ecx);
+		i = VDP_Push(req->vdc, req->ws, &ved_pretend_gz, ecx);
 	} else {
 		/* Anything else goes straight through */
-		i = VDP_Push(req, &ved_ved, ecx);
+		i = VDP_Push(req->vdc, req->ws, &ved_ved, ecx);
 	}
 
 	if (i == 0) {
-		i = VDP_DeliverObj(req);
+		i = VDP_DeliverObj(req->vdc, req->objcore);
 	} else {
 		VSLb(req->vsl, SLT_Error, "Failure to push ESI processors");
 		req->doclose = SC_OVERLOAD;
@@ -871,5 +902,5 @@ ved_deliver(struct req *req, struct boc *boc, int wantbody)
 	if (i && req->doclose == SC_NULL)
 		req->doclose = SC_REM_CLOSE;
 
-	VDP_close(req);
+	req->acct.resp_bodybytes += VDP_Close(req->vdc);
 }
