@@ -100,7 +100,7 @@ vfp_vep_callback(struct vfp_ctx *vc, void *priv, ssize_t l, enum vgz_flag flg)
 		VGZ_UpdateObj(vc, vef->vgz, VUA_UPDATE);
 		if (dl > 0) {
 			vef->tot += dl;
-			VFP_Extend(vc, dl);
+			VFP_Extend(vc, dl, VFP_OK);
 		}
 	} while (i != VGZ_ERROR &&
 	    (!VGZ_IbufEmpty(vef->vgz) || VGZ_ObufFull(vef->vgz)));
@@ -120,6 +120,13 @@ vfp_esi_end(struct vfp_ctx *vc, struct vef_priv *vef,
 	CHECK_OBJ_NOTNULL(vc, VFP_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(vef, VEF_MAGIC);
 
+	if (retval == VFP_ERROR) {
+		if (vef->error == 0)
+			vef->error = errno ? errno : EINVAL;
+	} else {
+		assert(retval == VFP_END);
+	}
+
 	vsb = VEP_Finish(vef->vep);
 
 	if (vsb != NULL) {
@@ -137,7 +144,8 @@ vfp_esi_end(struct vfp_ctx *vc, struct vef_priv *vef,
 	}
 
 	if (vef->vgz != NULL) {
-		VGZ_UpdateObj(vc, vef->vgz, VUA_END_GZIP);
+		if (retval == VFP_END)
+			VGZ_UpdateObj(vc, vef->vgz, VUA_END_GZIP);
 		if (VGZ_Destroy(&vef->vgz) != VGZ_END)
 			retval = VFP_Error(vc,
 			    "ESI+Gzip Failed at the very end");
@@ -292,8 +300,13 @@ vfp_esi_fini(struct vfp_ctx *vc, struct vfp_entry *vfe)
 	CHECK_OBJ_NOTNULL(vc, VFP_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(vfe, VFP_ENTRY_MAGIC);
 
-	if (vfe->priv1 != NULL)
-		(void)vfp_esi_end(vc, vfe->priv1, VFP_ERROR);
+	if (vfe->priv1 == NULL)
+		return;
+
+	if (vc->oc->stobj->stevedore == NULL)
+		errno = ENOMEM;
+
+	(void)vfp_esi_end(vc, vfe->priv1, VFP_ERROR);
 	vfe->priv1 = NULL;
 }
 
