@@ -39,6 +39,31 @@
 #include "vcl.h"
 #include "vtim.h"
 
+#define FETCH_STEPS \
+	FETCH_STEP(mkbereq,           MKBEREQ) \
+	FETCH_STEP(retry,             RETRY) \
+	FETCH_STEP(startfetch,        STARTFETCH) \
+	FETCH_STEP(condfetch,         CONDFETCH) \
+	FETCH_STEP(fetch,             FETCH) \
+	FETCH_STEP(fetchbody,         FETCHBODY) \
+	FETCH_STEP(fetchend,          FETCHEND) \
+	FETCH_STEP(error,             ERROR) \
+	FETCH_STEP(fail,              FAIL) \
+	FETCH_STEP(done,              DONE)
+
+typedef const struct fetch_step *vbf_state_f(struct worker *, struct busyobj *);
+
+struct fetch_step {
+	const char	*name;
+	vbf_state_f	*func;
+};
+
+#define FETCH_STEP(l, U) \
+    static vbf_state_f vbf_stp_##l; \
+    static const struct fetch_step F_STP_##U[1] = {{ .name = "Fetch Step" #l, .func = vbf_stp_##l, }};
+FETCH_STEPS
+#undef FETCH_STEP
+
 /*--------------------------------------------------------------------
  * Allocate an object, with fall-back to Transient.
  * XXX: This somewhat overlaps the stuff in stevedore.c
@@ -101,16 +126,20 @@ vbf_cleanup(struct busyobj *bo)
 	vfc = bo->vfc;
 	CHECK_OBJ_NOTNULL(vfc, VFP_CTX_MAGIC);
 
-	VFP_Close(vfc);
-	bo->filter_list = NULL;
+	bo->acct.beresp_bodybytes += VFP_Close(vfc);
+	bo->vfp_filter_list = NULL;
 
 	if (bo->director_state != DIR_S_NULL)
 		VDI_Finish(bo);
 }
 
 void
-Bereq_Rollback(struct busyobj *bo)
+Bereq_Rollback(VRT_CTX)
 {
+	struct busyobj *bo;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	bo = ctx->bo;
 	CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
 
 	if (bo->htc != NULL &&
@@ -119,10 +148,10 @@ Bereq_Rollback(struct busyobj *bo)
 		bo->htc->doclose = SC_RESP_CLOSE;
 
 	vbf_cleanup(bo);
-	VCL_TaskLeave(bo->privs);
+	VCL_TaskLeave(ctx, bo->privs);
 	VCL_TaskEnter(bo->privs);
 	HTTP_Clone(bo->bereq, bo->bereq0);
-	bo->filter_list = NULL;
+	bo->vfp_filter_list = NULL;
 	bo->err_reason = NULL;
 	WS_Rollback(bo->bereq->ws, bo->ws_bo);
 	WS_Rollback(bo->ws, bo->ws_bo);
@@ -213,7 +242,7 @@ vbf_beresp2obj(struct busyobj *bo)
  * Copy req->bereq and release req if no body
  */
 
-static enum fetch_step
+static const struct fetch_step * v_matchproto_(vbf_state_f)
 vbf_stp_mkbereq(struct worker *wrk, struct busyobj *bo)
 {
 	const char *q;
@@ -278,7 +307,7 @@ vbf_stp_mkbereq(struct worker *wrk, struct busyobj *bo)
  * Prepare the busyobj and fetch processors
  */
 
-static enum fetch_step
+static const struct fetch_step * v_matchproto_(vbf_state_f)
 vbf_stp_retry(struct worker *wrk, struct busyobj *bo)
 {
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
@@ -302,6 +331,8 @@ vbf_stp_retry(struct worker *wrk, struct busyobj *bo)
 	bo->do_esi = 0;
 	bo->do_stream = 1;
 	bo->was_304 = 0;
+	bo->err_code = 0;
+	bo->err_reason = NULL;
 
 	// XXX: BereqEnd + BereqAcct ?
 	VSL_ChgId(bo->vsl, "bereq", "retry", VXID_Get(wrk, VSL_BACKENDMARKER));
@@ -352,7 +383,7 @@ vbf_304_logic(struct busyobj *bo)
  * Setup bereq from bereq0, run vcl_backend_fetch
  */
 
-static enum fetch_step
+static const struct fetch_step * v_matchproto_(vbf_state_f)
 vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 {
 	int i;
@@ -395,6 +426,7 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 	if (wrk->handling == VCL_RET_ERROR)
 		return (F_STP_ERROR);
 
+	VSLb_ts_busyobj(bo, "Fetch", W_TIM_real(wrk));
 	i = VDI_GetHdr(bo);
 
 	now = W_TIM_real(wrk);
@@ -451,6 +483,10 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 	if (http_IsStatus(bo->beresp, 304) && vbf_304_logic(bo) < 0)
 		return (F_STP_ERROR);
 
+	if (bo->htc->doclose == SC_NULL &&
+	    http_GetHdrField(bo->bereq, H_Connection, "close", NULL))
+		bo->htc->doclose = SC_REQ_CLOSE;
+
 	VCL_backend_response_method(bo->vcl, wrk, NULL, bo, NULL);
 
 	if (bo->htc != NULL && bo->htc->doclose == SC_NULL &&
@@ -488,6 +524,7 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 		return (F_STP_ERROR);
 	}
 
+	VSLb_ts_busyobj(bo, "Process", W_TIM_real(wrk));
 	assert(oc->boc->state <= BOS_REQ_DONE);
 	if (oc->boc->state != BOS_REQ_DONE) {
 		bo->req = NULL;
@@ -512,7 +549,7 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 /*--------------------------------------------------------------------
  */
 
-static enum fetch_step
+static const struct fetch_step * v_matchproto_(vbf_state_f)
 vbf_stp_fetchbody(struct worker *wrk, struct busyobj *bo)
 {
 	ssize_t l;
@@ -557,9 +594,8 @@ vbf_stp_fetchbody(struct worker *wrk, struct busyobj *bo)
 
 		AZ(vfc->failed);
 		vfps = VFP_Suck(vfc, ptr, &l);
-		if (l > 0 && vfps != VFP_ERROR) {
-			bo->acct.beresp_bodybytes += l;
-			VFP_Extend(vfc, l);
+		if (l >= 0 && vfps != VFP_ERROR) {
+			VFP_Extend(vfc, l, vfps);
 			if (est >= l)
 				est -= l;
 			else
@@ -581,11 +617,10 @@ vbf_stp_fetchbody(struct worker *wrk, struct busyobj *bo)
 		}
 	}
 
-	ObjTrimStore(wrk, oc);
 	return (F_STP_FETCHEND);
 }
 
-static enum fetch_step
+static const struct fetch_step * v_matchproto_(vbf_state_f)
 vbf_stp_fetch(struct worker *wrk, struct busyobj *bo)
 {
 	struct objcore *oc;
@@ -608,13 +643,13 @@ vbf_stp_fetch(struct worker *wrk, struct busyobj *bo)
 		http_Unset(bo->beresp, H_Content_Encoding);
 		bo->do_gzip = bo->do_gunzip = 0;
 		bo->do_stream = 0;
-		bo->filter_list = "";
-	} else if (bo->filter_list == NULL) {
-		bo->filter_list = VBF_Get_Filter_List(bo);
+		bo->vfp_filter_list = "";
+	} else if (bo->vfp_filter_list == NULL) {
+		bo->vfp_filter_list = VBF_Get_Filter_List(bo);
 	}
 
-	if (bo->filter_list == NULL ||
-	    VCL_StackVFP(bo->vfc, bo->vcl, bo->filter_list)) {
+	if (bo->vfp_filter_list == NULL ||
+	    VCL_StackVFP(bo->vfc, bo->vcl, bo->vfp_filter_list)) {
 		(bo)->htc->doclose = SC_OVERLOAD;
 		vbf_cleanup(bo);
 		return (F_STP_ERROR);
@@ -671,7 +706,7 @@ vbf_stp_fetch(struct worker *wrk, struct busyobj *bo)
 	return (F_STP_FETCHEND);
 }
 
-static enum fetch_step
+static const struct fetch_step * v_matchproto_(vbf_state_f)
 vbf_stp_fetchend(struct worker *wrk, struct busyobj *bo)
 {
 
@@ -726,14 +761,14 @@ vbf_objiterator(void *priv, unsigned flush, const void *ptr, ssize_t len)
 		if (len < l)
 			l = len;
 		memcpy(pd, ps, l);
-		VFP_Extend(bo->vfc, l);
+		VFP_Extend(bo->vfc, l, l == len ? VFP_END : VFP_OK);
 		ps += l;
 		len -= l;
 	}
 	return (0);
 }
 
-static enum fetch_step
+static const struct fetch_step * v_matchproto_(vbf_state_f)
 vbf_stp_condfetch(struct worker *wrk, struct busyobj *bo)
 {
 	struct boc *stale_boc;
@@ -822,7 +857,7 @@ vbf_stp_condfetch(struct worker *wrk, struct busyobj *bo)
  * default true)
  */
 
-static enum fetch_step
+static const struct fetch_step * v_matchproto_(vbf_state_f)
 vbf_stp_error(struct worker *wrk, struct busyobj *bo)
 {
 	ssize_t l, ll, o;
@@ -926,7 +961,7 @@ vbf_stp_error(struct worker *wrk, struct busyobj *bo)
 		if (l > ll)
 			l = ll;
 		memcpy(ptr, VSB_data(synth_body) + o, l);
-		VFP_Extend(bo->vfc, l);
+		VFP_Extend(bo->vfc, l, l == ll ? VFP_END : VFP_OK);
 		ll -= l;
 		o += l;
 	}
@@ -943,8 +978,8 @@ vbf_stp_error(struct worker *wrk, struct busyobj *bo)
 /*--------------------------------------------------------------------
  */
 
-static enum fetch_step
-vbf_stp_fail(struct worker *wrk, const struct busyobj *bo)
+static const struct fetch_step * v_matchproto_(vbf_state_f)
+vbf_stp_fail(struct worker *wrk, struct busyobj *bo)
 {
 	struct objcore *oc;
 
@@ -964,9 +999,12 @@ vbf_stp_fail(struct worker *wrk, const struct busyobj *bo)
 /*--------------------------------------------------------------------
  */
 
-static enum fetch_step
-vbf_stp_done(void)
+static const struct fetch_step * v_matchproto_(vbf_state_f)
+vbf_stp_done(struct worker *wrk, struct busyobj *bo)
 {
+
+	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
+	CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
 	WRONG("Just plain wrong");
 	NEEDLESS(return (F_STP_DONE));
 }
@@ -974,9 +1012,10 @@ vbf_stp_done(void)
 static void v_matchproto_(task_func_t)
 vbf_fetch_thread(struct worker *wrk, void *priv)
 {
+	struct vrt_ctx ctx[1];
 	struct busyobj *bo;
 	struct objcore *oc;
-	enum fetch_step stp;
+	const struct fetch_step *stp;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CAST_OBJ_NOTNULL(bo, priv, BUSYOBJ_MAGIC);
@@ -1010,25 +1049,20 @@ vbf_fetch_thread(struct worker *wrk, void *priv)
 			AN(bo->req);
 		else
 			AZ(bo->req);
-		switch (stp) {
-#define FETCH_STEP(l, U, arg)						\
-		case F_STP_##U:						\
-			stp = vbf_stp_##l arg;				\
-			break;
-#include "tbl/steps.h"
-		default:
-			WRONG("Illegal fetch_step");
-		}
+		AN(stp);
+		AN(stp->name);
+		AN(stp->func);
+		stp = stp->func(wrk, bo);
 	}
 
 	assert(bo->director_state == DIR_S_NULL);
 
-	VCL_TaskLeave(bo->privs);
+	INIT_OBJ(ctx, VRT_CTX_MAGIC);
+	VCL_Bo2Ctx(ctx, bo);
+	VCL_TaskLeave(ctx, bo->privs);
 	http_Teardown(bo->bereq);
 	http_Teardown(bo->beresp);
-	// XXX after 6.4 release:
-	// bereq_body should have 0 or 1 references remaining,
-	// see VRB_Free() for the other end
+	// can not make assumptions about the number of references here #3434
 	if (bo->bereq_body != NULL)
 		(void) HSH_DerefObjCore(bo->wrk, &bo->bereq_body, 0);
 
@@ -1089,6 +1123,9 @@ VBF_Fetch(struct worker *wrk, struct req *req, struct objcore *oc,
 		WRONG("Wrong fetch mode");
 	}
 
+	bo->is_hitpass = req->is_hitpass;
+	bo->is_hitmiss = req->is_hitmiss;
+
 	VSLb(bo->vsl, SLT_Begin, "bereq %u %s", VXID(req->vsl->wid), how);
 	VSLb(bo->vsl, SLT_VCL_use, "%s", VCL_Name(bo->vcl));
 	VSLb(req->vsl, SLT_Link, "bereq %u %s", VXID(bo->vsl->wid), how);
