@@ -288,17 +288,20 @@ VGZ_Gzip(struct vgz *vg, const void **pptr, ssize_t *plen, enum vgz_flag flags)
  */
 
 static int v_matchproto_(vdp_init_f)
-vdp_gunzip_init(struct req *req, void **priv)
+vdp_gunzip_init(struct vdp_ctx *vdp, void **priv, struct objcore *oc)
 {
 	struct vgz *vg;
 	struct boc *boc;
+	struct req *req;
 	enum boc_state_e bos;
 	const char *p;
 	ssize_t dl;
 	uint64_t u;
 
+	CHECK_OBJ_NOTNULL(vdp, VDP_CTX_MAGIC);
+	CHECK_OBJ_ORNULL(oc, OBJCORE_MAGIC);
+	req = vdp->req;
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
-	CHECK_OBJ_NOTNULL(req->objcore, OBJCORE_MAGIC);
 
 	vg = VGZ_NewGunzip(req->vsl, "U D -");
 	AN(vg);
@@ -314,38 +317,33 @@ vdp_gunzip_init(struct req *req, void **priv)
 
 	req->resp_len = -1;
 
-	boc = HSH_RefBoc(req->objcore);
+	if (oc == NULL)
+		return (0);
+
+	boc = HSH_RefBoc(oc);
 	if (boc != NULL) {
 		CHECK_OBJ(boc, BOC_MAGIC);
 		bos = boc->state;
-		HSH_DerefBoc(req->wrk, req->objcore);
-	} else
-		bos = BOS_FINISHED;
+		HSH_DerefBoc(req->wrk, oc);
+		if (bos < BOS_FINISHED)
+			return (0); /* OA_GZIPBITS is not stable yet */
+	}
 
-	/* OA_GZIPBITS is not stable yet */
-	if (bos < BOS_FINISHED)
-		return (0);
-
-	p = ObjGetAttr(req->wrk, req->objcore, OA_GZIPBITS, &dl);
+	p = ObjGetAttr(req->wrk, oc, OA_GZIPBITS, &dl);
 	if (p != NULL && dl == 32) {
 		u = vbe64dec(p + 24);
-		/*
-		 * If the size is non-zero AND we are the top VDP
-		 * (ie: no ESI), we know what size the output will be.
-		 */
-		if (u != 0 &&
-		    VTAILQ_FIRST(&req->vdc->vdp)->vdp == &VDP_gunzip)
+		if (u != 0)
 			req->resp_len = u;
 	}
 	return (0);
 }
 
 static int v_matchproto_(vdp_fini_f)
-vdp_gunzip_fini(struct req *req, void **priv)
+vdp_gunzip_fini(struct vdp_ctx *vdc, void **priv)
 {
 	struct vgz *vg;
 
-	(void)req;
+	(void)vdc;
 	CAST_OBJ_NOTNULL(vg, *priv, VGZ_MAGIC);
 	AN(vg->m_buf);
 	(void)VGZ_Destroy(&vg);
@@ -354,7 +352,7 @@ vdp_gunzip_fini(struct req *req, void **priv)
 }
 
 static int v_matchproto_(vdp_bytes_f)
-vdp_gunzip_bytes(struct req *req, enum vdp_action act, void **priv,
+vdp_gunzip_bytes(struct vdp_ctx *vdx, enum vdp_action act, void **priv,
     const void *ptr, ssize_t len)
 {
 	enum vgzret_e vr;
@@ -363,8 +361,8 @@ vdp_gunzip_bytes(struct req *req, enum vdp_action act, void **priv,
 	struct worker *wrk;
 	struct vgz *vg;
 
-	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
-	wrk = req->wrk;
+	CHECK_OBJ_NOTNULL(vdx, VDP_CTX_MAGIC);
+	wrk = vdx->wrk;
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	(void)act;
 
@@ -386,8 +384,9 @@ vdp_gunzip_bytes(struct req *req, enum vdp_action act, void **priv,
 		if (vr < VGZ_OK)
 			return (-1);
 		if (vg->m_len == vg->m_sz || vr != VGZ_OK) {
-			if (VDP_bytes(req, VDP_FLUSH, vg->m_buf, vg->m_len))
-				return (req->vdc->retval);
+			if (VDP_bytes(vdx, vr == VGZ_END ? VDP_END : VDP_FLUSH,
+			    vg->m_buf, vg->m_len))
+				return (vdx->retval);
 			vg->m_len = 0;
 			VGZ_Obuf(vg, vg->m_buf, vg->m_sz);
 		}
@@ -561,7 +560,7 @@ vfp_gunzip_pull(struct vfp_ctx *vc, struct vfp_entry *vfe, void *p,
 		if (!VGZ_IbufEmpty(vg) || vp == VFP_END) {
 			vr = VGZ_Gunzip(vg, &dp, &dl);
 			if (vr == VGZ_END && !VGZ_IbufEmpty(vg))
-				return(VFP_Error(vc, "Junk after gzip data"));
+				return (VFP_Error(vc, "Junk after gzip data"));
 			if (vr < VGZ_OK)
 				return (VFP_Error(vc,
 				    "Invalid Gzip data: %s", vgz_msg(vg)));
@@ -574,7 +573,7 @@ vfp_gunzip_pull(struct vfp_ctx *vc, struct vfp_entry *vfe, void *p,
 		AN(VGZ_IbufEmpty(vg));
 	} while (vp == VFP_OK);
 	if (vr != VGZ_END)
-		return(VFP_Error(vc, "Gunzip error at the very end"));
+		return (VFP_Error(vc, "Gunzip error at the very end"));
 	return (vp);
 }
 
@@ -665,7 +664,7 @@ vfp_testgunzip_pull(struct vfp_ctx *vc, struct vfp_entry *vfe, void *p,
 			VGZ_Obuf(vg, vg->m_buf, vg->m_sz);
 			vr = VGZ_Gunzip(vg, &dp, &dl);
 			if (vr == VGZ_END && !VGZ_IbufEmpty(vg))
-				return(VFP_Error(vc, "Junk after gzip data"));
+				return (VFP_Error(vc, "Junk after gzip data"));
 			if (vr < VGZ_OK)
 				return (VFP_Error(vc,
 				    "Invalid Gzip data: %s", vgz_msg(vg)));
