@@ -154,7 +154,11 @@ Lck__Unlock(struct lock *lck, const char *p, int l)
 	 * implementations of pthread.
 	 * #endif
 	 */
+#ifdef PTHREAD_NULL
+	ilck->owner = PTHREAD_NULL;
+#else
 	memset(&ilck->owner, 0, sizeof ilck->owner);
+#endif
 	AZ(pthread_mutex_unlock(&ilck->mtx));
 	if (DO_DEBUG(DBG_WITNESS))
 		Lck_Witness_Unlock(ilck);
@@ -218,10 +222,10 @@ Lck_CondWait(pthread_cond_t *cond, struct lock *lck, vtim_real when)
 		assert(when > 1e9);
 		ts.tv_nsec = (long)(modf(when, &t) * 1e9);
 		ts.tv_sec = (long)t;
-		assert(ts.tv_nsec >= 0 && ts.tv_nsec < 999999999);
+		assert(ts.tv_nsec >= 0 && ts.tv_nsec <= 999999999);
 		errno = pthread_cond_timedwait(cond, &ilck->mtx, &ts);
 #if defined (__APPLE__)
-		if (errno == EINVAL && when > VTIM_real()) {
+		if (errno == EINVAL && when <= VTIM_real()) {
 			/*
 			 * Most kernels treat this as honest error,
 			 * recognizing that a thread has no way to
