@@ -52,6 +52,8 @@
 #include "vcli_serve.h"
 #include "vtim.h"
 #include "vcs.h"
+#include "vtcp.h"
+#include "vsa.h"
 
 /*
  * The panic string is constructed in a VSB, then copied to the
@@ -108,23 +110,46 @@ static const void *already_list[N_ALREADY];
 static int already_idx;
 
 int
-PAN_already(struct vsb *vsb, const void *ptr)
+PAN_dump_struct2(struct vsb *vsb, int block, const void *ptr,
+    const char *smagic, unsigned magic, const char *fmt, ...)
 {
+	va_list ap;
+	const unsigned *uptr;
 	int i;
 
+	AN(vsb);
+	va_start(ap, fmt);
+	VSB_vprintf(vsb, fmt, ap);
+	va_end(ap);
 	if (ptr == NULL) {
-		VSB_cat(vsb, "},\n");
-		return (1);
+		VSB_cat(vsb, " = NULL\n");
+		return (-1);
 	}
+	VSB_printf(vsb, " = %p {", ptr);
+	if (block)
+		VSB_putc(vsb, '\n');
 	for (i = 0; i < already_idx; i++) {
 		if (already_list[i] == ptr) {
-			VSB_cat(vsb, "  [Already dumped, see above]\n");
+			VSB_cat(vsb, "  [Already dumped, see above]");
+			if (block)
+				VSB_putc(vsb, '\n');
 			VSB_cat(vsb, "},\n");
-			return (1);
+			return (-2);
 		}
 	}
 	if (already_idx < N_ALREADY)
 		already_list[already_idx++] = ptr;
+	uptr = ptr;
+	if (*uptr != magic) {
+		VSB_printf(vsb, "  .magic = 0x%08x", *uptr);
+		VSB_printf(vsb, " EXPECTED: %s=0x%08x", smagic, magic);
+		if (block)
+			VSB_putc(vsb, '\n');
+		VSB_cat(vsb, "}\n");
+		return (-3);
+	}
+	if (block)
+		VSB_indent(vsb, 2);
 	return (0);
 }
 
@@ -134,15 +159,12 @@ static void
 pan_htc(struct vsb *vsb, const struct http_conn *htc)
 {
 
-	VSB_printf(vsb, "http_conn = %p {\n", htc);
-	if (PAN_already(vsb, htc))
+	if (PAN_dump_struct(vsb, htc, HTTP_CONN_MAGIC, "http_conn"))
 		return;
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, htc, HTTP_CONN_MAGIC);
 	if (htc->rfd != NULL)
 		VSB_printf(vsb, "fd = %d (@%p),\n", *htc->rfd, htc->rfd);
 	VSB_printf(vsb, "doclose = %s,\n", sess_close_2str(htc->doclose, 0));
-	WS_Panic(htc->ws, vsb);
+	WS_Panic(vsb, htc->ws);
 	VSB_printf(vsb, "{rxbuf_b, rxbuf_e} = {%p, %p},\n",
 	    htc->rxbuf_b, htc->rxbuf_e);
 	VSB_printf(vsb, "{pipeline_b, pipeline_e} = {%p, %p},\n",
@@ -166,12 +188,9 @@ pan_http(struct vsb *vsb, const char *id, const struct http *h)
 {
 	int i;
 
-	VSB_printf(vsb, "http[%s] = %p {\n", id, h);
-	if (PAN_already(vsb, h))
+	if (PAN_dump_struct(vsb, h, HTTP_MAGIC, "http[%s]", id))
 		return;
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, h, HTTP_MAGIC);
-	WS_Panic(h->ws, vsb);
+	WS_Panic(vsb, h->ws);
 	VSB_cat(vsb, "hdrs {\n");
 	VSB_indent(vsb, 2);
 	for (i = 0; i < h->nhd; ++i) {
@@ -187,14 +206,12 @@ pan_http(struct vsb *vsb, const char *id, const struct http *h)
 }
 
 /*--------------------------------------------------------------------*/
+
 static void
 pan_boc(struct vsb *vsb, const struct boc *boc)
 {
-	VSB_printf(vsb, "boc = %p {\n", boc);
-	if (PAN_already(vsb, boc))
+	if (PAN_dump_struct(vsb, boc, BOC_MAGIC, "boc"))
 		return;
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, boc, BOC_MAGIC);
 	VSB_printf(vsb, "refcnt = %u,\n", boc->refcount);
 	VSB_printf(vsb, "state = %s,\n", boc_state_2str(boc->state));
 	VSB_printf(vsb, "vary = %p,\n", boc->vary);
@@ -210,11 +227,8 @@ pan_objcore(struct vsb *vsb, const char *typ, const struct objcore *oc)
 {
 	const char *p;
 
-	VSB_printf(vsb, "objcore[%s] = %p {\n", typ, oc);
-	if (PAN_already(vsb, oc))
+	if (PAN_dump_struct(vsb, oc, OBJCORE_MAGIC, "objcore[%s]", typ))
 		return;
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, oc, OBJCORE_MAGIC);
 	VSB_printf(vsb, "refcnt = %d,\n", oc->refcnt);
 	VSB_cat(vsb, "flags = {");
 
@@ -265,12 +279,9 @@ pan_wrk(struct vsb *vsb, const struct worker *wrk)
 	unsigned m, u;
 	const char *p;
 
-	VSB_printf(vsb, "worker = %p {\n", wrk);
-	if (PAN_already(vsb, wrk))
+	if (PAN_dump_struct(vsb, wrk, WORKER_MAGIC, "worker"))
 		return;
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, wrk, WORKER_MAGIC);
-	WS_Panic(wrk->aws, vsb);
+	WS_Panic(vsb, wrk->aws);
 
 	m = wrk->cur_method;
 	VSB_cat(vsb, "VCL::method = ");
@@ -312,9 +323,8 @@ pan_vfp(struct vsb *vsb, const struct vfp_ctx *vfc)
 {
 	struct vfp_entry *vfe;
 
-	VSB_printf(vsb, "vfc = %p {\n", vfc);
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, vfc, VFP_CTX_MAGIC);
+	if (PAN_dump_struct(vsb, vfc, VFP_CTX_MAGIC, "vfc"))
+		return;
 	VSB_printf(vsb, "failed = %d,\n", vfc->failed);
 	VSB_printf(vsb, "req = %p,\n", vfc->req);
 	VSB_printf(vsb, "resp = %p,\n", vfc->resp);
@@ -342,41 +352,13 @@ pan_vfp(struct vsb *vsb, const struct vfp_ctx *vfc)
 	VSB_cat(vsb, "},\n");
 }
 
-static void
-pan_vdp(struct vsb *vsb, const struct vdp_ctx *vdc)
-{
-	struct vdp_entry *vde;
-
-	VSB_printf(vsb, "vdc = %p {\n", vdc);
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, vdc, VDP_CTX_MAGIC);
-	VSB_printf(vsb, "nxt = %p,\n", vdc->nxt);
-	VSB_printf(vsb, "retval = %d,\n", vdc->retval);
-
-	if (!VTAILQ_EMPTY(&vdc->vdp)) {
-		VSB_cat(vsb, "filters = {\n");
-		VSB_indent(vsb, 2);
-		VTAILQ_FOREACH(vde, &vdc->vdp, list)
-			VSB_printf(vsb, "%s = %p { priv = %p }\n",
-			    vde->vdp->name, vde, vde->priv);
-		VSB_indent(vsb, -2);
-		VSB_cat(vsb, "},\n");
-	}
-
-	VSB_indent(vsb, -2);
-	VSB_cat(vsb, "},\n");
-}
-
 static void
 pan_busyobj(struct vsb *vsb, const struct busyobj *bo)
 {
 	const char *p;
 
-	VSB_printf(vsb, "busyobj = %p {\n", bo);
-	if (PAN_already(vsb, bo))
+	if (PAN_dump_struct(vsb, bo, BUSYOBJ_MAGIC, "busyobj"))
 		return;
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, bo, BUSYOBJ_MAGIC);
 	VSB_printf(vsb, "end = %p,\n", bo->end);
 	VSB_printf(vsb, "retries = %u,\n", bo->retries);
 
@@ -389,10 +371,10 @@ pan_busyobj(struct vsb *vsb, const struct busyobj *bo)
 
 	if (bo->vfc != NULL)
 		pan_vfp(vsb, bo->vfc);
-	if (bo->filter_list != NULL)
-		VSB_printf(vsb, "filter_list = \"%s\",\n", bo->filter_list);
+	if (bo->vfp_filter_list != NULL)
+		VSB_printf(vsb, "filter_list = \"%s\",\n", bo->vfp_filter_list);
 
-	WS_Panic(bo->ws, vsb);
+	WS_Panic(vsb, bo->ws);
 	VSB_printf(vsb, "ws_bo = %p,\n", (void *)bo->ws_bo);
 
 	// bereq0 left out
@@ -413,7 +395,7 @@ pan_busyobj(struct vsb *vsb, const struct busyobj *bo)
 	VSB_cat(vsb, "flags = {");
 	p = "";
 /*lint -save -esym(438,p) -e539 */
-#define BO_FLAG(l, r, w, f, d)					\
+#define BO_FLAG(l, r, rr, rw, f, d)				\
 	if (bo->l) { VSB_printf(vsb, "%s" #l, p); p = ", "; }
 #include "tbl/bo_flags.h"
 /*lint -restore */
@@ -436,10 +418,8 @@ pan_busyobj(struct vsb *vsb, const struct busyobj *bo)
 static void
 pan_top(struct vsb *vsb, const struct reqtop *top)
 {
-	VSB_printf(vsb, "top = %p {\n", top);
-	if (PAN_already(vsb, top))
+	if (PAN_dump_struct(vsb, top, REQTOP_MAGIC, "top"))
 		return;
-	VSB_indent(vsb, 2);
 	pan_req(vsb, top->topreq);
 	pan_privs(vsb, top->privs);
 	VCL_Panic(vsb, "vcl0", top->vcl0);
@@ -452,14 +432,10 @@ pan_top(struct vsb *vsb, const struct reqtop *top)
 static void
 pan_req(struct vsb *vsb, const struct req *req)
 {
-	const char *stp;
 	const struct transport *xp;
 
-	VSB_printf(vsb, "req = %p {\n", req);
-	if (PAN_already(vsb, req))
+	if (PAN_dump_struct(vsb, req, REQ_MAGIC, "req"))
 		return;
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, req, REQ_MAGIC);
 	xp = req->transport;
 	VSB_printf(vsb, "vxid = %u, transport = %s", VXID(req->vsl->wid),
 	    xp == NULL ? "NULL" : xp->name);
@@ -472,15 +448,10 @@ pan_req(struct vsb *vsb, const struct req *req)
 		VSB_cat(vsb, "}");
 	}
 	VSB_cat(vsb, "\n");
-	switch (req->req_step) {
-#define REQ_STEP(l, u, arg) case R_STP_##u: stp = "R_STP_" #u; break;
-#include "tbl/steps.h"
-		default: stp = NULL;
-	}
-	if (stp != NULL)
-		VSB_printf(vsb, "step = %s,\n", stp);
+	if (req->req_step == NULL)
+		VSB_cat(vsb, "step = R_STP_TRANSPORT\n");
 	else
-		VSB_printf(vsb, "step = 0x%x,\n", req->req_step);
+		VSB_printf(vsb, "step = %s\n", req->req_step->name);
 
 	VSB_printf(vsb, "req_body = %s,\n",
 	    req->req_body_status ? req->req_body_status->name : "NULL");
@@ -499,14 +470,14 @@ pan_req(struct vsb *vsb, const struct req *req)
 	if (req->wrk != NULL)
 		pan_wrk(vsb, req->wrk);
 
-	WS_Panic(req->ws, vsb);
+	WS_Panic(vsb, req->ws);
 	if (VALID_OBJ(req->htc, HTTP_CONN_MAGIC))
 		pan_htc(vsb, req->htc);
 	pan_http(vsb, "req", req->http);
 	if (req->resp != NULL && req->resp->ws != NULL)
 		pan_http(vsb, "resp", req->resp);
 	if (req->vdc != NULL)
-		pan_vdp(vsb, req->vdc);
+		VDP_Panic(vsb, req->vdc);
 
 	VCL_Panic(vsb, "vcl", req->vcl);
 
@@ -533,6 +504,18 @@ pan_req(struct vsb *vsb, const struct req *req)
 
 /*--------------------------------------------------------------------*/
 
+#define pan_addr(vsb, sp, field) do {					\
+		struct suckaddr *sa;					\
+		char h[VTCP_ADDRBUFSIZE];				\
+		char p[VTCP_PORTBUFSIZE];				\
+									\
+		(void) SES_Get_##field##_addr((sp), &sa);		\
+		if (! VSA_Sane(sa))					\
+			break;						\
+		VTCP_name(sa, h, sizeof h, p, sizeof p);		\
+		VSB_printf((vsb), "%s.ip = %s:%s,\n", #field, h, p);	\
+	} while (0)
+
 static void
 pan_sess(struct vsb *vsb, const struct sess *sp)
 {
@@ -540,17 +523,21 @@ pan_sess(struct vsb *vsb, const struct sess *sp)
 	const char *cp;
 	const struct transport *xp;
 
-	VSB_printf(vsb, "sp = %p {\n", sp);
-	if (PAN_already(vsb, sp))
+	if (PAN_dump_struct(vsb, sp, SESS_MAGIC, "sess"))
 		return;
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, sp, SESS_MAGIC);
-	xp = XPORT_ByNumber(sp->sattr[SA_TRANSPORT]);
 	VSB_printf(vsb, "fd = %d, vxid = %u,\n",
 	    sp->fd, VXID(sp->vxid));
 	VSB_printf(vsb, "t_open = %f,\n", sp->t_open);
 	VSB_printf(vsb, "t_idle = %f,\n", sp->t_idle);
-	WS_Panic(sp->ws, vsb);
+
+	if (! VALID_OBJ(sp, SESS_MAGIC)) {
+		VSB_indent(vsb, -2);
+		VSB_cat(vsb, "},\n");
+		return;
+	}
+
+	WS_Panic(vsb, sp->ws);
+	xp = XPORT_ByNumber(sp->sattr[SA_TRANSPORT]);
 	VSB_printf(vsb, "transport = %s",
 	    xp == NULL ? "<none>" : xp->name);
 	if (xp != NULL && xp->sess_panic != NULL) {
@@ -561,6 +548,8 @@ pan_sess(struct vsb *vsb, const struct sess *sp)
 		VSB_cat(vsb, "}");
 	}
 	VSB_cat(vsb, "\n");
+
+	// duplicated below, remove ?
 	ci = SES_Get_String_Attr(sp, SA_CLIENT_IP);
 	cp = SES_Get_String_Attr(sp, SA_CLIENT_PORT);
 	if (VALID_OBJ(sp->listen_sock, LISTEN_SOCK_MAGIC))
@@ -569,6 +558,17 @@ pan_sess(struct vsb *vsb, const struct sess *sp)
 	else
 		VSB_printf(vsb, "client = %s %s <unknown>\n", ci, cp);
 
+	if (VALID_OBJ(sp->listen_sock, LISTEN_SOCK_MAGIC)) {
+		VSB_printf(vsb, "local.endpoint = %s,\n",
+			   sp->listen_sock->endpoint);
+		VSB_printf(vsb, "local.socket = %s,\n",
+			   sp->listen_sock->name);
+	}
+	pan_addr(vsb, sp, local);
+	pan_addr(vsb, sp, remote);
+	pan_addr(vsb, sp, server);
+	pan_addr(vsb, sp, client);
+
 	VSB_indent(vsb, -2);
 	VSB_cat(vsb, "},\n");
 }
@@ -687,6 +687,22 @@ pan_threadattr(struct vsb *vsb)
 }
 #endif
 
+static void
+pan_argv(struct vsb *vsb)
+{
+	int i;
+
+	VSB_cat(pan_vsb, "argv = {\n");
+	VSB_indent(vsb, 2);
+	for (i = 0; i < heritage.argc; i++) {
+		VSB_printf(vsb, "[%d] = ", i);
+		VSB_quote(vsb, heritage.argv[i], -1, VSB_QUOTE_CSTR);
+		VSB_cat(vsb, ",\n");
+	}
+	VSB_indent(vsb, -2);
+	VSB_cat(vsb, "}\n");
+
+}
 /*--------------------------------------------------------------------*/
 
 static void __attribute__((__noreturn__))
@@ -755,6 +771,8 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 	if (err)
 		VSB_printf(pan_vsb, "errno = %d (%s)\n", err, vstrerror(err));
 
+	pan_argv(pan_vsb);
+
 	VSB_printf(pan_vsb, "pthread.self = %p\n", TRUST_ME(pthread_self()));
 
 	q = THR_GetName();
@@ -777,6 +795,7 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 		if (bo != NULL)
 			VSL_Flush(bo->vsl, 0);
 		VMOD_Panic(pan_vsb);
+		pan_pool(pan_vsb);
 	} else {
 		VSB_cat(pan_vsb, "Feature short panic suppressed details.\n");
 	}
