@@ -42,26 +42,29 @@
 struct vrg_priv {
 	unsigned		magic;
 #define VRG_PRIV_MAGIC		0xb886e711
+	struct req		*req;
 	ssize_t			range_low;
 	ssize_t			range_high;
 	ssize_t			range_off;
 };
 
 static int v_matchproto_(vdp_fini_f)
-vrg_range_fini(struct req *req, void **priv)
+vrg_range_fini(struct vdp_ctx *vdc, void **priv)
 {
 	struct vrg_priv *vrg_priv;
 
-	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+	CHECK_OBJ_NOTNULL(vdc, VDP_CTX_MAGIC);
 	CAST_OBJ_NOTNULL(vrg_priv, *priv, VRG_PRIV_MAGIC);
-	if (vrg_priv->range_off < vrg_priv->range_high)
-		Req_Fail(req, SC_RANGE_SHORT);
+	if (vrg_priv->range_off < vrg_priv->range_high) {
+		Req_Fail(vrg_priv->req, SC_RANGE_SHORT);
+		vrg_priv->req->vdc->retval = -1;
+	}
 	*priv = NULL;	/* struct on ws, no need to free */
 	return (0);
 }
 
 static int v_matchproto_(vdp_bytes_f)
-vrg_range_bytes(struct req *req, enum vdp_action act, void **priv,
+vrg_range_bytes(struct vdp_ctx *vdx, enum vdp_action act, void **priv,
     const void *ptr, ssize_t len)
 {
 	int retval = 0;
@@ -69,7 +72,7 @@ vrg_range_bytes(struct req *req, enum vdp_action act, void **priv,
 	const char *p = ptr;
 	struct vrg_priv *vrg_priv;
 
-	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+	CHECK_OBJ_NOTNULL(vdx, VDP_CTX_MAGIC);
 	AN(priv);
 	CAST_OBJ_NOTNULL(vrg_priv, *priv, VRG_PRIV_MAGIC);
 
@@ -88,9 +91,9 @@ vrg_range_bytes(struct req *req, enum vdp_action act, void **priv,
 	if (vrg_priv->range_off >= vrg_priv->range_high)
 		act = VDP_END;
 	if (l > 0)
-		retval = VDP_bytes(req, act, p, l);
+		retval = VDP_bytes(vdx, act, p, l);
 	else if (l == 0 && act > VDP_NULL)
-		retval = VDP_bytes(req, act, p, 0);
+		retval = VDP_bytes(vdx, act, p, 0);
 	return (retval || act == VDP_END ? 1 : 0);
 }
 
@@ -176,6 +179,7 @@ vrg_dorange(struct req *req, const char *r, void **priv)
 		return ("WS too small");
 
 	INIT_OBJ(vrg_priv, VRG_PRIV_MAGIC);
+	vrg_priv->req = req;
 	vrg_priv->range_off = 0;
 	vrg_priv->range_low = low;
 	vrg_priv->range_high = high + 1;
@@ -242,19 +246,24 @@ vrg_ifrange(struct req *req)
 }
 
 static int v_matchproto_(vdp_init_f)
-vrg_range_init(struct req *req, void **priv)
+vrg_range_init(struct vdp_ctx *vdc, void **priv, struct objcore *oc)
 {
 	const char *r;
 	const char *err;
+	struct req *req;
 
+	CHECK_OBJ_NOTNULL(vdc, VDP_CTX_MAGIC);
+	(void)oc;
+	req = vdc->req;
+	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 	assert(http_GetHdr(req->http, H_Range, &r));
-	if (!vrg_ifrange(req))	// rfc7233,l,455,456
+	if (!vrg_ifrange(req))		// rfc7233,l,455,456
 		return (1);
 	err = vrg_dorange(req, r, priv);
 	if (err == NULL)
 		return (*priv == NULL ? 1 : 0);
 
-	VSLb(req->vsl, SLT_Debug, "RANGE_FAIL %s", err);
+	VSLb(vdc->vsl, SLT_Debug, "RANGE_FAIL %s", err);
 	if (req->resp_len >= 0)
 		http_PrintfHeader(req->resp,
 		    "Content-Range: bytes */%jd",
