@@ -131,8 +131,7 @@ Req_New(const struct worker *wrk, struct sess *sp)
 	p = (void*)PRNDUP(p + sizeof(*req->htc));
 
 	req->vdc = (void*)p;
-	INIT_OBJ(req->vdc, VDP_CTX_MAGIC);
-	VTAILQ_INIT(&req->vdc->vdp);
+	memset(req->vdc, 0, sizeof *req->vdc);
 	p = (void*)PRNDUP(p + sizeof(*req->vdc));
 
 	req->htc = (void*)p;
@@ -152,6 +151,8 @@ Req_New(const struct worker *wrk, struct sess *sp)
 	req->t_prev = NAN;
 	req->t_req = NAN;
 
+	req->req_step = R_STP_TRANSPORT;
+
 	return (req);
 }
 
@@ -189,11 +190,17 @@ Req_Release(struct req *req)
  */
 
 void
-Req_Rollback(struct req *req)
+Req_Rollback(VRT_CTX)
 {
+	struct req *req;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	req = ctx->req;
+	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+
 	if (IS_TOPREQ(req))
-		VCL_TaskLeave(req->top->privs);
-	VCL_TaskLeave(req->privs);
+		VCL_TaskLeave(ctx, req->top->privs);
+	VCL_TaskLeave(ctx, req->privs);
 	VCL_TaskEnter(req->privs);
 	if (IS_TOPREQ(req))
 		VCL_TaskEnter(req->top->privs);
@@ -248,6 +255,7 @@ Req_Cleanup(struct sess *sp, struct worker *wrk, struct req *req)
 	req->hash_ignore_busy = 0;
 	req->esi_level = 0;
 	req->is_hit = 0;
+	req->req_step = R_STP_TRANSPORT;
 
 	if (WS_Overflowed(req->ws))
 		wrk->stats->ws_client_overflow++;
