@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -41,6 +43,55 @@
 #include "common/heritage.h"
 #include "common/vsmw.h"
 
+/* ------------------------------------------------------------
+ * strands helpers - move elsewhere?
+ */
+
+static unsigned
+strands_len(const struct strands *s)
+{
+	unsigned r = 0;
+	int i;
+
+	for (i = 0; i < s->n; i++) {
+		if (s->p[i] == NULL || *s->p[i] == '\0')
+			continue;
+		r += strlen(s->p[i]);
+	}
+
+	return (r);
+}
+
+/*
+ * like VRT_Strands(), but truncating instead of failing for end of buffer
+ *
+ * returns number of bytes including NUL
+ */
+static unsigned
+strands_cat(char *buf, unsigned bufl, const struct strands *s)
+{
+	unsigned l = 0, ll;
+	int i;
+
+	/* NUL-terminated */
+	assert(bufl > 0);
+	bufl--;
+
+	for (i = 0; i < s->n && bufl > 0; i++) {
+		if (s->p[i] == NULL || *s->p[i] == '\0')
+			continue;
+		ll = strlen(s->p[i]);
+		if (ll > bufl)
+			ll = bufl;
+		memcpy(buf, s->p[i], ll);
+		l += ll;
+		buf += ll;
+		bufl -= ll;
+	}
+	*buf = '\0';	/* NUL-terminated */
+	return (l + 1);
+}
+
 /* These cannot be struct lock, which depends on vsm/vsl working */
 static pthread_mutex_t vsl_mtx;
 static pthread_mutex_t vsc_mtx;
@@ -229,6 +280,20 @@ VSLv(enum VSL_tag_e tag, uint32_t vxid, const char *fmt, va_list ap)
 
 }
 
+void
+VSLs(enum VSL_tag_e tag, uint32_t vxid, const struct strands *s)
+{
+	unsigned n, mlen = cache_param->vsl_reclen;
+	char buf[mlen];
+
+	if (vsl_tag_is_masked(tag))
+		return;
+
+	n = strands_cat(buf, mlen, s);
+
+	vslr(tag, vxid, buf, n);
+}
+
 void
 VSL(enum VSL_tag_e tag, uint32_t vxid, const char *fmt, ...)
 {
@@ -302,6 +367,42 @@ VSLbt(struct vsl_log *vsl, enum VSL_tag_e tag, txt t)
 		VSL_Flush(vsl, 0);
 }
 
+/*--------------------------------------------------------------------
+ * VSL-buffered-strands
+ */
+void
+VSLbs(struct vsl_log *vsl, enum VSL_tag_e tag, const struct strands *s)
+{
+	unsigned l, mlen;
+
+	vsl_sanity(vsl);
+	if (vsl_tag_is_masked(tag))
+		return;
+	mlen = cache_param->vsl_reclen;
+
+	/* including NUL */
+	l = strands_len(s) + 1;
+	if (l > mlen)
+		l = mlen;
+
+	assert(vsl->wlp < vsl->wle);
+
+	/* Flush if necessary */
+	if (VSL_END(vsl->wlp, l) >= vsl->wle)
+		VSL_Flush(vsl, 1);
+	assert(VSL_END(vsl->wlp, l) < vsl->wle);
+
+	mlen = strands_cat(VSL_DATA(vsl->wlp), l, s);
+	assert(l == mlen);
+
+	vsl->wlp = vsl_hdr(tag, vsl->wlp, l, vsl->wid);
+	assert(vsl->wlp < vsl->wle);
+	vsl->wlr++;
+
+	if (DO_DEBUG(DBG_SYNCVSL))
+		VSL_Flush(vsl, 0);
+}
+
 /*--------------------------------------------------------------------
  * VSL-buffered
  */
@@ -371,19 +472,26 @@ VSLb(struct vsl_log *vsl, enum VSL_tag_e tag, const char *fmt, ...)
 	va_end(ap);
 }
 
+#define Tf6 "%ju.%06ju"
+#define Ta6(t) (uintmax_t)floor((t)), (uintmax_t)floor((t) * 1e6) % 1000000U
+
 void
 VSLb_ts(struct vsl_log *vsl, const char *event, vtim_real first,
     vtim_real *pprev, vtim_real now)
 {
 
-	/* XXX: Make an option to turn off some unnecessary timestamp
-	   logging. This must be done carefully because some functions
-	   (e.g. V1L_Open) takes the last timestamp as its initial
-	   value for timeout calculation. */
+	/*
+	 * XXX: Make an option to turn off some unnecessary timestamp
+	 * logging. This must be done carefully because some functions
+	 * (e.g. V1L_Open) takes the last timestamp as its initial
+	 * value for timeout calculation.
+	 */
 	vsl_sanity(vsl);
+	AN(event);
+	AN(pprev);
 	assert(!isnan(now) && now != 0.);
-	VSLb(vsl, SLT_Timestamp, "%s: %.6f %.6f %.6f",
-	    event, now, now - first, now - *pprev);
+	VSLb(vsl, SLT_Timestamp, "%s: " Tf6 " " Tf6 " " Tf6,
+	    event, Ta6(now), Ta6(now - first), Ta6(now - *pprev));
 	*pprev = now;
 }
 
@@ -502,7 +610,7 @@ vsm_vsmw_unlock(void)
 void
 VSM_Init(void)
 {
-	int i;
+	unsigned u;
 
 	assert(UINT_MAX % VSL_SEGMENTS == VSL_SEGMENTS - 1);
 
@@ -536,8 +644,8 @@ VSM_Init(void)
 	vsl_head->segsize = vsl_segsize;
 	vsl_head->offset[0] = 0;
 	vsl_head->segment_n = vsl_segment_n;
-	for (i = 1; i < VSL_SEGMENTS; i++)
-		vsl_head->offset[i] = -1;
+	for (u = 1; u < VSL_SEGMENTS; u++)
+		vsl_head->offset[u] = -1;
 	VWMB();
 	memcpy(vsl_head->marker, VSL_HEAD_MARKER, sizeof vsl_head->marker);
 }
