@@ -172,6 +172,39 @@ VCL_Rel_CliCtx(struct vrt_ctx **ctx)
 
 /*--------------------------------------------------------------------*/
 
+/* VRT_fail() can be called
+ * - from the vcl sub via a vmod
+ * - via a PRIV_TASK .fini callback
+ *
+ * if this happens during init, we fail it
+ * if during fini, we ignore, because otherwise VMOD authors would be forced to
+ * handle VCL_MET_FINI specifically everywhere.
+ */
+
+static int
+vcl_event_handling(VRT_CTX)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+
+	if (*ctx->handling == 0)
+		return (0);
+
+	assert(*ctx->handling == VCL_RET_FAIL);
+
+	if (ctx->method == VCL_MET_INIT)
+		return (1);
+
+	/*
+	 * EVENT_WARM / EVENT_COLD: method == 0
+	 * must not set handling
+	 */
+	assert(ctx->method == VCL_MET_FINI);
+
+	*ctx->handling = 0;
+	VRT_fail(ctx, "VRT_fail() from vcl_fini{} has no effect");
+	return (0);
+}
+
 static int
 vcl_send_event(struct vcl *vcl, enum vcl_event_e ev, struct vsb **msg)
 {
@@ -215,14 +248,8 @@ vcl_send_event(struct vcl *vcl, enum vcl_event_e ev, struct vsb **msg)
 
 	VCL_TaskEnter(cli_task_privs);
 	r = ctx->vcl->conf->event_vcl(ctx, ev);
-	VCL_TaskLeave(cli_task_privs);
-
-	/* if the warm event did not get to vcl_init, vcl_fini
-	 * won't be run, so handling may be zero */
-	if (method && *ctx->handling && *ctx->handling != VCL_RET_OK) {
-		assert(ev == VCL_EVENT_LOAD);
-		r = 1;
-	}
+	VCL_TaskLeave(ctx, cli_task_privs);
+	r |= vcl_event_handling(ctx);
 
 	*msg = VCL_Rel_CliCtx(&ctx);
 
@@ -251,48 +278,48 @@ vcl_find(const char *name)
 
 /*--------------------------------------------------------------------*/
 
+static void
+vcl_panic_conf(struct vsb *vsb, const struct VCL_conf *conf)
+{
+	int i;
+	const struct vpi_ii *ii;
+
+	if (PAN_dump_struct(vsb, conf, VCL_CONF_MAGIC, "conf"))
+		return;
+	VSB_printf(vsb, "syntax = \"%u\",\n", conf->syntax);
+	VSB_cat(vsb, "srcname = {\n");
+	VSB_indent(vsb, 2);
+	for (i = 0; i < conf->nsrc; ++i)
+		VSB_printf(vsb, "\"%s\",\n", conf->srcname[i]);
+	VSB_indent(vsb, -2);
+	VSB_cat(vsb, "},\n");
+	VSB_cat(vsb, "instances = {\n");
+	VSB_indent(vsb, 2);
+	ii = conf->instance_info;
+	while (ii != NULL && ii->p != NULL) {
+		VSB_printf(vsb, "\"%s\" = %p,\n", ii->name,
+		    (const void *)*(const uintptr_t *)ii->p);
+		ii++;
+	}
+	VSB_indent(vsb, -2);
+	VSB_cat(vsb, "},\n");
+	VSB_indent(vsb, -2);
+	VSB_cat(vsb, "},\n");
+}
+
 void
 VCL_Panic(struct vsb *vsb, const char *nm, const struct vcl *vcl)
 {
-	const struct vpi_ii *ii;
-	int i;
 
 	AN(vsb);
-	if (vcl == NULL)
+	if (PAN_dump_struct(vsb, vcl, VCL_MAGIC, "vcl[%s]", nm))
 		return;
-	VSB_printf(vsb, "%s = {\n", nm);
-	VSB_indent(vsb, 2);
-	PAN_CheckMagic(vsb, vcl, VCL_MAGIC);
 	VSB_printf(vsb, "name = \"%s\",\n", vcl->loaded_name);
 	VSB_printf(vsb, "busy = %u,\n", vcl->busy);
 	VSB_printf(vsb, "discard = %u,\n", vcl->discard);
 	VSB_printf(vsb, "state = %s,\n", vcl->state);
 	VSB_printf(vsb, "temp = %s,\n", vcl->temp ? vcl->temp->name : "(null)");
-	VSB_cat(vsb, "conf = {\n");
-	VSB_indent(vsb, 2);
-	if (vcl->conf == NULL) {
-		VSB_cat(vsb, "conf = NULL\n");
-	} else {
-		PAN_CheckMagic(vsb, vcl->conf, VCL_CONF_MAGIC);
-		VSB_printf(vsb, "syntax = \"%u\",\n", vcl->conf->syntax);
-		VSB_cat(vsb, "srcname = {\n");
-		VSB_indent(vsb, 2);
-		for (i = 0; i < vcl->conf->nsrc; ++i)
-			VSB_printf(vsb, "\"%s\",\n", vcl->conf->srcname[i]);
-		VSB_indent(vsb, -2);
-		VSB_cat(vsb, "instances = {\n");
-		VSB_indent(vsb, 2);
-		ii = vcl->conf->instance_info;
-		while (ii != NULL && ii->p != NULL) {
-			VSB_printf(vsb, "\"%s\" = %p,\n", ii->name,
-			    (const void *)*(const uintptr_t *)ii->p);
-			ii++;
-		}
-		VSB_indent(vsb, -2);
-		VSB_cat(vsb, "},\n");
-	}
-	VSB_indent(vsb, -2);
-	VSB_cat(vsb, "},\n");
+	vcl_panic_conf(vsb, vcl->conf);
 	VSB_indent(vsb, -2);
 	VSB_cat(vsb, "},\n");
 }
@@ -756,7 +783,7 @@ vcl_cli_list(struct cli *cli, const char * const *av, void *priv)
 			VSB_printf(vsb, "\t<-\t(%d label%s)",
 			    vcl->nlabels, vcl->nlabels > 1 ? "s" : "");
 		}
-		VSB_printf(vsb, "\n");
+		VSB_cat(vsb, "\n");
 	}
 	VCLI_VTE(cli, &vsb, 80);
 }
