@@ -33,6 +33,8 @@
 
 #include "config.h"
 
+#include <stdlib.h>
+
 #include "cache_varnishd.h"
 
 #include "cache_objhead.h"
@@ -83,6 +85,14 @@ VRT_synth(VRT_CTX, VCL_INT code, VCL_STRING reason)
 		return;
 	}
 
+	if (reason && !WS_Inside(ctx->ws, reason, NULL)) {
+		reason = WS_Copy(ctx->ws, reason, -1);
+		if (!reason) {
+			VRT_fail(ctx, "Workspace overflow");
+			return;
+		}
+	}
+
 	if (ctx->req == NULL) {
 		CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
 		ctx->bo->err_code = (uint16_t)code;
@@ -99,7 +109,7 @@ VRT_synth(VRT_CTX, VCL_INT code, VCL_STRING reason)
 /*--------------------------------------------------------------------*/
 
 void
-VRT_acl_log(VRT_CTX, const char *msg)
+VPI_acl_log(VRT_CTX, const char *msg)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
@@ -222,27 +232,6 @@ VRT_AllocStrandsWS(struct ws *ws, int n)
 	return (s);
 }
 
-/*--------------------------------------------------------------------
- * Build STRANDS from what is essentially a STRING_LIST
- */
-
-VCL_STRANDS
-VPI_BundleStrands(int n, struct strands *s, char const **d, const char *f, ...)
-{
-	va_list ap;
-
-	assert(n > 0);
-	s->n = n;
-	s->p = d;
-	*d++ = f;
-	va_start(ap, f);
-	while(--n)
-		*d++ = va_arg(ap, const char *);
-	assert(va_arg(ap, const char *) == vrt_magic_string_end);
-	va_end(ap);
-	return (s);
-}
-
 /*--------------------------------------------------------------------
  * Compare two STRANDS
  */
@@ -590,6 +579,35 @@ VRT_UpperLowerStrands(VRT_CTX, VCL_STRANDS s, int up)
 	return (r);
 }
 
+// rfc7230,l,1243,1244
+// ASCII VCHAR + TAB + obs-text (0x80-ff)
+static inline VCL_BOOL
+validhdr(const char *p)
+{
+	AN(p);
+	for(;*p != '\0'; p++)
+		if (! vct_ishdrval(*p))
+			return (0);
+	return (1);
+}
+
+/*--------------------------------------------------------------------*/
+VCL_BOOL
+VRT_ValidHdr(VRT_CTX, VCL_STRANDS s)
+{
+	int i;
+
+	(void) ctx;
+
+	for (i = 0; i < s->n; i++) {
+		if (s->p[i] == NULL || s->p[i][0] == '\0')
+			continue;
+		if (! validhdr(s->p[i]))
+			return (0);
+	}
+
+	return (1);
+}
 /*--------------------------------------------------------------------*/
 
 VCL_VOID
@@ -604,19 +622,23 @@ VRT_SetHdr(VRT_CTX , VCL_HEADER hs, const char *p, ...)
 	AN(hs->what);
 	hp = VRT_selecthttp(ctx, hs->where);
 	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
-	va_start(ap, p);
 	if (p == vrt_magic_string_unset) {
 		http_Unset(hp, hs->what);
 	} else {
+		va_start(ap, p);
 		b = VRT_String(hp->ws, hs->what + 1, p, ap);
+		va_end(ap);
 		if (b == NULL) {
 			VSLb(ctx->vsl, SLT_LostHeader, "%s", hs->what + 1);
-		} else {
-			http_Unset(hp, hs->what);
-			http_SetHeader(hp, b);
+			return;
 		}
+		if (FEATURE(FEATURE_VALIDATE_HEADERS) && ! validhdr(b)) {
+			VRT_fail(ctx, "Bad header %s", b);
+			return;
+		}
+		http_Unset(hp, hs->what);
+		http_SetHeader(hp, b);
 	}
-	va_end(ap);
 }
 
 /*--------------------------------------------------------------------*/
@@ -634,6 +656,14 @@ VRT_handling(VRT_CTX, unsigned hand)
 	*ctx->handling = hand;
 }
 
+unsigned
+VRT_handled(VRT_CTX)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	AN(ctx->handling);
+	return (*ctx->handling);
+}
+
 /*--------------------------------------------------------------------*/
 
 VCL_VOID
@@ -783,16 +813,19 @@ VRT_Rollback(VRT_CTX, VCL_HTTP hp)
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
+	if (ctx->method & VCL_MET_PIPE) {
+		VRT_fail(ctx, "Cannot rollback in vcl_pipe {}");
+		return;
+	}
 	if (hp == ctx->http_req) {
 		CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
-		Req_Rollback(ctx->req);
+		Req_Rollback(ctx);
 		if (ctx->method & VCL_MET_DELIVER)
 			XXXAZ(Resp_Setup_Deliver(ctx->req));
 		if (ctx->method & VCL_MET_SYNTH)
 			Resp_Setup_Synth(ctx->req);
 	} else if (hp == ctx->http_bereq) {
-		CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
-		Bereq_Rollback(ctx->bo);
+		Bereq_Rollback(ctx);
 	} else
 		WRONG("VRT_Rollback 'hp' invalid");
 }
@@ -800,7 +833,7 @@ VRT_Rollback(VRT_CTX, VCL_HTTP hp)
 /*--------------------------------------------------------------------*/
 
 VCL_VOID
-VRT_synth_page(VRT_CTX, VCL_STRANDS s)
+VRT_synth_strands(VRT_CTX, VCL_STRANDS s)
 {
 	struct vsb *vsb;
 	int i;
@@ -815,80 +848,110 @@ VRT_synth_page(VRT_CTX, VCL_STRANDS s)
 	}
 }
 
-/*--------------------------------------------------------------------*/
+VCL_VOID
+VRT_synth_blob(VRT_CTX, VCL_BLOB b)
+{
+	struct vsb *vsb;
+	CAST_OBJ_NOTNULL(vsb, ctx->specific, VSB_MAGIC);
+
+	if (b->len > 0 && b->blob != NULL)
+		VSB_bcat(vsb, b->blob, b->len);
+}
 
 VCL_VOID
+VRT_synth_page(VRT_CTX, VCL_STRANDS s)
+{
+	VRT_synth_strands(ctx, s);
+}
+
+/*--------------------------------------------------------------------*/
+
+static VCL_STRING
+vrt_ban_error(VRT_CTX, VCL_STRING err)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	AN(ctx->vsl);
+	AN(err);
+
+	VSLb(ctx->vsl, SLT_VCL_Error, "ban(): %s", err);
+	return (err);
+}
+
+VCL_STRING
 VRT_ban_string(VRT_CTX, VCL_STRING str)
 {
 	char *a1, *a2, *a3;
 	char **av;
 	struct ban_proto *bp;
-	const char *err;
+	const char *err = NULL, *berr = NULL;
 	int i;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	AN(ctx->vsl);
 
-	if (str == NULL) {
-		VSLb(ctx->vsl, SLT_VCL_Error, "ban(): Null argument");
-		return;
-	}
+	if (str == NULL)
+		return (vrt_ban_error(ctx, "Null argument"));
 
 	bp = BAN_Build();
-	if (bp == NULL) {
-		VSLb(ctx->vsl, SLT_VCL_Error, "ban(): Out of Memory");
-		return;
-	}
+	if (bp == NULL)
+		return (vrt_ban_error(ctx, "Out of Memory"));
+
 	av = VAV_Parse(str, NULL, ARGV_NOESC);
 	AN(av);
 	if (av[0] != NULL) {
-		VSLb(ctx->vsl, SLT_VCL_Error, "ban(): %s", av[0]);
+		err = av[0];
 		VAV_Free(av);
 		BAN_Abandon(bp);
-		return;
+		return (vrt_ban_error(ctx, err));
 	}
 	for (i = 0; ;) {
 		a1 = av[++i];
 		if (a1 == NULL) {
-			VSLb(ctx->vsl, SLT_VCL_Error,
-			    "ban(): No ban conditions found.");
+			err = "No ban conditions found.";
 			break;
 		}
 		a2 = av[++i];
 		if (a2 == NULL) {
-			VSLb(ctx->vsl, SLT_VCL_Error,
-			    "ban(): Expected comparison operator.");
+			err = "Expected comparison operator.";
 			break;
 		}
 		a3 = av[++i];
 		if (a3 == NULL) {
-			VSLb(ctx->vsl, SLT_VCL_Error,
-			    "ban(): Expected second operand.");
+			err = "Expected second operand.";
 			break;
 		}
-		err = BAN_AddTest(bp, a1, a2, a3);
-		if (err) {
-			VSLb(ctx->vsl, SLT_VCL_Error, "ban(): %s", err);
+		berr = BAN_AddTest(bp, a1, a2, a3);
+		if (berr != NULL)
 			break;
-		}
+
 		if (av[++i] == NULL) {
-			err = BAN_Commit(bp);
-			if (err == NULL)
+			berr = BAN_Commit(bp);
+			if (berr == NULL)
 				bp = NULL;
-			else
-				VSLb(ctx->vsl, SLT_VCL_Error, "ban(): %s", err);
 			break;
 		}
 		if (strcmp(av[i], "&&")) {
-			VSLb(ctx->vsl, SLT_VCL_Error,
-			    "ban(): Expected && between conditions,"
-			    " found \"%s\"", av[i]);
+			err = WS_Printf(ctx->ws, "Expected && between "
+			    "conditions, found \"%s\"", av[i]);
+			if (err == NULL)
+				err = "Expected && between conditions "
+				    "(workspace overflow)";
 			break;
 		}
 	}
+	if (berr != NULL) {
+		AZ(err);
+		err = WS_Copy(ctx->ws, berr, -1);
+		if (err == NULL)
+			err = "Unknown error (workspace overflow)";
+		berr = NULL;
+	}
+	AZ(berr);
 	if (bp != NULL)
 		BAN_Abandon(bp);
 	VAV_Free(av);
+	if (err == NULL)
+		return (NULL);
+	return (vrt_ban_error(ctx, err));
 }
 
 VCL_BYTES
@@ -982,7 +1045,7 @@ VRT_ipcmp(VRT_CTX, VCL_IP sua1, VCL_IP sua2)
 
 	if (sua1 == NULL || sua2 == NULL) {
 		VRT_fail(ctx, "%s: Illegal IP", __func__);
-		return(1);
+		return (1);
 	}
 	return (VSA_Compare_IP(sua1, sua2));
 }
@@ -1032,3 +1095,70 @@ VRT_Format_Proxy(struct vsb *vsb, VCL_INT version, VCL_IP sac, VCL_IP sas,
 {
 	VPX_Format_Proxy(vsb, (int)version, sac, sas, auth);
 }
+
+/*
+ * Clone a struct vrt_endpoint in a single malloc() allocation
+ */
+
+//lint -e{662}  Possible of out-of-bounds pointer (___ beyond end of data)
+//lint -e{826}  Suspicious pointer-to-pointer conversion (area to o small
+struct vrt_endpoint *
+VRT_Endpoint_Clone(const struct vrt_endpoint * const vep)
+{
+	size_t sz;
+	struct vrt_endpoint *nvep;
+	struct vrt_blob *blob = NULL;
+	struct suckaddr *sa;
+	size_t uds_len = 0;
+	char *p, *e;
+
+	CHECK_OBJ_NOTNULL(vep, VRT_ENDPOINT_MAGIC);
+	sz = sizeof *nvep;
+	if (vep->ipv4)
+		sz += vsa_suckaddr_len;
+	if (vep->ipv6)
+		sz += vsa_suckaddr_len;
+	if (vep->uds_path != NULL) {
+		uds_len = strlen(vep->uds_path) + 1;
+		sz += uds_len;
+	}
+	if (vep->preamble != NULL && vep->preamble->len) {
+		sz += sizeof(*blob);
+		sz += vep->preamble->len;
+	}
+	p = calloc(1, sz);
+	AN(p);
+	e = p + sz;
+	nvep = (void*)p;
+	p += sizeof *nvep;
+	INIT_OBJ(nvep, VRT_ENDPOINT_MAGIC);
+	if (vep->ipv4) {
+		sa = (void*)p;
+		memcpy(sa, vep->ipv4, vsa_suckaddr_len);
+		nvep->ipv4 = sa;
+		p += vsa_suckaddr_len;
+	}
+	if (vep->ipv6) {
+		sa = (void*)p;
+		memcpy(sa, vep->ipv6, vsa_suckaddr_len);
+		nvep->ipv6 = sa;
+		p += vsa_suckaddr_len;
+	}
+	if (vep->preamble != NULL && vep->preamble->len) {
+		/* Before uds because we need p to be aligned still */
+		blob = (void*)p;
+		p += sizeof(*blob);
+		nvep->preamble = blob;
+		memcpy(p, vep->preamble->blob, vep->preamble->len);
+		blob->len = vep->preamble->len;
+		blob->blob = p;
+		p += vep->preamble->len;
+	}
+	if (uds_len) {
+		memcpy(p, vep->uds_path, uds_len);
+		nvep->uds_path = p;
+		p += uds_len;
+	}
+	assert(p == e);
+	return (nvep);
+}
