@@ -189,9 +189,10 @@ VRT_r_obj_reason(VRT_CTX)
 static inline int
 beresp_filter_fixed(VRT_CTX, const char *s)
 {
-	if (ctx->bo->filter_list == NULL)
+	if (ctx->bo->vfp_filter_list == NULL)
 		return (0);
-	VRT_fail(ctx, "beresp.filters are already fixed, beresp.%s is undefined", s);
+	VRT_fail(ctx,
+	    "beresp.filters are already fixed, beresp.%s is undefined", s);
 	return (1);
 }
 
@@ -229,9 +230,20 @@ VRT_r_beresp_##field(VRT_CTX)						\
 	return (ctx->bo->field);					\
 }
 
-#define BO_FLAG(l, r, w, f, d)			\
-	VBERESPR##r(l, #l, f)			\
-	VBERESPW##w(l, #l, f)
+#define VBEREQR0(field, str, fltchk)
+#define VBEREQR1(field, str, fltchk)					\
+VCL_BOOL								\
+VRT_r_bereq_##field(VRT_CTX)						\
+{									\
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
+	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);			\
+	return (ctx->bo->field);					\
+}
+
+#define BO_FLAG(l, r, rr, rw, f, d)		\
+	VBEREQR##r(l, #l, f)			\
+	VBERESPR##rr(l, #l, f)			\
+	VBERESPW##rw(l, #l, f)
 #include "tbl/bo_flags.h"
 
 #undef VBERESPWF0
@@ -245,14 +257,6 @@ VRT_r_beresp_##field(VRT_CTX)						\
 #undef VBERESPR1
 /*--------------------------------------------------------------------*/
 
-VCL_BOOL
-VRT_r_bereq_is_bgfetch(VRT_CTX)
-{
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
-	return (ctx->bo->is_bgfetch);
-}
-
 VCL_BOOL
 VRT_r_bereq_uncacheable(VRT_CTX)
 {
@@ -296,11 +300,19 @@ VRT_r_beresp_uncacheable(VRT_CTX)
 VCL_STRING
 VRT_r_client_identity(VRT_CTX)
 {
+	const char *id;
+
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
-	if (ctx->req->client_identity != NULL)
-		return (ctx->req->client_identity);
-	return (SES_Get_String_Attr(ctx->req->sp, SA_CLIENT_IP));
+	if (ctx->req != NULL) {
+		CHECK_OBJ(ctx->req, REQ_MAGIC);
+		id = ctx->req->client_identity;
+	} else {
+		CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
+		id = ctx->bo->client_identity;
+	}
+	if (id != NULL)
+		return (id);
+	return (SES_Get_String_Attr(ctx->sp, SA_CLIENT_IP));
 }
 
 VCL_VOID
