@@ -160,15 +160,45 @@ WRK_Thread(struct pool *qp, size_t stacksize, unsigned thread_workspace)
  * Summing of stats into pool counters
  */
 
-static void
-pool_addstat(struct VSC_main_wrk *dst, struct VSC_main_wrk *src)
+static unsigned
+wrk_addstat(const struct worker *wrk, const struct pool_task *tp, unsigned locked)
+{
+	struct pool *pp;
+
+	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
+	pp = wrk->pool;
+	CHECK_OBJ_NOTNULL(pp, POOL_MAGIC);
+	if (locked)
+		Lck_AssertHeld(&pp->mtx);
+
+	if ((tp == NULL && wrk->stats->summs > 0) ||
+	    (wrk->stats->summs >= cache_param->wthread_stats_rate)) {
+		if (!locked)
+			Lck_Lock(&pp->mtx);
+
+		pp->a_stat->summs++;
+		VSC_main_Summ_wrk_wrk(pp->a_stat, wrk->stats);
+		memset(wrk->stats, 0, sizeof *wrk->stats);
+
+		if (!locked)
+			Lck_Unlock(&pp->mtx);
+	}
+
+	return (tp == NULL ? 0 : 1);
+}
+
+void
+WRK_AddStat(const struct worker *wrk)
 {
 
-	dst->summs++;
-	VSC_main_Summ_wrk_wrk(dst, src);
-	memset(src, 0, sizeof *src);
+	(void)wrk_addstat(wrk, wrk->task, 0);
+	wrk->stats->summs++;
 }
 
+/*--------------------------------------------------------------------
+ * Pool reserve calculation
+ */
+
 static unsigned
 pool_reserve(void)
 {
@@ -366,12 +396,9 @@ Pool_Work_Thread(struct pool *pp, struct worker *wrk)
 			}
 		}
 
-		if ((tp == NULL && wrk->stats->summs > 0) ||
-		    (wrk->stats->summs >= cache_param->wthread_stats_rate))
-			pool_addstat(pp->a_stat, wrk->stats);
-
-		if (tp != NULL) {
+		if (wrk_addstat(wrk, tp, 1)) {
 			wrk->stats->summs++;
+			AN(tp);
 		} else if (pp->b_stat != NULL && pp->a_stat->summs) {
 			/* Nothing to do, push pool stats into global pool */
 			tps.func = pool_stat_summ;
@@ -389,7 +416,17 @@ Pool_Work_Thread(struct pool *pp, struct worker *wrk)
 			pp->nidle++;
 			do {
 				// see signaling_note at the top for explanation
-				if (wrk->vcl == NULL)
+				if (DO_DEBUG(DBG_VCLREL) &&
+				    pp->b_stat == NULL && pp->a_stat->summs)
+					/* We've released the VCL, but
+					 * there are pool stats not pushed
+					 * to the global stats and some
+					 * thread is busy pushing
+					 * stats. Set a 1 second timeout
+					 * so that we'll wake up and get a
+					 * chance to push stats. */
+					tmo = wrk->lastused + 1.;
+				else if (wrk->vcl == NULL)
 					tmo = 0;
 				else if (DO_DEBUG(DBG_VTC_MODE))
 					tmo =  wrk->lastused+1.;
@@ -398,10 +435,32 @@ Pool_Work_Thread(struct pool *pp, struct worker *wrk)
 				i = Lck_CondWait(&wrk->cond, &pp->mtx, tmo);
 				if (i == ETIMEDOUT)
 					VCL_Rel(&wrk->vcl);
-			} while (wrk->task->func == NULL);
-			tpx = *wrk->task;
-			tp = &tpx;
-			wrk->stats->summs++;
+				if (wrk->task->func != NULL) {
+					/* We have been handed a new task */
+					tpx = *wrk->task;
+					tp = &tpx;
+					wrk->stats->summs++;
+				} else if (pp->b_stat != NULL &&
+				    pp->a_stat->summs) {
+					/* Woken up to release the VCL,
+					 * and noticing that there are
+					 * pool stats not pushed to the
+					 * global stats and no active
+					 * thread currently doing
+					 * it. Remove ourself from the
+					 * idle queue and take on the
+					 * task. */
+					assert(pp->nidle > 0);
+					VTAILQ_REMOVE(&pp->idle_queue,
+					    wrk->task, list);
+					pp->nidle--;
+					tps.func = pool_stat_summ;
+					tps.priv = pp->a_stat;
+					pp->a_stat = pp->b_stat;
+					pp->b_stat = NULL;
+					tp = &tps;
+				}
+			} while (tp == NULL);
 		}
 		Lck_Unlock(&pp->mtx);
 
@@ -469,7 +528,8 @@ pool_breed(struct pool *qp)
 	AZ(pthread_attr_getstacksize(&tp_attr, &pi->stacksize));
 	pi->qp = qp;
 
-	if (pthread_create(&tp, &tp_attr, pool_thread, pi)) {
+	errno = pthread_create(&tp, &tp_attr, pool_thread, pi);
+	if (errno) {
 		FREE_OBJ(pi);
 		VSL(SLT_Debug, 0, "Create worker thread failed %d %s",
 		    errno, vstrerror(errno));
@@ -519,6 +579,7 @@ pool_herder(void *priv)
 	unsigned wthread_min;
 	uintmax_t dq = (1ULL << 31);
 	vtim_mono dqt = 0;
+	int r = 0;
 
 	CAST_OBJ_NOTNULL(pp, priv, POOL_MAGIC);
 
@@ -539,7 +600,9 @@ pool_herder(void *priv)
 		 * Instead we implement a watchdog and kill the worker if
 		 * nothing has been dequeued for that long.
 		 */
-		if (pp->lqueue == 0) {
+		if (VTAILQ_EMPTY(&pp->queues[TASK_QUEUE_HIGHEST_PRIORITY])) {
+			/* Watchdog only applies to no movement on the
+			 * highest priority queue (TASK_QUEUE_BO) */
 			dq = pp->ndequeued + 1;
 		} else if (dq != pp->ndequeued) {
 			dq = pp->ndequeued;
@@ -620,11 +683,15 @@ pool_herder(void *priv)
 		if (pp->lqueue == 0) {
 			if (DO_DEBUG(DBG_VTC_MODE))
 				delay = 0.5;
-			(void)Lck_CondWait(&pp->herder_cond, &pp->mtx,
-				VTIM_real() + delay);
-		} else
+			r = Lck_CondWait(&pp->herder_cond, &pp->mtx,
+			    VTIM_real() + delay);
+		} else if (pp->nthr >= cache_param->wthread_max) {
 			/* XXX: unsafe counters */
-			VSC_C_main->threads_limited++;
+			if (r != ETIMEDOUT)
+				VSC_C_main->threads_limited++;
+			r = Lck_CondWait(&pp->herder_cond, &pp->mtx,
+			    VTIM_real() + 1.0);
+		}
 		Lck_Unlock(&pp->mtx);
 	}
 	return (NULL);
