@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -33,22 +35,24 @@
 #include "cache/cache_filter.h"
 #include "cache_http1.h"
 
+#include "vtcp.h"
+
 /*--------------------------------------------------------------------*/
 
 static int v_matchproto_(vdp_bytes_f)
-v1d_bytes(struct req *req, enum vdp_action act, void **priv,
+v1d_bytes(struct vdp_ctx *vdx, enum vdp_action act, void **priv,
     const void *ptr, ssize_t len)
 {
 	ssize_t wl = 0;
 
-	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+	CHECK_OBJ_NOTNULL(vdx, VDP_CTX_MAGIC);
 	(void)priv;
 
-	AZ(req->vdc->nxt);		/* always at the bottom of the pile */
+	AZ(vdx->nxt);		/* always at the bottom of the pile */
 
 	if (len > 0)
-		wl = V1L_Write(req->wrk, ptr, len);
-	if (act > VDP_NULL && V1L_Flush(req->wrk))
+		wl = V1L_Write(vdx->wrk, ptr, len);
+	if (act > VDP_NULL && V1L_Flush(vdx->wrk))
 		return (-1);
 	if (len != wl)
 		return (-1);
@@ -74,7 +78,7 @@ v1d_error(struct req *req, const char *msg)
 	VSLb(req->vsl, SLT_RespReason, "Internal Server Error");
 
 	req->wrk->stats->client_resp_500++;
-	(void)write(req->sp->fd, r_500, sizeof r_500 - 1);
+	VTCP_Assert(write(req->sp->fd, r_500, sizeof r_500 - 1));
 	req->doclose = SC_TX_EOF;
 }
 
@@ -85,7 +89,7 @@ void v_matchproto_(vtr_deliver_f)
 V1D_Deliver(struct req *req, struct boc *boc, int sendbody)
 {
 	int err = 0, chunked = 0;
-	unsigned u;
+	enum sess_close sc;
 	uint64_t hdrbytes, bytes;
 
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
@@ -112,7 +116,7 @@ V1D_Deliver(struct req *req, struct boc *boc, int sendbody)
 				req->doclose = SC_TX_EOF;
 			}
 		}
-		if (VDP_Push(req, &v1d_vdp, NULL)) {
+		if (VDP_Push(req->vdc, req->ws, &v1d_vdp, NULL)) {
 			v1d_error(req, "workspace_thread overflow");
 			AZ(req->wrk->v1l);
 			return;
@@ -130,8 +134,9 @@ V1D_Deliver(struct req *req, struct boc *boc, int sendbody)
 	}
 
 	AZ(req->wrk->v1l);
-	V1L_Open(req->wrk, req->wrk->aws,
-		 &req->sp->fd, req->vsl, req->t_prev, cache_param->http1_iovs);
+	V1L_Open(req->wrk, req->wrk->aws, &req->sp->fd, req->vsl,
+	    req->t_prev + SESS_TMO(req->sp, send_timeout),
+	    cache_param->http1_iovs);
 
 	if (WS_Overflowed(req->wrk->aws)) {
 		v1d_error(req, "workspace_thread overflow");
@@ -146,23 +151,19 @@ V1D_Deliver(struct req *req, struct boc *boc, int sendbody)
 			(void)V1L_Flush(req->wrk);
 		if (chunked)
 			V1L_Chunked(req->wrk);
-		err = VDP_DeliverObj(req);
+		err = VDP_DeliverObj(req->vdc, req->objcore);
 		if (!err && chunked)
 			V1L_EndChunk(req->wrk);
 	}
 
-	u = V1L_Close(req->wrk, &bytes);
+	sc = V1L_Close(req->wrk, &bytes);
 	AZ(req->wrk->v1l);
 
-	/* Bytes accounting */
-	if (bytes < hdrbytes)
-		req->acct.resp_hdrbytes += bytes;
-	else {
-		req->acct.resp_hdrbytes += hdrbytes;
-		req->acct.resp_bodybytes += bytes - hdrbytes;
-	}
+	req->acct.resp_hdrbytes += hdrbytes;
+	req->acct.resp_bodybytes += VDP_Close(req->vdc);
 
-	if ((u || err) && req->sp->fd >= 0)
-		Req_Fail(req, SC_REM_CLOSE);
-	VDP_close(req);
+	if (sc == SC_NULL && err && req->sp->fd >= 0)
+		sc = SC_REM_CLOSE;
+	if (sc != SC_NULL)
+		Req_Fail(req, sc);
 }
