@@ -107,6 +107,7 @@ http1_new_session(struct worker *wrk, void *arg)
 	struct sess *sp;
 	struct req *req;
 	uintptr_t *u;
+	ssize_t sz;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CAST_OBJ_NOTNULL(req, arg, REQ_MAGIC);
@@ -114,7 +115,7 @@ http1_new_session(struct worker *wrk, void *arg)
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 
 	HTC_RxInit(req->htc, req->ws);
-	if (!SES_Reserve_proto_priv(sp, &u)) {
+	if (!SES_Reserve_proto_priv(sp, &u, &sz)) {
 		/* Out of session workspace. Free the req, close the sess,
 		 * and do not set a new task func, which will exit the
 		 * worker thread. */
@@ -123,6 +124,7 @@ http1_new_session(struct worker *wrk, void *arg)
 		SES_Delete(sp, SC_RX_JUNK, NAN);
 		return;
 	}
+	assert(sz == sizeof u);
 	http1_setstate(sp, H1NEWREQ);
 	wrk->task->func = http1_req;
 	wrk->task->priv = req;
@@ -325,22 +327,8 @@ HTTP1_Session(struct worker *wrk, struct req *req)
 				    req->htc->rxbuf_e - req->htc->rxbuf_b;
 				Req_AcctLogCharge(wrk->stats, req);
 				Req_Release(req);
-				switch (hs) {
-				case HTC_S_CLOSE:
-					SES_Delete(sp, SC_REM_CLOSE, NAN);
-					return;
-				case HTC_S_TIMEOUT:
-					SES_Delete(sp, SC_RX_TIMEOUT, NAN);
-					return;
-				case HTC_S_OVERFLOW:
-					SES_Delete(sp, SC_RX_OVERFLOW, NAN);
-					return;
-				case HTC_S_EOF:
-					SES_Delete(sp, SC_REM_CLOSE, NAN);
-					return;
-				default:
-					WRONG("htc_status (bad)");
-				}
+				SES_DeleteHS(sp, hs, NAN);
+				return;
 			}
 			if (hs == HTC_S_IDLE) {
 				wrk->stats->sess_herd++;
@@ -389,17 +377,15 @@ HTTP1_Session(struct worker *wrk, struct req *req)
 					return;
 				}
 			}
-			req->req_step = R_STP_TRANSPORT;
+			assert(req->req_step == R_STP_TRANSPORT);
+			VCL_TaskEnter(req->privs);
+			VCL_TaskEnter(req->top->privs);
 			http1_setstate(sp, H1PROC);
 		} else if (st == H1PROC) {
 			req->task->func = http1_req;
 			req->task->priv = req;
 			wrk->stats->client_req++;
 			CNT_Embark(wrk, req);
-			if (req->req_step == R_STP_TRANSPORT) {
-				VCL_TaskEnter(req->privs);
-				VCL_TaskEnter(req->top->privs);
-			}
 			if (CNT_Request(req) == REQ_FSM_DISEMBARK)
 				return;
 			AZ(req->top->vcl0);
@@ -428,6 +414,8 @@ HTTP1_Session(struct worker *wrk, struct req *req)
 			HTC_RxInit(req->htc, req->ws);
 			if (req->htc->rxbuf_e != req->htc->rxbuf_b)
 				wrk->stats->sess_readahead++;
+			if (FEATURE(FEATURE_BUSY_STATS_RATE))
+				WRK_AddStat(wrk);
 			http1_setstate(sp, H1NEWREQ);
 		} else {
 			WRONG("Wrong H1 session state");
