@@ -4,6 +4,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -38,6 +40,7 @@
 
 #include "vend.h"
 #include "vtim.h"
+#include "vtcp.h"
 
 static const char h2_resp_101[] =
 	"HTTP/1.1 101 Switching Protocols\r\n"
@@ -99,42 +102,37 @@ h2_init_sess(const struct worker *wrk, struct sess *sp,
 	uintptr_t *up;
 	struct h2_sess *h2;
 
-	if (SES_Get_proto_priv(sp, &up)) {
-		/* Already reserved if we came via H1 */
-		SES_Reserve_proto_priv(sp, &up);
-		*up = 0;
-	}
-	if (*up == 0) {
-		if (srq == NULL)
-			srq = Req_New(wrk, sp);
-		AN(srq);
-		h2 = h2s;
-		AN(h2);
-		INIT_OBJ(h2, H2_SESS_MAGIC);
-		h2->srq = srq;
-		h2->htc = srq->htc;
-		h2->ws = srq->ws;
-		h2->vsl = srq->vsl;
-		VSL_Flush(h2->vsl, 0);
-		h2->vsl->wid = sp->vxid;
-		h2->htc->rfd = &sp->fd;
-		h2->sess = sp;
-		h2->rxthr = pthread_self();
-		AZ(pthread_cond_init(h2->winupd_cond, NULL));
-		VTAILQ_INIT(&h2->streams);
-		VTAILQ_INIT(&h2->txqueue);
-		h2_local_settings(&h2->local_settings);
-		h2->remote_settings = H2_proto_settings;
-		h2->decode = decode;
+	/* proto_priv session attribute will always have been set up by H1
+	 * before reaching here. */
+	AZ(SES_Get_proto_priv(sp, &up));
+	assert(*up == 0);
 
-		AZ(VHT_Init(h2->dectbl,
-			h2->local_settings.header_table_size));
+	if (srq == NULL)
+		srq = Req_New(wrk, sp);
+	AN(srq);
+	h2 = h2s;
+	AN(h2);
+	INIT_OBJ(h2, H2_SESS_MAGIC);
+	h2->srq = srq;
+	h2->htc = srq->htc;
+	h2->ws = srq->ws;
+	h2->vsl = srq->vsl;
+	VSL_Flush(h2->vsl, 0);
+	h2->vsl->wid = sp->vxid;
+	h2->htc->rfd = &sp->fd;
+	h2->sess = sp;
+	h2->rxthr = pthread_self();
+	AZ(pthread_cond_init(h2->winupd_cond, NULL));
+	VTAILQ_INIT(&h2->streams);
+	VTAILQ_INIT(&h2->txqueue);
+	h2_local_settings(&h2->local_settings);
+	h2->remote_settings = H2_proto_settings;
+	h2->decode = decode;
+
+	AZ(VHT_Init(h2->dectbl, h2->local_settings.header_table_size));
+
+	*up = (uintptr_t)h2;
 
-		SES_Reserve_proto_priv(sp, &up);
-		*up = (uintptr_t)h2;
-	}
-	AN(up);
-	CAST_OBJ_NOTNULL(h2, (void*)(*up), H2_SESS_MAGIC);
 	return (h2);
 }
 
@@ -147,11 +145,12 @@ h2_del_sess(struct worker *wrk, struct h2_sess *h2, enum sess_close reason)
 	CHECK_OBJ_NOTNULL(h2, H2_SESS_MAGIC);
 	AZ(h2->refcnt);
 	assert(VTAILQ_EMPTY(&h2->streams));
+	AN(reason);
 
 	VHT_Fini(h2->dectbl);
 	AZ(pthread_cond_destroy(h2->winupd_cond));
-	req = h2->srq;
-	AZ(req->ws->r);
+	TAKE_OBJ_NOTNULL(req, &h2->srq, REQ_MAGIC);
+	assert(!WS_IsReserved(req->ws));
 	sp = h2->sess;
 	Req_Cleanup(sp, wrk, req);
 	Req_Release(req);
@@ -163,16 +162,15 @@ h2_del_sess(struct worker *wrk, struct h2_sess *h2, enum sess_close reason)
 enum htc_status_e v_matchproto_(htc_complete_f)
 H2_prism_complete(struct http_conn *htc)
 {
-	int l;
+	size_t sz;
 
 	CHECK_OBJ_NOTNULL(htc, HTTP_CONN_MAGIC);
-	l = htc->rxbuf_e - htc->rxbuf_b;
-	if (l >= sizeof(H2_prism) &&
-	    !memcmp(htc->rxbuf_b, H2_prism, sizeof(H2_prism)))
-		return (HTC_S_COMPLETE);
-	if (l < sizeof(H2_prism) && !memcmp(htc->rxbuf_b, H2_prism, l))
-		return (HTC_S_MORE);
-	return (HTC_S_JUNK);
+	sz = sizeof(H2_prism);
+	if (htc->rxbuf_b + sz > htc->rxbuf_e)
+		sz = htc->rxbuf_e - htc->rxbuf_b;
+	if (memcmp(htc->rxbuf_b, H2_prism, sz))
+		return (HTC_S_JUNK);
+	return (sz == sizeof(H2_prism) ? HTC_S_COMPLETE : HTC_S_MORE);
 }
 
 
@@ -240,7 +238,7 @@ h2_ou_rel(struct worker *wrk, struct req *req)
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 	AZ(req->vcl);
-	Req_Cleanup(req->sp, wrk, req);
+	Req_AcctLogCharge(wrk->stats, req);
 	Req_Release(req);
 	return (0);
 }
@@ -259,6 +257,7 @@ h2_ou_session(struct worker *wrk, struct h2_sess *h2,
 	}
 
 	sz = write(h2->sess->fd, h2_resp_101, strlen(h2_resp_101));
+	VTCP_Assert(sz);
 	if (sz != strlen(h2_resp_101)) {
 		VSLb(h2->vsl, SLT_Debug, "H2: Upgrade: Error writing 101"
 		    " response: %s\n", vstrerror(errno));
@@ -281,9 +280,9 @@ h2_ou_session(struct worker *wrk, struct h2_sess *h2,
 	/* Start req thread */
 	r2 = h2_new_req(wrk, h2, 1, req);
 	req->transport = &H2_transport;
-	req->req_step = R_STP_TRANSPORT;
-	req->task.func = h2_do_req;
-	req->task.priv = req;
+	assert(req->req_step == R_STP_TRANSPORT);
+	req->task->func = h2_do_req;
+	req->task->priv = req;
 	r2->scheduled = 1;
 	r2->state = H2_S_CLOS_REM; // rfc7540,l,489,491
 	req->err_code = 0;
@@ -299,7 +298,7 @@ h2_ou_session(struct worker *wrk, struct h2_sess *h2,
 		h2_del_req(wrk, r2);
 		return (0);
 	}
-	if (Pool_Task(wrk->pool, &req->task, TASK_QUEUE_REQ)) {
+	if (Pool_Task(wrk->pool, req->task, TASK_QUEUE_REQ)) {
 		r2->scheduled = 0;
 		h2_del_req(wrk, r2);
 		VSLb(h2->vsl, SLT_Debug, "H2: No Worker-threads");
@@ -369,36 +368,36 @@ h2_new_session(struct worker *wrk, void *arg)
 	}
 	assert(HTC_S_COMPLETE == H2_prism_complete(h2->htc));
 	HTC_RxPipeline(h2->htc, h2->htc->rxbuf_b + sizeof(H2_prism));
-	WS_Reset(h2->ws, 0);
+	WS_Rollback(h2->ws, 0);
 	HTC_RxInit(h2->htc, h2->ws);
-	AN(h2->ws->r);
+	AN(WS_Reservation(h2->ws));
 	VSLb(h2->vsl, SLT_Debug, "H2: Got pu PRISM");
 
 	THR_SetRequest(h2->srq);
-	AN(h2->ws->r);
+	AN(WS_Reservation(h2->ws));
 
 	l = h2_enc_settings(&h2->local_settings, settings, sizeof (settings));
-	AN(h2->ws->r);
+	AN(WS_Reservation(h2->ws));
 	H2_Send_Get(wrk, h2, h2->req0);
-	AN(h2->ws->r);
+	AN(WS_Reservation(h2->ws));
 	H2_Send_Frame(wrk, h2,
 	    H2_F_SETTINGS, H2FF_NONE, l, 0, settings);
-	AN(h2->ws->r);
+	AN(WS_Reservation(h2->ws));
 	H2_Send_Rel(h2, h2->req0);
-	AN(h2->ws->r);
+	AN(WS_Reservation(h2->ws));
 
 	/* and off we go... */
 	h2->cond = &wrk->cond;
 
 	while (h2_rxframe(wrk, h2)) {
-		WS_Reset(h2->ws, 0);
+		WS_Rollback(h2->ws, 0);
 		HTC_RxInit(h2->htc, h2->ws);
 		if (WS_Overflowed(h2->ws)) {
 			VSLb(h2->vsl, SLT_Debug, "H2: Empty Rx Workspace");
 			h2->error = H2CE_INTERNAL_ERROR;
 			break;
 		}
-		AN(h2->ws->r);
+		AN(WS_Reservation(h2->ws));
 	}
 
 	AN(h2->error);
@@ -434,8 +433,7 @@ h2_new_session(struct worker *wrk, void *arg)
 	h2->cond = NULL;
 	assert(h2->refcnt == 1);
 	h2_del_req(wrk, h2->req0);
-	/* TODO: proper sess close reason */
-	h2_del_sess(wrk, h2, SC_RX_JUNK);
+	h2_del_sess(wrk, h2, h2->error->reason);
 }
 
 struct transport H2_transport = {
