@@ -101,7 +101,7 @@ VJ_Init(const char *j_arg)
 		if (av[0] != NULL)
 			ARGV_ERR("-j argument: %s\n", av[0]);
 		if (av[1] == NULL)
-			ARGV_ERR("-j argument is emtpy\n");
+			ARGV_ERR("-j argument is empty\n");
 		vjt = MGT_Pick(vj_choice, av[1], "jail");
 		CHECK_OBJ_NOTNULL(vjt, JAIL_TECH_MAGIC);
 		(void)vjt->init(av + 2);
@@ -180,6 +180,7 @@ VJ_make_subdir(const char *dname, const char *what, struct vsb *vsb)
 	if (vjt->make_subdir != NULL)
 		return (vjt->make_subdir(dname, what, vsb));
 
+	VJ_master(JAIL_MASTER_FILE);
 	if (mkdir(dname, 0755) < 0 && errno != EEXIST) {
 		e = errno;
 		if (vsb != NULL) {
@@ -193,9 +194,32 @@ VJ_make_subdir(const char *dname, const char *what, struct vsb *vsb)
 		}
 		return (1);
 	}
+	VJ_master(JAIL_MASTER_LOW);
 	return (0);
 }
 
+void
+VJ_unlink(const char *fname)
+{
+	VJ_master(JAIL_MASTER_FILE);
+	if (unlink(fname)) {
+		fprintf(stderr, "Could not delete '%s': %s\n",
+		    fname, strerror(errno));
+	}
+	VJ_master(JAIL_MASTER_LOW);
+}
+
+void
+VJ_rmdir(const char *dname)
+{
+	VJ_master(JAIL_MASTER_FILE);
+	if (rmdir(dname)) {
+		fprintf(stderr, "Could not rmdir '%s': %s\n",
+		    dname, strerror(errno));
+	}
+	VJ_master(JAIL_MASTER_LOW);
+}
+
 void
 VJ_fix_fd(int fd, enum jail_fixfd_e what)
 {
