@@ -46,16 +46,17 @@ struct parspec {
 	const char	*def;
 	const char	*units;
 	const char	*descr;
-	int		 flags;
-#define DELAYED_EFFECT	(1<<0)
-#define EXPERIMENTAL	(1<<1)
-#define MUST_RESTART	(1<<2)
-#define MUST_RELOAD	(1<<3)
-#define WIZARD		(1<<4)
-#define PROTECTED	(1<<5)
-#define OBJ_STICKY	(1<<6)
-#define ONLY_ROOT	(1<<7)
-#define NOT_IMPLEMENTED	(1<<8)
+	unsigned	 flags;
+#define DELAYED_EFFECT		(1<<0)
+#define EXPERIMENTAL		(1<<1)
+#define MUST_RESTART		(1<<2)
+#define MUST_RELOAD		(1<<3)
+#define WIZARD			(1<<4)
+#define PROTECTED		(1<<5)
+#define OBJ_STICKY		(1<<6)
+#define ONLY_ROOT		(1<<7)
+#define NOT_IMPLEMENTED		(1<<8)
+#define PLATFORM_DEPENDENT	(1<<9)
 
 	const char	*dyn_min_reason;
 	const char	*dyn_max_reason;
@@ -65,12 +66,14 @@ struct parspec {
 	char		*dyn_def;
 };
 
-tweak_t tweak_bool;
+tweak_t tweak_boolean;
 tweak_t tweak_bytes;
 tweak_t tweak_bytes_u;
 tweak_t tweak_double;
 tweak_t tweak_poolparam;
 tweak_t tweak_string;
+tweak_t tweak_thread_pool_min;
+tweak_t tweak_thread_pool_max;
 tweak_t tweak_timeout;
 tweak_t tweak_uint;
 tweak_t tweak_vsl_buffer;
