@@ -115,14 +115,13 @@ tweak_double(struct vsb *vsb, const struct parspec *par, const char *arg)
 	volatile double *dest;
 
 	dest = par->priv;
-	return (tweak_generic_double(vsb, dest, arg,
-	    par->min, par->max, "%g"));
+	return (tweak_generic_double(vsb, dest, arg, par->min, par->max, "%g"));
 }
 
 /*--------------------------------------------------------------------*/
 
 int
-tweak_bool(struct vsb *vsb, const struct parspec *par, const char *arg)
+tweak_boolean(struct vsb *vsb, const struct parspec *par, const char *arg)
 {
 	volatile unsigned *dest;
 
@@ -378,7 +377,9 @@ tweak_string(struct vsb *vsb, const struct parspec *par, const char *arg)
 	if (arg == NULL) {
 		VSB_quote(vsb, *p, -1, 0);
 	} else if (arg == JSON_FMT) {
-		VSB_quote(vsb, *p, -1, VSB_QUOTE_JSON|VSB_QUOTE_CSTR);
+		VSB_putc(vsb, '"');
+		VSB_quote(vsb, *p, -1, VSB_QUOTE_JSON);
+		VSB_putc(vsb, '"');
 	} else {
 		REPLACE(*p, arg);
 	}
@@ -449,3 +450,37 @@ tweak_poolparam(struct vsb *vsb, const struct parspec *par, const char *arg)
 	}
 	return (retval);
 }
+
+/*--------------------------------------------------------------------
+ * Thread pool tweaks.
+ *
+ * The min/max values automatically update the opposites appropriate
+ * limit, so they don't end up crossing.
+ */
+
+int
+tweak_thread_pool_min(struct vsb *vsb, const struct parspec *par,
+    const char *arg)
+{
+	if (tweak_uint(vsb, par, arg))
+		return (-1);
+
+	MCF_ParamConf(MCF_MINIMUM, "thread_pool_max",
+	    "%u", mgt_param.wthread_min);
+	MCF_ParamConf(MCF_MAXIMUM, "thread_pool_reserve",
+	    "%u", mgt_param.wthread_min * 950 / 1000);
+	return (0);
+}
+
+int
+tweak_thread_pool_max(struct vsb *vsb, const struct parspec *par,
+    const char *arg)
+{
+
+	if (tweak_uint(vsb, par, arg))
+		return (-1);
+
+	MCF_ParamConf(MCF_MAXIMUM, "thread_pool_min",
+	    "%u", mgt_param.wthread_max);
+	return (0);
+}
