@@ -45,11 +45,14 @@
 #include "vss.h"
 #include "vtcp.h"
 
+// max. PROXY payload length (excl. sig) - XXX parameter?
+#define VPX_MAX_LEN 1024
+
 struct vpx_tlv {
 	unsigned		magic;
 #define VPX_TLV_MAGIC		0xdeb9a4a5
 	unsigned		len;
-	char			tlv[1];
+	char			tlv[];
 };
 
 static inline int
@@ -72,6 +75,7 @@ vpx_proto1(const struct worker *wrk, const struct req *req)
 	int i;
 	char *p, *q;
 	struct suckaddr *sa;
+	ssize_t sz;
 	int pfam = -1;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
@@ -118,8 +122,9 @@ vpx_proto1(const struct worker *wrk, const struct req *req)
 		return (-1);
 	}
 
-	if (! SES_Reserve_client_addr(req->sp, &sa))
+	if (! SES_Reserve_client_addr(req->sp, &sa, &sz))
 		return (vpx_ws_err(req));
+	assert (sz == vsa_suckaddr_len);
 
 	if (VSS_ResolveOne(sa, fld[1], fld[3],
 	    pfam, SOCK_STREAM, AI_NUMERICHOST | AI_NUMERICSERV) == NULL) {
@@ -132,8 +137,9 @@ vpx_proto1(const struct worker *wrk, const struct req *req)
 	if (! SES_Set_String_Attr(req->sp, SA_CLIENT_PORT, fld[3]))
 		return (vpx_ws_err(req));
 
-	if (! SES_Reserve_server_addr(req->sp, &sa))
+	if (! SES_Reserve_server_addr(req->sp, &sa, &sz))
 		return (vpx_ws_err(req));
+	assert (sz == vsa_suckaddr_len);
 
 	if (VSS_ResolveOne(sa, fld[2], fld[4],
 	    pfam, SOCK_STREAM, AI_NUMERICHOST | AI_NUMERICSERV) == NULL) {
@@ -326,13 +332,15 @@ vpx_proto2(const struct worker *wrk, struct req *req)
 	char *d, *tlv_start;
 	sa_family_t pfam = 0xff;
 	struct suckaddr *sa = NULL;
+	ssize_t sz;
 	char ha[VTCP_ADDRBUFSIZE];
 	char pa[VTCP_PORTBUFSIZE];
 	char hb[VTCP_ADDRBUFSIZE];
 	char pb[VTCP_PORTBUFSIZE];
 	struct vpx_tlv_iter vpi[1], vpi2[1];
 	struct vpx_tlv *tlv;
-	unsigned l, hdr_len, flen, alen;
+	uint16_t l;
+	unsigned hdr_len, flen, alen;
 	unsigned const plen = 2, aoff = 16;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
@@ -341,6 +349,7 @@ vpx_proto2(const struct worker *wrk, struct req *req)
 
 	assert(req->htc->rxbuf_e - req->htc->rxbuf_b >= 16L);
 	l = vbe16dec(req->htc->rxbuf_b + 14);
+	assert(l <= VPX_MAX_LEN); // vpx_complete()
 	hdr_len = l + 16L;
 	assert(req->htc->rxbuf_e >= req->htc->rxbuf_b + hdr_len);
 	HTC_RxPipeline(req->htc, req->htc->rxbuf_b + hdr_len);
@@ -409,8 +418,9 @@ vpx_proto2(const struct worker *wrk, struct req *req)
 	pp = ap + 2 * alen;
 
 	/* src/client */
-	if (! SES_Reserve_client_addr(req->sp, &sa))
+	if (! SES_Reserve_client_addr(req->sp, &sa, &sz))
 		return (vpx_ws_err(req));
+	assert(sz == vsa_suckaddr_len);
 	AN(VSA_BuildFAP(sa, pfam, ap, alen, pp, plen));
 	VTCP_name(sa, hb, sizeof hb, pb, sizeof pb);
 
@@ -418,8 +428,9 @@ vpx_proto2(const struct worker *wrk, struct req *req)
 	pp += plen;
 
 	/* dst/server */
-	if (! SES_Reserve_server_addr(req->sp, &sa))
+	if (! SES_Reserve_server_addr(req->sp, &sa, &sz))
 		return (vpx_ws_err(req));
+	assert(sz == vsa_suckaddr_len);
 	AN(VSA_BuildFAP(sa, pfam, ap, alen, pp, plen));
 	VTCP_name(sa, ha, sizeof ha, pa, sizeof pa);
 
@@ -462,8 +473,9 @@ vpx_proto2(const struct worker *wrk, struct req *req)
 	INIT_OBJ(tlv, VPX_TLV_MAGIC);
 	tlv->len = tlv_len;
 	memcpy(tlv->tlv, tlv_start, tlv_len);
-	if (! SES_Reserve_proxy_tlv(req->sp, &up))
+	if (! SES_Reserve_proxy_tlv(req->sp, &up, &sz))
 		return (vpx_ws_err(req));
+	assert(sz == sizeof up);
 	*up = (uintptr_t)tlv;
 	return (0);
 }
@@ -476,7 +488,7 @@ static enum htc_status_e v_matchproto_(htc_complete_f)
 vpx_complete(struct http_conn *htc)
 {
 	size_t z, l;
-	unsigned j;
+	uint16_t j;
 	char *p, *q;
 
 	CHECK_OBJ_NOTNULL(htc, HTTP_CONN_MAGIC);
@@ -505,6 +517,8 @@ vpx_complete(struct http_conn *htc)
 			if (l < 16)
 				return (HTC_S_MORE);
 			j = vbe16dec(p + 14);
+			if (j > VPX_MAX_LEN)
+				return (HTC_S_OVERFLOW);
 			if (l < 16L + j)
 				return (HTC_S_MORE);
 			return (HTC_S_COMPLETE);
@@ -532,12 +546,11 @@ vpx_new_session(struct worker *wrk, void *arg)
 	assert(sizeof vpx2_sig == 12);
 
 	HTC_RxInit(req->htc, req->ws);
-	hs = HTC_RxStuff(req->htc, vpx_complete,
-	    NULL, NULL, NAN, sp->t_idle + cache_param->timeout_idle, NAN,
-	    1024);			// XXX ?
+	hs = HTC_RxStuff(req->htc, vpx_complete, NULL, NULL, NAN,
+	    sp->t_idle + cache_param->timeout_idle, NAN, VPX_MAX_LEN);
 	if (hs != HTC_S_COMPLETE) {
 		Req_Release(req);
-		SES_Delete(sp, SC_RX_JUNK, NAN);
+		SES_DeleteHS(sp, hs, NAN);
 		return;
 	}
 	p = req->htc->rxbuf_b;
@@ -744,6 +757,7 @@ VPX_Send_Proxy(int fd, int version, const struct sess *sp)
 		WRONG("Wrong proxy version");
 
 	r = write(fd, VSB_data(vsb), VSB_len(vsb));
+	VTCP_Assert(r);
 
 	if (!DO_DEBUG(DBG_PROTOCOL))
 		return (r);
