@@ -4,6 +4,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -122,7 +124,7 @@ struct smp_sign {
 	uint64_t		length;		/* NB: Must be last */
 };
 
-#define SMP_SIGN_SPACE		(sizeof(struct smp_sign) + SHA256_LEN)
+#define SMP_SIGN_SPACE		(sizeof(struct smp_sign) + VSHA256_LEN)
 
 /*
  * A segment pointer.
@@ -168,7 +170,7 @@ struct smp_object {
 
 struct smp_signctx {
 	struct smp_sign		*ss;
-	struct SHA256Context	ctx;
+	struct VSHA256Context	ctx;
 	uint32_t		unique;
 	const char		*id;
 };
@@ -230,7 +232,7 @@ struct smp_sc {
 	const struct stevedore	*stevedore;
 	int			fd;
 	const char		*filename;
-	off_t			mediasize;
+	uint64_t		mediasize;
 	uintptr_t		align;
 	uint32_t		granularity;
 	uint32_t		unique;
