@@ -123,6 +123,7 @@ static int show_help = 0;
 static int help_line = 0;
 static int keep_running = 1;
 static int hide_unseen = 1;
+static int raw_vsc = 0;
 static int page_start = 0;
 static int current = 0;
 static int rebuild = 0;
@@ -256,7 +257,7 @@ sample_points(void)
 	VTAILQ_FOREACH(pt, &ptlist, list) {
 		AN(pt->vpt);
 		AN(pt->vpt->ptr);
-		v = *pt->vpt->ptr;
+		v = VSC_Value(pt->vpt);
 		if (v == 0 && !pt->seen)
 			continue;
 		if (!pt->seen) {
@@ -869,8 +870,12 @@ draw_bar_b(void)
 		    verbosity->label);
 		X -= strlen(verbosity->label) + 2;
 	}
-	if (!hide_unseen)
+	if (!hide_unseen) {
 		mvwprintw(w_bar_b, 0, X - 6, "%s", "UNSEEN");
+		X -= 8;
+	}
+	if (raw_vsc)
+		mvwprintw(w_bar_b, 0, X - 3, "%s", "RAW");
 
 	wnoutrefresh(w_bar_b);
 }
@@ -935,7 +940,7 @@ handle_common_keypress(enum kb_e kb)
 }
 
 static void
-handle_points_keypress(enum kb_e kb)
+handle_points_keypress(struct vsc *vsc, enum kb_e kb)
 {
 
 	switch (kb) {
@@ -973,6 +978,11 @@ handle_points_keypress(enum kb_e kb)
 		hide_unseen = 1 - hide_unseen;
 		rebuild = REBUILD_NEXT;
 		break;
+	case KB_RAW:
+		AN(VSC_Arg(vsc, 'r', NULL));
+		raw_vsc = VSC_IsRaw(vsc);
+		rebuild = REBUILD_NEXT;
+		break;
 	case KB_SCALE:
 		scale = 1 - scale;
 		rebuild = REBUILD_NEXT;
@@ -1046,6 +1056,7 @@ handle_help_keypress(enum kb_e kb)
 		help_line = bindings_help_len;
 		break;
 	case KB_UNSEEN:
+	case KB_RAW:
 	case KB_SCALE:
 	case KB_ACCEL:
 	case KB_DECEL:
@@ -1072,7 +1083,7 @@ handle_help_keypress(enum kb_e kb)
 }
 
 static void
-handle_keypress(int ch)
+handle_keypress(struct vsc *vsc, int ch)
 {
 	enum kb_e kb;
 
@@ -1091,7 +1102,7 @@ handle_keypress(int ch)
 	if (show_help)
 		handle_help_keypress(kb);
 	else
-		handle_points_keypress(kb);
+		handle_points_keypress(vsc, kb);
 }
 
 static void * v_matchproto_(VSC_new_f)
@@ -1104,7 +1115,7 @@ newpt(void *priv, const struct VSC_point *const vpt)
 	rebuild |= REBUILD_NEXT;
 	AN(pt);
 	pt->vpt = vpt;
-	pt->last = *pt->vpt->ptr;
+	pt->last = VSC_Value(vpt);
 	pt->ma_10.nmax = 10;
 	pt->ma_100.nmax = 100;
 	pt->ma_1000.nmax = 1000;
@@ -1166,6 +1177,7 @@ do_curses(struct vsm *vsm, struct vsc *vsc)
 
 	VSC_State(vsc, newpt, delpt, NULL);
 
+	raw_vsc = VSC_IsRaw(vsc);
 	rebuild |= REBUILD_FIRST;
 	(void)VSC_Iter(vsc, vsm, NULL, NULL);
 	build_pt_array();
@@ -1201,7 +1213,7 @@ do_curses(struct vsm *vsm, struct vsc *vsc)
 			break;
 #endif
 		default:
-			handle_keypress(ch);
+			handle_keypress(vsc, ch);
 			break;
 		}
 	}
