@@ -40,6 +40,7 @@
 #include <unistd.h>
 
 #include "vtc.h"
+#include "vtc_log.h"
 
 #include "vav.h"
 #include "vrnd.h"
@@ -89,7 +90,7 @@ static const struct cmds global_cmds[] = {
 	{ NULL, NULL }
 };
 
-static const struct cmds cmds[] = {
+static const struct cmds top_cmds[] = {
 #define CMD_TOP(n) { #n, cmd_##n },
 #include "cmds.h"
 	{ NULL, NULL }
@@ -101,7 +102,7 @@ static struct macro *
 macro_def_int(const char *name, const char *fmt, va_list ap)
 {
 	struct macro *m;
-	char buf[256];
+	char buf[512];
 
 	VTAILQ_FOREACH(m, &macro_list, list)
 		if (!strcmp(name, m->name))
@@ -218,7 +219,7 @@ macro_get(const char *b, const char *e)
 
 	if (l == 4 && !memcmp(b, "date", l)) {
 		double t = VTIM_real();
-		retval = malloc(64);
+		retval = malloc(VTIM_FORMAT_SIZE);
 		AN(retval);
 		VTIM_format(t, retval);
 		return (retval);
@@ -308,14 +309,14 @@ macro_expand(struct vtclog *vl, const char *text)
 
 
 void
-parse_string(const char *spec, const struct cmds *cmd, void *priv,
-    struct vtclog *vl)
+parse_string(struct vtclog *vl, void *priv, const char *spec)
 {
 	char *token_s[MAX_TOKENS], *token_e[MAX_TOKENS];
 	struct vsb *token_exp;
 	char *e, *p, *q, *f, *buf;
 	int nest_brace;
 	int tn;
+	unsigned n, m;
 	const struct cmds *cp;
 
 	AN(spec);
@@ -424,7 +425,26 @@ parse_string(const char *spec, const struct cmds *cmd, void *priv,
 			}
 		}
 
-		for (cp = cmd; cp->name != NULL; cp++)
+
+/* SECTION: loop loop
+ *
+ * loop NUMBER STRING
+ *         Process STRING as a specification, NUMBER times.
+ *
+ * This works inside all specification strings
+ */
+
+		if (!strcmp(token_s[0], "loop")) {
+			n = strtoul(token_s[1], NULL, 0);
+			for (m = 0; m < n; m++) {
+				vtc_log(vl, 4, "Loop #%u", m);
+				parse_string(vl, priv, token_s[2]);
+			}
+			continue;
+		}
+
+		AN(vl->cmds);
+		for (cp = vl->cmds; cp->name != NULL; cp++)
 			if (!strcmp(token_s[0], cp->name))
 				break;
 
@@ -438,7 +458,7 @@ parse_string(const char *spec, const struct cmds *cmd, void *priv,
 			vtc_fatal(vl, "Unknown command: \"%s\"", token_s[0]);
 
 		assert(cp->cmd != NULL);
-		cp->cmd(token_s, priv, cmd, vl);
+		cp->cmd(token_s, priv, vl);
 	}
 }
 
@@ -451,7 +471,7 @@ reset_cmds(const struct cmds *cmd)
 {
 
 	for (; cmd->name != NULL; cmd++)
-		cmd->cmd(NULL, NULL, NULL, NULL);
+		cmd->cmd(NULL, NULL, NULL);
 }
 
 /**********************************************************************
@@ -473,7 +493,7 @@ fail_out(void)
 		vtc_stop = 1;
 	vtc_log(vltop, 1, "RESETTING after %s", tfn);
 	reset_cmds(global_cmds);
-	reset_cmds(cmds);
+	reset_cmds(top_cmds);
 	vtc_error |= old_err;
 
 	if (vtc_error)
@@ -506,6 +526,7 @@ exec_file(const char *fn, const char *script, const char *tmpdir,
 	vtc_loginit(logbuf, loglen);
 	vltop = vtc_logopen("top");
 	AN(vltop);
+	vtc_log_set_cmd(vltop, top_cmds);
 
 	vtc_log(vltop, 1, "TEST %s starting", fn);
 
@@ -545,6 +566,6 @@ exec_file(const char *fn, const char *script, const char *tmpdir,
 	vtc_stop = 0;
 
 	vtc_thread = pthread_self();
-	parse_string(script, cmds, NULL, vltop);
+	parse_string(vltop, NULL, script);
 	return (fail_out());
 }
