@@ -106,7 +106,7 @@ client_tcp_connect(struct vtclog *vl, const char *addr, double tmo,
 		   const char **errp)
 {
 	int fd;
-	char mabuf[32], mpbuf[32];
+	char mabuf[VTCP_ADDRBUFSIZE], mpbuf[VTCP_PORTBUFSIZE];
 
 	AN(addr);
 	AN(errp);
@@ -141,7 +141,7 @@ uds_open(void *priv, const struct sockaddr_un *uds)
 	VTCP_nonblocking(s);
 	i = connect(s, (const void*)uds, sl);
 	if (i == 0)
-		return(s);
+		return (s);
 	if (errno != EINPROGRESS) {
 		closefd(&s);
 		return (-1);
@@ -233,7 +233,7 @@ client_new(const char *name)
 	ALLOC_OBJ(c, CLIENT_MAGIC);
 	AN(c);
 	REPLACE(c->name, name);
-	c->vl = vtc_logopen(name);
+	c->vl = vtc_logopen("%s", name);
 	AN(c->vl);
 	c->vsp = Sess_New(c->vl, name);
 	AN(c->vsp);
@@ -330,7 +330,6 @@ cmd_client(CMD_ARGS)
 	struct client *c, *c2;
 
 	(void)priv;
-	(void)cmd;
 
 	if (av == NULL) {
 		/* Reset and free */
