@@ -51,6 +51,7 @@
 #include "vnum.h"
 #include "vrnd.h"
 #include "vss.h"
+#include "vsa.h"
 #include "vsub.h"
 #include "vtcp.h"
 #include "vtim.h"
@@ -116,6 +117,7 @@ static int bad_backend_fd;
 
 static int cleaner_fd = -1;
 static pid_t cleaner_pid;
+const char *default_listen_addr;
 
 static struct buf *
 get_buf(void)
@@ -559,7 +561,7 @@ i_mode(void)
 	 */
 
 	VSB_clear(vsb);
-	build_path(topbuild, "lib", "libvmod_", "/.libs", vsb);
+	build_path(topbuild, "vmod", ".libs", "", vsb);
 	AZ(VSB_finish(vsb));
 	vmod_path = strdup(VSB_data(vsb));
 	AN(vmod_path);
@@ -583,6 +585,7 @@ ip_magic(void)
 	struct suckaddr *sa;
 	char abuf[VTCP_ADDRBUFSIZE];
 	char pbuf[VTCP_PORTBUFSIZE];
+	char *s;
 
 	/*
 	 * In FreeBSD jails localhost/127.0.0.1 becomes the jails IP#
@@ -594,10 +597,17 @@ ip_magic(void)
 	sa = VSS_ResolveOne(NULL, "127.0.0.1", "0", 0, SOCK_STREAM, 0);
 	AN(sa);
 	bad_backend_fd = VTCP_bind(sa, NULL);
+	if (bad_backend_fd < 0) {
+		free(sa);
+		sa = VSS_ResolveFirst(NULL, "localhost", "0", 0, SOCK_STREAM, 0);
+		AN(sa);
+		bad_backend_fd = VTCP_bind(sa, NULL);
+	}
 	assert(bad_backend_fd >= 0);
-	free(sa);
 	VTCP_myname(bad_backend_fd, abuf, sizeof abuf, pbuf, sizeof(pbuf));
 	extmacro_def("localhost", "%s", abuf);
+	s = strdup(abuf);
+	AN(s);
 
 #if defined (__APPLE__)
 	/*
@@ -609,7 +619,22 @@ ip_magic(void)
 #endif
 
 	/* Expose a backend that is forever down. */
-	extmacro_def("bad_backend", "%s %s", abuf, pbuf);
+	if (VSA_Get_Proto(sa) == AF_INET)
+		extmacro_def("bad_backend", "%s:%s", abuf, pbuf);
+	else
+		extmacro_def("bad_backend", "[%s]:%s", abuf, pbuf);
+
+	/* our default bind/listen address */
+	if (VSA_Get_Proto(sa) == AF_INET)
+		bprintf(abuf, "%s:0", s);
+	else
+		bprintf(abuf, "[%s]:0", s);
+	free(s);
+
+	extmacro_def("listen_addr", "%s", abuf);
+	default_listen_addr = strdup(abuf);
+	AN(default_listen_addr);
+	free(sa);
 
 	/*
 	 * We need an IP number which will not repond, ever, and that is a
@@ -686,6 +711,7 @@ main(int argc, char * const *argv)
 	int use_cleaner = 0;
 	uintmax_t bufsiz;
 	const char *p;
+	char buf[PATH_MAX];
 
 	argv0 = strrchr(argv[0], '/');
 	if (argv0 == NULL)
@@ -698,7 +724,7 @@ main(int argc, char * const *argv)
 	else
 		tmppath = strdup("/tmp");
 
-	cwd = getcwd(NULL, PATH_MAX);
+	cwd = getcwd(buf, sizeof buf);
 	extmacro_def("pwd", "%s", cwd);
 
 	vmod_path = NULL;
