@@ -69,7 +69,6 @@ cmd_vtest(CMD_ARGS)
 {
 
 	(void)priv;
-	(void)cmd;
 	(void)vl;
 
 	if (av == NULL)
@@ -91,7 +90,6 @@ cmd_varnishtest(CMD_ARGS)
 {
 
 	(void)priv;
-	(void)cmd;
 	(void)vl;
 
 	if (av == NULL)
@@ -215,7 +213,6 @@ cmd_shell(CMD_ARGS)
 	int ok = 0;
 
 	(void)priv;
-	(void)cmd;
 
 	if (av == NULL)
 		return;
@@ -262,7 +259,6 @@ void v_matchproto_(cmd_f)
 cmd_err_shell(CMD_ARGS)
 {
 	(void)priv;
-	(void)cmd;
 
 	if (av == NULL)
 		return;
@@ -295,7 +291,6 @@ cmd_setenv(CMD_ARGS)
 	int force;
 
 	(void)priv;
-	(void)cmd;
 
 	if (av == NULL)
 		return;
@@ -330,7 +325,6 @@ cmd_delay(CMD_ARGS)
 	double f;
 
 	(void)priv;
-	(void)cmd;
 	if (av == NULL)
 		return;
 	AN(av[1]);
@@ -371,6 +365,28 @@ dns_works(void)
 	return (0);
 }
 
+/**********************************************************************
+ * Test if IPv4/IPv6 works
+ */
+
+static int
+ipvx_works(const char *target)
+{
+	struct suckaddr *sa;
+	int fd;
+
+	sa = VSS_ResolveOne(NULL, target, "0", 0, SOCK_STREAM, 0);
+	if (sa == NULL)
+		return (0);
+	fd = VTCP_bind(sa, NULL);
+	free(sa);
+	if (fd >= 0) {
+		VTCP_close(&fd);
+		return (1);
+	}
+	return (0);
+}
+
 /* SECTION: feature feature
  *
  * Test that the required feature(s) for a test are available, and skip
@@ -381,6 +397,10 @@ dns_works(void)
  *        The SO_RCVTIMEO socket option is working
  * 64bit
  *        The environment is 64 bits
+ * ipv4
+ *	  127.0.0.1 work
+ * ipv6
+ *	  [::1] work
  * dns
  *        DNS lookups are working
  * topbuild
@@ -398,7 +418,6 @@ dns_works(void)
  * ignore_unknown_macro
  *        Do not fail the test if a string of the form ${...} is not
  *        recognized as a macro.
- *
  * persistent_storage
  *        Varnish was built with the deprecated persistent storage.
  *
@@ -420,7 +439,6 @@ cmd_feature(CMD_ARGS)
 	int good;
 
 	(void)priv;
-	(void)cmd;
 
 	if (av == NULL)
 		return;
@@ -446,6 +464,8 @@ cmd_feature(CMD_ARGS)
 #endif
 		}
 
+		FEATURE("ipv4", ipvx_works("127.0.0.1"));
+		FEATURE("ipv6", ipvx_works("[::1]"));
 		FEATURE("pcre_jit", VRE_has_jit);
 		FEATURE("64bit", sizeof(void*) == 8);
 		FEATURE("dns", dns_works());
