@@ -72,6 +72,21 @@ static pthread_mutex_t			syslog_mtx;
 static VTAILQ_HEAD(, syslog_srv)	syslogs =
     VTAILQ_HEAD_INITIALIZER(syslogs);
 
+#define SYSLOGCMDS \
+	CMD_SYSLOG(expect) \
+	CMD_SYSLOG(recv)
+
+#define CMD_SYSLOG(nm) static cmd_f cmd_syslog_##nm;
+SYSLOGCMDS
+#undef CMD_SYSLOG
+
+static const struct cmds syslog_cmds[] = {
+#define CMD_SYSLOG(n) { #n, cmd_syslog_##n },
+SYSLOGCMDS
+#undef CMD_SYSLOG
+	{ NULL, NULL }
+};
+
 static const char * const syslog_levels[] = {
 	"emerg",
 	"alert",
@@ -246,16 +261,17 @@ syslog_new(const char *name, struct vtclog *vl)
 	ALLOC_OBJ(s, SYSLOG_SRV_MAGIC);
 	AN(s);
 	REPLACE(s->name, name);
-	s->vl = vtc_logopen(s->name);
+	s->vl = vtc_logopen("%s", s->name);
 	AN(s->vl);
+	vtc_log_set_cmd(s->vl, syslog_cmds);
 
-	bprintf(s->bind, "%s", "127.0.0.1 0");
+	bprintf(s->bind, "%s", default_listen_addr);
 	s->repeat = 1;
 	s->sock = -1;
 	s->lvl = -1;
 	s->timeout = vtc_maxdur * .5;		// XXX
 
-	vl = vtc_logopen(s->name);
+	vl = vtc_logopen("%s", s->name);
 	AN(vl);
 
 	s->rxbuf_sz = s->rxbuf_left = 2048*1024;
@@ -342,6 +358,8 @@ syslog_bind(struct syslog_srv *s)
 	const char *err;
 	char aaddr[VTCP_ADDRBUFSIZE];
 	char aport[VTCP_PORTBUFSIZE];
+	char buf[vsa_suckaddr_len];
+	struct suckaddr *sua;
 
 	CHECK_OBJ_NOTNULL(s, SYSLOG_SRV_MAGIC);
 
@@ -353,10 +371,15 @@ syslog_bind(struct syslog_srv *s)
 		    "Syslog server bind address (%s) cannot be resolved: %s",
 		    s->bind, err);
 	assert(s->sock > 0);
-	VTCP_myname(s->sock, aaddr, sizeof aaddr, aport, sizeof aport);
+	sua = VSA_getsockname(s->sock, buf, sizeof buf);
+	AN(sua);
+	VTCP_name(sua, aaddr, sizeof aaddr, aport, sizeof aport);
 	macro_def(s->vl, s->name, "addr", "%s", aaddr);
 	macro_def(s->vl, s->name, "port", "%s", aport);
-	macro_def(s->vl, s->name, "sock", "%s %s", aaddr, aport);
+	if (VSA_Get_Proto(sua) == AF_INET)
+		macro_def(s->vl, s->name, "sock", "%s:%s", aaddr, aport);
+	else
+		macro_def(s->vl, s->name, "sock", "[%s]:%s", aaddr, aport);
 	/* Record the actual port, and reuse it on subsequent starts */
 	bprintf(s->bind, "%s %s", aaddr, aport);
 }
@@ -371,7 +394,6 @@ cmd_syslog_expect(CMD_ARGS)
 	char *cmp, *spec;
 
 	(void)vl;
-	(void)cmd;
 	CAST_OBJ_NOTNULL(s, priv, SYSLOG_SRV_MAGIC);
 	AZ(strcmp(av[0], "expect"));
 	av++;
@@ -407,7 +429,6 @@ cmd_syslog_recv(CMD_ARGS)
 	struct syslog_srv *s;
 
 	CAST_OBJ_NOTNULL(s, priv, SYSLOG_SRV_MAGIC);
-	(void)cmd;
 	(void)vl;
 	AZ(strcmp(av[0], "recv"));
 	av++;
@@ -419,15 +440,6 @@ cmd_syslog_recv(CMD_ARGS)
 	syslog_rx(s, lvl);
 }
 
-static const struct cmds syslog_cmds[] = {
-#define CMD_SYSLOG(n) { #n, cmd_syslog_##n },
-	/* session */
-	CMD_SYSLOG(expect)
-	CMD_SYSLOG(recv)
-#undef CMD_SYSLOG
-	{ NULL, NULL }
-};
-
 /**********************************************************************
  * Syslog server thread
  */
@@ -445,7 +457,7 @@ syslog_thread(void *priv)
 	for (i = 0; i < s->repeat; i++) {
 		if (s->repeat > 1)
 			vtc_log(s->vl, 3, "Iteration %d", i);
-		parse_string(s->spec, syslog_cmds, s, s->vl);
+		parse_string(s->vl, s, s->spec);
 		vtc_log(s->vl, 3, "shutting fd %d", s->sock);
 	}
 	VUDP_close(&s->sock);
@@ -548,7 +560,6 @@ cmd_syslog(CMD_ARGS)
 	struct syslog_srv *s;
 
 	(void)priv;
-	(void)cmd;
 
 	if (av == NULL) {
 		/* Reset and free */
