@@ -195,16 +195,6 @@ wait_running(const struct varnish *v)
  * Varnishlog gatherer thread
  */
 
-static void
-vsl_catchup(const struct varnish *v)
-{
-	int vsl_idle;
-
-	vsl_idle = v->vsl_idle;
-	while (!vtc_error && vsl_idle == v->vsl_idle)
-		VTIM_sleep(0.1);
-}
-
 static void *
 varnishlog_thread(void *priv)
 {
@@ -320,7 +310,7 @@ varnish_new(const char *name)
 
 	REPLACE(v->jail, "");
 
-	v->vl = vtc_logopen(name);
+	v->vl = vtc_logopen("%s", name);
 	AN(v->vl);
 
 	vsb = macro_expandf(v->vl, "${tmpdir}/%s", name);
@@ -402,7 +392,7 @@ varnish_launch(struct varnish *v)
 	char *r = NULL;
 
 	/* Create listener socket */
-	v->cli_fd = VTCP_listen_on("127.0.0.1:0", NULL, 1, &err);
+	v->cli_fd = VTCP_listen_on(default_listen_addr, NULL, 1, &err);
 	if (err != NULL)
 		vtc_fatal(v->vl, "Create CLI listen socket failed: %s", err);
 	assert(v->cli_fd > 0);
@@ -424,12 +414,13 @@ varnish_launch(struct varnish *v)
 	VSB_cat(vsb, " -l 2m");
 	VSB_cat(vsb, " -p auto_restart=off");
 	VSB_cat(vsb, " -p syslog_cli_traffic=off");
-	VSB_cat(vsb, " -p sigsegv_handler=on");
 	VSB_cat(vsb, " -p thread_pool_min=10");
 	VSB_cat(vsb, " -p debug=+vtc_mode");
 	VSB_cat(vsb, " -p vsl_mask=+Debug");
+	VSB_cat(vsb, " -p h2_initial_window_size=1m");
+	VSB_cat(vsb, " -p h2_rx_window_low_water=64k");
 	if (!v->has_a_arg) {
-		VSB_printf(vsb, " -a '%s'", "127.0.0.1:0");
+		VSB_printf(vsb, " -a '%s'", default_listen_addr);
 		if (v->proto != NULL)
 			VSB_printf(vsb, ",%s", v->proto);
 	}
@@ -539,20 +530,86 @@ varnish_launch(struct varnish *v)
 	AZ(pthread_create(&v->tp_vsl, NULL, varnishlog_thread, v));
 }
 
+#define VARNISH_LAUNCH(v)				\
+	do {						\
+		CHECK_OBJ_NOTNULL(v, VARNISH_MAGIC);	\
+		if (v->cli_fd < 0)			\
+			varnish_launch(v);		\
+		if (vtc_error)				\
+			return;				\
+	} while (0)
+
 /**********************************************************************
  * Start a Varnish
  */
 
+static void
+varnish_listen(const struct varnish *v, char *la)
+{
+	const char *a, *p, *n, *n2;
+	char m[64], s[256];
+	unsigned first;
+
+	n2 = "";
+	first = 1;
+
+	while (*la != '\0') {
+		n = la;
+		la = strchr(la, ' ');
+		AN(la);
+		*la = '\0';
+		a = ++la;
+		la = strchr(la, ' ');
+		AN(la);
+		*la = '\0';
+		p = ++la;
+		la = strchr(la, '\n');
+		AN(la);
+		*la = '\0';
+		la++;
+
+		AN(*n);
+		AN(*a);
+		AN(*p);
+
+		if (*p == '-') {
+			bprintf(s, "%s", a);
+			a = "0.0.0.0";
+			p = "0";
+		} else if (strchr(a, ':')) {
+			bprintf(s, "[%s]:%s", a, p);
+		} else {
+			bprintf(s, "%s:%s", a, p);
+		}
+
+		if (first) {
+			vtc_log(v->vl, 2, "Listen on %s %s", a, p);
+			macro_def(v->vl, v->name, "addr", "%s", a);
+			macro_def(v->vl, v->name, "port", "%s", p);
+			macro_def(v->vl, v->name, "sock", "%s", s);
+			first = 0;
+		}
+
+		if (!strcmp(n, n2))
+			continue;
+
+		bprintf(m, "%s_addr", n);
+		macro_def(v->vl, v->name, m, "%s", a);
+		bprintf(m, "%s_port", n);
+		macro_def(v->vl, v->name, m, "%s", p);
+		bprintf(m, "%s_sock", n);
+		macro_def(v->vl, v->name, m, "%s", s);
+		n2 = n;
+	}
+}
+
 static void
 varnish_start(struct varnish *v)
 {
 	enum VCLI_status_e u;
-	char *resp = NULL, *h, *p;
+	char *resp = NULL;
 
-	if (v->cli_fd < 0)
-		varnish_launch(v);
-	if (vtc_error)
-		return;
+	VARNISH_LAUNCH(v);
 	vtc_log(v->vl, 2, "Start");
 	u = varnish_ask_cli(v, "start", &resp);
 	if (vtc_error)
@@ -576,17 +633,7 @@ varnish_start(struct varnish *v)
 	if (u != CLIS_OK)
 		vtc_fatal(v->vl,
 		    "CLI debug.listen_address command failed: %u %s", u, resp);
-	h = resp;
-	p = strchr(h, '\n');
-	if (p != NULL)
-		*p = '\0';
-	p = strchr(h, ' ');
-	AN(p);
-	*p++ = '\0';
-	vtc_log(v->vl, 2, "Listen on %s %s", h, p);
-	macro_def(v->vl, v->name, "addr", "%s", h);
-	macro_def(v->vl, v->name, "port", "%s", p);
-	macro_def(v->vl, v->name, "sock", "%s %s", h, p);
+	varnish_listen(v, resp);
 	free(resp);
 	/* Wait for vsl logging to get underway */
 	while (v->vsl_rec == 0)
@@ -600,10 +647,8 @@ varnish_start(struct varnish *v)
 static void
 varnish_stop(struct varnish *v)
 {
-	if (v->cli_fd < 0)
-		varnish_launch(v);
-	if (vtc_error)
-		return;
+
+	VARNISH_LAUNCH(v);
 	vtc_log(v->vl, 2, "Stop");
 	(void)varnish_ask_cli(v, "stop", NULL);
 	wait_stopped(v);
@@ -674,10 +719,7 @@ varnish_cli_json(struct varnish *v, const char *cli)
 	const char *errptr;
 	struct vjsn *vj;
 
-	if (v->cli_fd < 0)
-		varnish_launch(v);
-	if (vtc_error)
-		return;
+	VARNISH_LAUNCH(v);
 	u = varnish_ask_cli(v, cli, &resp);
 	vtc_log(v->vl, 2, "CLI %03u <%s>", u, cli);
 	if (u != CLIS_OK)
@@ -703,18 +745,12 @@ varnish_cli(struct varnish *v, const char *cli, unsigned exp, const char *re)
 	const char *errptr;
 	int err;
 
+	VARNISH_LAUNCH(v);
 	if (re != NULL) {
 		vre = VRE_compile(re, 0, &errptr, &err);
 		if (vre == NULL)
 			vtc_fatal(v->vl, "Illegal regexp");
 	}
-	if (v->cli_fd < 0)
-		varnish_launch(v);
-	if (vtc_error) {
-		if (vre != NULL)
-			VRE_free(&vre);
-		return;
-	}
 	u = varnish_ask_cli(v, cli, &resp);
 	vtc_log(v->vl, 2, "CLI %03u <%s>", u, cli);
 	if (exp != 0 && exp != (unsigned)u)
@@ -738,10 +774,7 @@ varnish_vcl(struct varnish *v, const char *vcl, int fail, char **resp)
 	struct vsb *vsb;
 	enum VCLI_status_e u;
 
-	if (v->cli_fd < 0)
-		varnish_launch(v);
-	if (vtc_error)
-		return;
+	VARNISH_LAUNCH(v);
 	vsb = VSB_new_auto();
 	AN(vsb);
 
@@ -777,10 +810,7 @@ varnish_vclbackend(struct varnish *v, const char *vcl)
 	struct vsb *vsb, *vsb2;
 	enum VCLI_status_e u;
 
-	if (v->cli_fd < 0)
-		varnish_launch(v);
-	if (vtc_error)
-		return;
+	VARNISH_LAUNCH(v);
 	vsb = VSB_new_auto();
 	AN(vsb);
 
@@ -834,7 +864,7 @@ do_stat_dump_cb(void *priv, const struct VSC_point * const pt)
 
 	if (strcmp(pt->ctype, "uint64_t"))
 		return (0);
-	u = *pt->ptr;
+	u = VSC_Value(pt);
 
 	if (strcmp(dp->arg, "*")) {
 		if (fnmatch(dp->arg, pt->name, 0))
@@ -846,10 +876,11 @@ do_stat_dump_cb(void *priv, const struct VSC_point * const pt)
 }
 
 static void
-varnish_vsc(const struct varnish *v, const char *arg)
+varnish_vsc(struct varnish *v, const char *arg)
 {
 	struct dump_priv dp;
 
+	VARNISH_LAUNCH(v);
 	memset(&dp, 0, sizeof dp);
 	dp.v = v;
 	dp.arg = arg;
@@ -861,12 +892,29 @@ varnish_vsc(const struct varnish *v, const char *arg)
  * Check statistics
  */
 
-struct stat_priv {
-	char target_pattern[256];
-	uintmax_t val;
-	const struct varnish *v;
+struct stat_arg {
+	const char	*pattern;
+	uintmax_t	val;
+	unsigned	good;
 };
 
+struct stat_priv {
+	struct stat_arg	lhs;
+	struct stat_arg	rhs;
+};
+
+static int
+stat_match(const char *pattern, const char *name)
+{
+
+	if (strchr(pattern, '.') == NULL) {
+		if (fnmatch("MAIN.*", name, 0))
+			return (FNM_NOMATCH);
+		name += 5;
+	}
+	return (fnmatch(pattern, name, 0));
+}
+
 static int
 do_expect_cb(void *priv, const struct VSC_point * const pt)
 {
@@ -875,70 +923,76 @@ do_expect_cb(void *priv, const struct VSC_point * const pt)
 	if (pt == NULL)
 		return (0);
 
-	if (fnmatch(sp->target_pattern, pt->name, 0))
-		return (0);
+	if (!sp->lhs.good && stat_match(sp->lhs.pattern, pt->name) == 0) {
+		AZ(strcmp(pt->ctype, "uint64_t"));
+		AN(pt->ptr);
+		sp->lhs.val = VSC_Value(pt);
+		sp->lhs.good = 1;
+	}
 
-	AZ(strcmp(pt->ctype, "uint64_t"));
-	AN(pt->ptr);
-	sp->val = *pt->ptr;
-	return (1);
+	if (sp->rhs.pattern == NULL) {
+		sp->rhs.good = 1;
+	} else if (!sp->rhs.good &&
+	    stat_match(sp->rhs.pattern, pt->name) == 0) {
+		AZ(strcmp(pt->ctype, "uint64_t"));
+		AN(pt->ptr);
+		sp->rhs.val = VSC_Value(pt);
+		sp->rhs.good = 1;
+	}
+
+	return (sp->lhs.good && sp->rhs.good);
 }
 
 /**********************************************************************
  */
 
 static void
-varnish_expect(const struct varnish *v, char * const *av)
+varnish_expect(struct varnish *v, char * const *av)
 {
-	uint64_t ref;
-	int good;
-	char *r;
-	char *p;
-	int i, not = 0;
 	struct stat_priv sp;
+	int good, i, not;
+	uintmax_t u;
+	char *l, *p;
 
-	r = av[0];
-	if (r[0] == '!') {
-		not = 1;
-		r++;
+	VARNISH_LAUNCH(v);
+	ZERO_OBJ(&sp, sizeof sp);
+	l = av[0];
+	not = (*l == '!');
+	if (not) {
+		l++;
 		AZ(av[1]);
 	} else {
 		AN(av[1]);
 		AN(av[2]);
-	}
-	p = strrchr(r, '.');
-	if (p == NULL) {
-		bprintf(sp.target_pattern, "MAIN.%s", r);
-	} else {
-		bprintf(sp.target_pattern, "%s", r);
+		u = strtoumax(av[2], &p, 0);
+		if (u != UINTMAX_MAX && *p == '\0')
+			sp.rhs.val = u;
+		else
+			sp.rhs.pattern = av[2];
 	}
 
-	sp.val = 0;
-	sp.v = v;
-	ref = 0;
-	good = 0;
+	sp.lhs.pattern = l;
+
 	for (i = 0; i < 50; i++, (void)usleep(100000)) {
 		(void)VSM_Status(v->vsm_vsc);
+		sp.lhs.good = sp.rhs.good = 0;
 		good = VSC_Iter(v->vsc, v->vsm_vsc, do_expect_cb, &sp);
-		if (!good) {
+		if (!good)
 			good = -2;
+		if (good < 0)
 			continue;
-		}
 
 		if (not)
-			vtc_fatal(v->vl, "Found (not expected): %s", av[0]+1);
+			vtc_fatal(v->vl, "Found (not expected): %s", l);
 
-		good = 0;
-		ref = strtoumax(av[2], &p, 0);
-		if (ref == UINTMAX_MAX || *p)
-			vtc_fatal(v->vl, "Syntax error in number (%s)", av[2]);
-		if      (!strcmp(av[1], "==")) { if (sp.val == ref) good = 1; }
-		else if (!strcmp(av[1], "!=")) { if (sp.val != ref) good = 1; }
-		else if (!strcmp(av[1], ">"))  { if (sp.val > ref)  good = 1; }
-		else if (!strcmp(av[1], "<"))  { if (sp.val < ref)  good = 1; }
-		else if (!strcmp(av[1], ">=")) { if (sp.val >= ref) good = 1; }
-		else if (!strcmp(av[1], "<=")) { if (sp.val <= ref) good = 1; }
-		else
+		good = -1;
+		if (!strcmp(av[1], "==")) good = (sp.lhs.val == sp.rhs.val);
+		if (!strcmp(av[1], "!=")) good = (sp.lhs.val != sp.rhs.val);
+		if (!strcmp(av[1], ">" )) good = (sp.lhs.val >  sp.rhs.val);
+		if (!strcmp(av[1], "<" )) good = (sp.lhs.val <  sp.rhs.val);
+		if (!strcmp(av[1], ">=")) good = (sp.lhs.val >= sp.rhs.val);
+		if (!strcmp(av[1], "<=")) good = (sp.lhs.val <= sp.rhs.val);
+		if (good == -1)
 			vtc_fatal(v->vl, "comparison %s unknown", av[1]);
 		if (good)
 			break;
@@ -948,22 +1002,33 @@ varnish_expect(const struct varnish *v, char * const *av)
 	}
 	if (good == -2) {
 		if (not) {
-			vtc_log(v->vl, 2, "not found (as expected): %s",
-			    av[0] + 1);
+			vtc_log(v->vl, 2, "not found (as expected): %s", l);
 			return;
 		}
-		vtc_fatal(v->vl, "stats field %s unknown", av[0]);
+		vtc_fatal(v->vl, "stats field %s unknown",
+		    sp.lhs.good ? sp.rhs.pattern : sp.lhs.pattern);
 	}
 
 	if (good == 1) {
-		vtc_log(v->vl, 2, "as expected: %s (%ju) %s %s",
-		    av[0], sp.val, av[1], av[2]);
+		vtc_log(v->vl, 2, "as expected: %s (%ju) %s %s (%ju)",
+		    av[0], sp.lhs.val, av[1], av[2], sp.rhs.val);
 	} else {
 		vtc_fatal(v->vl, "Not true: %s (%ju) %s %s (%ju)",
-		    av[0], (uintmax_t)sp.val, av[1], av[2], (uintmax_t)ref);
+		    av[0], sp.lhs.val, av[1], av[2], sp.rhs.val);
 	}
 }
 
+static void
+vsl_catchup(struct varnish *v)
+{
+	int vsl_idle;
+
+	VARNISH_LAUNCH(v);
+	vsl_idle = v->vsl_idle;
+	while (!vtc_error && vsl_idle == v->vsl_idle)
+		VTIM_sleep(0.1);
+}
+
 /* SECTION: varnish varnish
  *
  * Define and interact with varnish instances.
@@ -1017,6 +1082,12 @@ varnish_expect(const struct varnish *v, char * const *av)
  * \-start
  *         Start the child process.
  *
+ *         Once successfully started, the following macros are available for
+ *         the default listen address: ``${vNAME_addr}``, ``${vNAME_port}``
+ *         and ``${vNAME_sock}``. Additional macros are available, including
+ *         the listen address name for each address vNAME listens to, like for
+ *         example: ``${vNAME_a0_addr}``.
+ *
  * \-stop
  *         Stop the child process.
  *
@@ -1036,11 +1107,14 @@ varnish_expect(const struct varnish *v, char * const *av)
  *         Once Varnish is stopped, clean everything after it. This is only used
  *         in very few tests and you should never need it.
  *
+ * \-expectexit NUMBER
+ *         Expect varnishd to exit(3) with this value
+ *
  * Once Varnish is started, you can talk to it (as you would through
  * ``varnishadm``) with these additional switches::
  *
  *         varnish vNAME [-cli STRING] [-cliok STRING] [-clierr STRING]
- *                       [-clijson STRING] [-expect STRING OP NUMBER]
+ *                       [-clijson STRING]
  *
  * \-cli STRING|-cliok STRING|-clierr STATUS STRING|-cliexpect REGEXP STRING
  *         All four of these will send STRING to the CLI, the only difference
@@ -1052,14 +1126,22 @@ varnish_expect(const struct varnish *v, char * const *av)
  *	   Send STRING to the CLI, expect success (CLIS_OK/200) and check
  *	   that the response is parsable JSON.
  *
- * \-expect PATTERN OP NUMBER
+ * It is also possible to interact with its shared memory (as you would
+ * through tools like ``varnishstat``) with additional switches:
+ *
+ * \-expect \!PATTERN|PATTERN OP NUMBER|PATTERN OP PATTERN
  *         Look into the VSM and make sure the first VSC counter identified by
  *         PATTERN has a correct value. OP can be ==, >, >=, <, <=. For
  *         example::
  *
  *                 varnish v1 -expect SM?.s1.g_space > 1000000
- * \-expectexit NUMBER
- *	   Expect varnishd to exit(3) with this value
+ *                 varnish v1 -expect cache_hit >= cache_hit_grace
+ *
+ *         In the \! form the test fails if a counter matches PATTERN.
+ *
+ *         The ``MAIN.`` namespace can be omitted from PATTERN.
+ *
+ *         The test takes up to 5 seconds before timing out.
  *
  * \-vsc PATTERN
  *         Dump VSC counters matching PATTERN.
@@ -1075,7 +1157,6 @@ cmd_varnish(CMD_ARGS)
 	struct varnish *v, *v2;
 
 	(void)priv;
-	(void)cmd;
 
 	if (av == NULL) {
 		/* Reset and free */
