@@ -44,7 +44,7 @@ LOCK(perpool)
 LOCK(pipestat)
 LOCK(probe)
 LOCK(sess)
-LOCK(tcp_pool)
+LOCK(conn_pool)
 LOCK(vbe)
 LOCK(vcapace)
 LOCK(vcl)
