@@ -52,6 +52,11 @@
 	    "Do all regular expression and string matching caseless."	\
 	)
 
+#define VSL_OPT_E							\
+	VOPT("E", "[-E]", "Display ESI transactions",			\
+	    "Display ESI transactions and other client transactions."	\
+	)
+
 #define VSL_OPT_i							\
 	VOPT("i:", "[-i <taglist>]", "Include tags",			\
 	    "Include log records of these tags in output. Taglist is"   \
