@@ -4,6 +4,8 @@
  *
  * Author: Tollef Fog Heen <tfheen@redpill-linpro.com>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -36,6 +38,7 @@
 #define VRE_H_INCLUDED
 
 struct vre;
+struct vsb;
 
 struct vre_limits {
 	unsigned	match;
@@ -52,10 +55,11 @@ extern const unsigned VRE_has_jit;
 extern const unsigned VRE_CASELESS;
 extern const unsigned VRE_NOTEMPTY;
 
-vre_t *VRE_compile(const char *, int, const char **, int *);
+vre_t *VRE_compile(const char *, unsigned, const char **, int *);
 int VRE_exec(const vre_t *code, const char *subject, int length,
     int startoffset, int options, int *ovector, int ovecsize,
     const volatile struct vre_limits *lim);
 void VRE_free(vre_t **);
+void VRE_quote(struct vsb *, const char *);
 
 #endif /* VRE_H_INCLUDED */
