@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -29,6 +31,8 @@
  * const char **VAV_Parse(const char *s, int *argc, int flag)
  *	Parse a command like line into an argv[]
  *	Index zero contains NULL or an error message
+ *	(The error message is a static const char* and does not
+ *	need saving or copying.)
  *	"double quotes" and backslash substitution is handled.
  *
  * void VAV_Free(const char **argv)
