@@ -6,6 +6,8 @@
  * Author: Dag-Erling Smørgrav <des@des.no>
  * Author: Cecilie Fritzvold <cecilihf@linpro.no>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -46,14 +48,15 @@
 
 /*
  * Take a string provided by the user and break it up into address and
- * port parts.  Examples of acceptable input include:
+ * port parts. The address and port separator may be either a colon or
+ * a whitespace. Examples of acceptable input include:
  *
- * "localhost" - "localhost:80"
- * "127.0.0.1" - "127.0.0.1:80"
- * "0.0.0.0" - "0.0.0.0:80"
- * "[::1]" - "[::1]:80"
- * "[::]" - "[::]:80"
- * "::1" - "[::1]:80"
+ * "localhost" - "localhost:80" - "localhost 80"
+ * "127.0.0.1" - "127.0.0.1:80" - "127.0.0.1 80"
+ * "0.0.0.0"   - "0.0.0.0:80"   - "0.0.0.0 80"
+ * "[::1]"     - "[::1]:80"     - "[::1] 80"
+ * "[::]"      - "[::]:80"      - "[::] 80"
+ * "::1"       - "[::1]:80"     - "[::1] 80"
  *
  * See also RFC5952
  */
@@ -97,6 +100,56 @@ vss_parse(char *str, char **addr, char **port)
 	return (NULL);
 }
 
+static int
+vss_resolve(const char *addr, const char *def_port, int family, int socktype,
+    int flags, struct addrinfo **res, const char **errp)
+{
+	struct addrinfo hints;
+	char *p, *hp, *pp;
+	int ret;
+
+	AN(addr);
+	AN(res);
+	AZ(*res);
+	AN(errp);
+	*errp = NULL;
+
+	memset(&hints, 0, sizeof hints);
+	hints.ai_family = family;
+	hints.ai_socktype = socktype;
+	hints.ai_flags = flags;
+
+	p = strdup(addr);
+	AN(p);
+	*errp = vss_parse(p, &hp, &pp);
+	if (*errp != NULL) {
+		free(p);
+		return (-1);
+	}
+	if (pp != NULL)
+		def_port = pp;
+	ret = getaddrinfo(hp, def_port, &hints, res);
+	free(p);
+
+	if (ret == EAI_SYSTEM)
+		*errp = vstrerror(errno);
+	else if (ret != 0)
+		*errp = gai_strerror(ret);
+
+	return (ret);
+}
+
+static struct suckaddr *
+vss_alloc_suckaddr(void *dst, const struct addrinfo *ai)
+{
+
+	AN(ai);
+	if (dst == NULL)
+		return (VSA_Malloc(ai->ai_addr, ai->ai_addrlen));
+
+	return (VSA_Build(dst, ai->ai_addr, ai->ai_addrlen));
+}
+
 /*
  * Look up an address, using a default port if provided, and call
  * the callback function with the suckaddrs we find.
@@ -106,34 +159,21 @@ vss_parse(char *str, char **addr, char **port)
 
 int
 VSS_resolver_socktype(const char *addr, const char *def_port,
-    vss_resolved_f *func, void *priv, const char **err, int socktype)
+    vss_resolved_f *func, void *priv, const char **errp, int socktype)
 {
-	struct addrinfo hints, *res0, *res;
+	struct addrinfo *res0 = NULL, *res;
 	struct suckaddr *vsa;
-	char *h;
-	char *adp, *hop;
 	int ret;
 
-	*err = NULL;
-	h = strdup(addr);
-	AN(h);
-	*err = vss_parse(h, &hop, &adp);
-	if (*err != NULL) {
-		free(h);
-		return (-1);
-	}
-	if (adp != NULL)
-		def_port = adp;
+	AN(addr);
+	AN(func);
+	AN(errp);
 
-	memset(&hints, 0, sizeof hints);
-	hints.ai_socktype = socktype;
-	hints.ai_flags = AI_PASSIVE;
-	ret = getaddrinfo(hop, def_port, &hints, &res0);
-	free(h);
-	if (ret != 0) {
-		*err = gai_strerror(ret);
+	ret = vss_resolve(addr, def_port, AF_UNSPEC, socktype, AI_PASSIVE,
+	    &res0, errp);
+	if (ret != 0)
 		return (-1);
-	}
+
 	for (res = res0; res != NULL; res = res->ai_next) {
 		vsa = VSA_Malloc(res->ai_addr, res->ai_addrlen);
 		if (vsa != NULL) {
@@ -149,8 +189,52 @@ VSS_resolver_socktype(const char *addr, const char *def_port,
 
 int
 VSS_resolver(const char *addr, const char *def_port, vss_resolved_f *func,
-    void *priv, const char **err)
+    void *priv, const char **errp)
 {
 	return (VSS_resolver_socktype(
-	    addr, def_port, func, priv, err, SOCK_STREAM));
+	    addr, def_port, func, priv, errp, SOCK_STREAM));
+}
+
+struct suckaddr *
+VSS_ResolveOne(void *dst, const char *addr, const char *def_port,
+    int family, int socktype, int flags)
+{
+	struct addrinfo *res = NULL;
+	struct suckaddr *retval = NULL;
+	const char *err;
+	int ret;
+
+	AN(addr);
+	ret = vss_resolve(addr, def_port, family, socktype, flags, &res, &err);
+	if (ret == 0 && res != NULL && res->ai_next == NULL) {
+		AZ(err);
+		retval = vss_alloc_suckaddr(dst, res);
+	}
+	if (res != NULL)
+		freeaddrinfo(res);
+	return (retval);
+}
+
+struct suckaddr *
+VSS_ResolveFirst(void *dst, const char *addr, const char *def_port,
+    int family, int socktype, int flags)
+{
+	struct addrinfo *res0 = NULL, *res;
+	struct suckaddr *retval = NULL;
+	const char *err;
+	int ret;
+
+	AN(addr);
+	ret = vss_resolve(addr, def_port, family, socktype, flags, &res0, &err);
+	if (ret == 0)
+		AZ(err);
+
+	for (res = res0; res != NULL; res = res->ai_next) {
+		retval = vss_alloc_suckaddr(dst, res);
+		if (retval != NULL)
+			break;
+	}
+	if (res0 != NULL)
+		freeaddrinfo(res0);
+	return (retval);
 }
