@@ -48,8 +48,10 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
+#include <stdlib.h>
 
 #include "vdef.h"
+#include "miniobj.h"
 #include "vas.h"
 #include "vsa.h"
 #include "vss.h"
@@ -72,7 +74,9 @@ vtcp_sa_to_ascii(const void *sa, socklen_t l, char *abuf, unsigned alen,
 		 * XXX this printf is shitty, but we may not have space
 		 * for the gai_strerror in the bufffer :-(
 		 */
-		printf("getnameinfo = %d %s\n", i, gai_strerror(i));
+		fprintf(stderr, "getnameinfo = %d %s\n", i, gai_strerror(i));
+		if (i == EAI_SYSTEM)
+			fprintf(stderr, "errno = %d %s\n", errno, vstrerror(errno));
 		if (abuf != NULL)
 			(void)snprintf(abuf, alen, "Conversion");
 		if (pbuf != NULL)
@@ -97,6 +101,7 @@ VTCP_name(const struct suckaddr *addr, char *abuf, unsigned alen,
 	socklen_t sl;
 
 	sa = VSA_Get_Sockaddr(addr, &sl);
+	AN(sa);
 	vtcp_sa_to_ascii(sa, sl, abuf, alen, pbuf, plen);
 }
 
@@ -105,12 +110,11 @@ VTCP_name(const struct suckaddr *addr, char *abuf, unsigned alen,
 struct suckaddr *
 VTCP_my_suckaddr(int sock)
 {
-	struct sockaddr_storage addr_s;
-	socklen_t l;
+	struct suckaddr *r;
 
-	l = sizeof addr_s;
-	AZ(getsockname(sock, (void *)&addr_s, &l));
-	return (VSA_Malloc(&addr_s, l));
+	r = malloc(vsa_suckaddr_len);
+	AN(VSA_getsockname(sock, r, vsa_suckaddr_len));
+	return (r);
 }
 
 /*--------------------------------------------------------------------*/
@@ -118,12 +122,10 @@ VTCP_my_suckaddr(int sock)
 void
 VTCP_myname(int sock, char *abuf, unsigned alen, char *pbuf, unsigned plen)
 {
-	struct sockaddr_storage addr_s;
-	socklen_t l;
+	char buf[vsa_suckaddr_len];
 
-	l = sizeof addr_s;
-	AZ(getsockname(sock, (void *)&addr_s, &l));
-	vtcp_sa_to_ascii(&addr_s, l, abuf, alen, pbuf, plen);
+	VTCP_name(VSA_getsockname(sock, buf, sizeof buf),
+		  abuf, alen, pbuf, plen);
 }
 
 /*--------------------------------------------------------------------*/
@@ -131,12 +133,12 @@ VTCP_myname(int sock, char *abuf, unsigned alen, char *pbuf, unsigned plen)
 void
 VTCP_hisname(int sock, char *abuf, unsigned alen, char *pbuf, unsigned plen)
 {
-	struct sockaddr_storage addr_s;
-	socklen_t l;
+	char buf[vsa_suckaddr_len];
+	struct suckaddr *sua;
 
-	l = sizeof addr_s;
-	if (!getpeername(sock, (void*)&addr_s, &l))
-		vtcp_sa_to_ascii(&addr_s, l, abuf, alen, pbuf, plen);
+	sua = VSA_getpeername(sock, buf, sizeof buf);
+	if (sua != NULL)
+		VTCP_name(sua, abuf, alen, pbuf, plen);
 	else {
 		(void)snprintf(abuf, alen, "<none>");
 		(void)snprintf(pbuf, plen, "<none>");
@@ -192,8 +194,7 @@ VTCP_filter_http(int sock)
 int
 VTCP_fastopen(int sock, int depth)
 {
-	return (setsockopt(sock, SOL_TCP, TCP_FASTOPEN,
-	    &depth, sizeof depth));
+	return (setsockopt(sock, SOL_TCP, TCP_FASTOPEN, &depth, sizeof depth));
 }
 
 #else
@@ -344,7 +345,7 @@ VTCP_close(int *s)
 
 	i = close(*s);
 
-	assert(VTCP_Check(i));
+	VTCP_Assert(i);
 	*s = -1;
 }
 
@@ -370,25 +371,50 @@ VTCP_set_read_timeout(int s, vtim_dur seconds)
 /*--------------------------------------------------------------------
  */
 
+struct vto_priv {
+	unsigned		magic;
+#define VTO_PRIV_MAGIC		0xca70b0e7
+	int			latest_errno;
+	int			fd;
+	double			timeout;
+};
+
 static int v_matchproto_(vss_resolved_f)
 vtcp_open_callback(void *priv, const struct suckaddr *sa)
 {
-	/* XXX: vtim_dur? */
-	double *p = priv;
+	struct vto_priv *vto;
+	int fd;
 
-	return (VTCP_connect(sa, (int)floor(*p * 1e3)));
+	CAST_OBJ_NOTNULL(vto, priv, VTO_PRIV_MAGIC);
+
+	errno = 0;
+	fd = VTCP_connect(sa, (int)floor(vto->timeout * 1e3));
+	if (fd >= 0) {
+		vto->fd = fd;
+		vto->latest_errno = 0;
+		return (1);
+	}
+	vto->latest_errno = errno;
+	return (0);
 }
 
 int
 VTCP_open(const char *addr, const char *def_port, vtim_dur timeout,
     const char **errp)
 {
+	struct vto_priv vto[1];
 
 	AN(errp);
 	assert(timeout >= 0);
+	INIT_OBJ(vto, VTO_PRIV_MAGIC);
+	vto->fd = -1;
+	vto->timeout = timeout;
 
-	return (VSS_resolver(addr, def_port, vtcp_open_callback,
-	    &timeout, errp));
+	if (VSS_resolver(addr, def_port, vtcp_open_callback, vto, errp) < 0)
+		return (-1);
+	if (vto->fd < 0)
+		*errp = strerror(vto->latest_errno);
+	return (vto->fd);
 }
 
 /*--------------------------------------------------------------------
@@ -558,28 +584,47 @@ VTCP_check_hup(int sock)
  */
 
 int
-VTCP_Check(int a)
+VTCP_Check(ssize_t a)
 {
 	if (a == 0)
 		return (1);
+	if (a > 0)
+		return (1);
 	if (errno == ECONNRESET || errno == ENOTCONN || errno == EPIPE)
 		return (1);
+	/* Accept EAGAIN (and EWOULDBLOCK in case they are not the same)
+	 * as errno values. Even though our sockets are all non-blocking,
+	 * when a SO_{SND|RCV}TIMEO expires, read() or write() on the
+	 * socket will return (-1) and errno set to EAGAIN. (This is not
+	 * documented in the read(2) and write(2) manpages, but is
+	 * described in the socket(7) manpage.) */
+	if (errno == EAGAIN || errno == EWOULDBLOCK)
+		return (1);
+	/* tcp(7): The other end didn't acknowledge retransmitted data after
+	 * some time. */
+	if (errno == ETIMEDOUT)
+		return (1);
+	/* #3539 various errnos documented on linux as POSIX.1 */
+	if (errno == ENETDOWN || errno == ENETUNREACH || errno == ENETRESET ||
+	    errno == ECONNABORTED || /* ECONNRESET see above */
+	    errno == EHOSTUNREACH || errno == EHOSTDOWN) {
+		return (1);
+	}
+
 #if (defined (__SVR4) && defined (__sun))
+	if (errno == ECONNREFUSED)	// in r02702.vtc
+		return (1);
 	if (errno == EPROTO)
 		return (1);
 #endif
-#if (defined (__SVR4) && defined (__sun)) || defined (__NetBSD__)
+#if (defined (__SVR4) && defined (__sun)) ||		\
+    defined (__NetBSD__) ||				\
+    defined (__APPLE__)
 	/*
-	 * Solaris returns EINVAL if the other end unexpectedly reset the
-	 * connection.
-	 * This is a bug in Solaris and documented behaviour on NetBSD.
-	 */
-	if (errno == EINVAL || errno == ETIMEDOUT)
-		return (1);
-#elif defined (__APPLE__)
-	/*
-	 * MacOS returns EINVAL if the other end unexpectedly reset
+	 * Solaris and MacOS returns EINVAL if the other end unexpectedly reset
 	 * the connection.
+	 *
+	 * On NetBSD it is documented behaviour.
 	 */
 	if (errno == EINVAL)
 		return (1);
@@ -609,5 +654,6 @@ VTCP_read(int fd, void *ptr, size_t len, vtim_dur tmo)
 			return (-2);
 	}
 	i = read(fd, ptr, len);
+	VTCP_Assert(i);
 	return (i < 0 ? -1 : i);
 }
