@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -414,8 +416,13 @@ VTIM_sleep(vtim_dur t)
 #endif
 }
 
+/*
+ * VTIM_timeval and VTIM_timespec may need variants with different signatures
+ * when vtim_real / vtim_mono typedefs are changed
+ */
+
 struct timeval
-VTIM_timeval(vtim_real t)
+VTIM_timeval(vtim_dur t)
 {
 	struct timeval tv;
 
@@ -426,7 +433,7 @@ VTIM_timeval(vtim_real t)
 }
 
 struct timespec
-VTIM_timespec(vtim_real t)
+VTIM_timespec(vtim_dur t)
 {
 	struct timespec tv;
 
