@@ -335,8 +335,7 @@ vsl_R_arg(struct VSL_data *vsl, const char *arg)
 	strcat(buf, p);
 	vsl->R_opt_p = VNUM_duration(buf);
 	if (isnan(vsl->R_opt_p) || vsl->R_opt_p <= 0.0)
-		return (vsl_diag(vsl,
-			"-R: Syntax error: Invalid duration"));
+		return (vsl_diag(vsl, "-R: Syntax error: Invalid duration"));
 	return (1);
 }
 
@@ -361,6 +360,10 @@ VSL_Arg(struct VSL_data *vsl, int opt, const char *arg)
 		/* Caseless regular expressions */
 		vsl->C_opt = 1;
 		return (1);
+	case 'E':
+		vsl->E_opt = 1;
+		vsl->c_opt = 1;
+		return (1);
 	case 'i': case 'x': return (vsl_ix_arg(vsl, opt, arg));
 	case 'I': case 'X': return (vsl_IX_arg(vsl, opt, arg));
 	case 'L':
