@@ -5,6 +5,8 @@
  *
  * Author: Martin Blix Grydeland <martin@varnish-software.com>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -189,7 +191,7 @@ struct VSLQ {
 	struct vtx_tree		tree;
 	VTAILQ_HEAD(,vtx)	ready;
 	VTAILQ_HEAD(,vtx)	incomplete;
-	unsigned		n_outstanding;
+	int			n_outstanding;
 	struct chunkhead	shmrefs;
 	VTAILQ_HEAD(,vtx)	cache;
 	unsigned		n_cache;
@@ -205,7 +207,7 @@ struct VSLQ {
 		struct VSL_transaction	*ptrans[2];
 		struct VSLC_ptr		start;
 		ssize_t			len;
-		size_t			offset;
+		ssize_t			offset;
 	} raw;
 };
 
@@ -274,6 +276,7 @@ vslc_vtx_next(const struct VSL_cursor *cursor)
 {
 	struct vslc_vtx *c;
 	const uint32_t *ptr;
+	unsigned overrun;
 
 	CAST_OBJ_NOTNULL(c, cursor->priv_data, VSLC_VTX_MAGIC);
 	assert(&c->cursor == cursor);
@@ -287,7 +290,8 @@ vslc_vtx_next(const struct VSL_cursor *cursor)
 			c->cursor.rec.ptr = c->synth->data;
 			c->synth = VTAILQ_NEXT(c->synth, list);
 		} else {
-			assert(c->offset <= c->vtx->len);
+			overrun = c->offset > c->vtx->len;
+			AZ(overrun);
 			if (c->offset == c->vtx->len)
 				return (vsl_end);
 
@@ -427,7 +431,7 @@ chunk_shm_to_buf(struct VSLQ *vslq, struct chunk *chunk)
 }
 
 /* Append a set of records to a vtx structure */
-static void
+static enum vsl_status
 vtx_append(struct VSLQ *vslq, struct vtx *vtx, const struct VSLC_ptr *start,
     size_t len)
 {
@@ -435,11 +439,13 @@ vtx_append(struct VSLQ *vslq, struct vtx *vtx, const struct VSLC_ptr *start,
 	enum vsl_check i;
 
 	AN(vtx);
-	if (len == 0)
-		return;
+	AN(len);
 	AN(start);
 
 	i = VSL_Check(vslq->c, start);
+	if (i == vsl_check_e_inval)
+		return (vsl_e_overrun);
+
 	if (i == vsl_check_valid && !VTAILQ_EMPTY(&vtx->shmchunks_free)) {
 		/* Shmref it */
 		chunk = VTAILQ_FIRST(&vtx->shmchunks_free);
@@ -454,7 +460,6 @@ vtx_append(struct VSLQ *vslq, struct vtx *vtx, const struct VSLC_ptr *start,
 		/* Append to shmref list */
 		VTAILQ_INSERT_TAIL(&vslq->shmrefs, chunk, shm.shmref);
 	} else {
-		assert(i != vsl_check_e_inval);
 		/* Buffer it */
 		chunk = VTAILQ_LAST(&vtx->chunks, chunkhead);
 		CHECK_OBJ_ORNULL(chunk, CHUNK_MAGIC);
@@ -469,6 +474,7 @@ vtx_append(struct VSLQ *vslq, struct vtx *vtx, const struct VSLC_ptr *start,
 		}
 	}
 	vtx->len += len;
+	return (vsl_more);
 }
 
 /* Allocate a new vtx structure */
@@ -1088,7 +1094,7 @@ VSLQ_New(struct VSL_data *vsl, struct VSL_cursor **cp,
 	struct VSLQ *vslq;
 
 	CHECK_OBJ_NOTNULL(vsl, VSL_MAGIC);
-	if (grouping > VSL_g_session) {
+	if (grouping >= VSL_g__MAX) {
 		(void)vsl_diag(vsl, "Illegal query grouping");
 		return (NULL);
 	}
@@ -1270,15 +1276,60 @@ vslq_shmref_check(struct VSLQ *vslq)
 	return (vsl_more);
 }
 
+static unsigned
+vslq_candidate(struct VSLQ *vslq, const uint32_t *ptr)
+{
+	enum VSL_transaction_e type;
+	enum VSL_reason_e reason;
+	struct VSL_data *vsl;
+	enum VSL_tag_e tag;
+	unsigned p_vxid;
+	int i;
+
+	CHECK_OBJ_NOTNULL(vslq, VSLQ_MAGIC);
+	AN(ptr);
+
+	assert(vslq->grouping != VSL_g_raw);
+	if (vslq->grouping == VSL_g_session)
+		return (1); /* All are needed */
+
+	vsl = vslq->vsl;
+	CHECK_OBJ_NOTNULL(vsl, VSL_MAGIC);
+	if (vslq->grouping == VSL_g_vxid) {
+		if (!vsl->c_opt && !vsl->b_opt)
+			return (1); /* Implies also !vsl->E_opt */
+		if (!vsl->b_opt && !VSL_CLIENT(ptr))
+			return (0);
+		if (!vsl->c_opt && !VSL_BACKEND(ptr))
+			return (0);
+		/* Need to parse the Begin tag - fallthrough to below */
+	}
+
+	tag = VSL_TAG(ptr);
+	assert(tag == SLT_Begin);
+	i = vtx_parse_link(VSL_CDATA(ptr), &type, &p_vxid, &reason);
+	if (i != 3 || type == VSL_t_unknown)
+		return (0);
+
+	if (type == VSL_t_sess)
+		return (0);
+
+	if (vslq->grouping == VSL_g_vxid && reason == VSL_r_esi && !vsl->E_opt)
+		return (0);
+
+	return (1);
+}
+
 /* Process next input record */
 static enum vsl_status
 vslq_next(struct VSLQ *vslq)
 {
+	const uint32_t *ptr;
 	struct VSL_cursor *c;
 	enum vsl_status r;
 	enum VSL_tag_e tag;
 	ssize_t len;
-	unsigned vxid;
+	unsigned vxid, keep;
 	struct vtx *vtx;
 
 	c = vslq->c;
@@ -1295,10 +1346,12 @@ vslq_next(struct VSLQ *vslq)
 		    c->rec.ptr;
 		if (len == 0)
 			return (r);
-		tag = (enum VSL_tag_e)VSL_TAG(VSL_NEXT(c->rec.ptr));
+		ptr = VSL_NEXT(c->rec.ptr);
+		tag = (enum VSL_tag_e)VSL_TAG(ptr);
 	} else {
 		vxid = VSL_ID(c->rec.ptr);
 		len = VSL_NEXT(c->rec.ptr) - c->rec.ptr;
+		ptr = c->rec.ptr;
 	}
 	assert(len > 0);
 	if (vxid == 0)
@@ -1306,13 +1359,16 @@ vslq_next(struct VSLQ *vslq)
 		return (r);
 
 	vtx = vtx_lookup(vslq, vxid);
-	if (vtx == NULL && tag == SLT_Begin) {
+	keep = tag != SLT_Begin || vslq_candidate(vslq, ptr);
+	if (vtx == NULL && tag == SLT_Begin && keep) {
 		vtx = vtx_add(vslq, vxid);
 		AN(vtx);
 	}
 	if (vtx != NULL) {
-		vtx_append(vslq, vtx, &c->rec, len);
-		vtx_scan(vslq, vtx);
+		AN(keep);
+		r = vtx_append(vslq, vtx, &c->rec, len);
+		if (r == vsl_more)
+			vtx_scan(vslq, vtx);
 	}
 
 	return (r);
