@@ -431,21 +431,16 @@ def emit_vcl_tnames(fo, tokens):
 
 
 #######################################################################
-def emit_file(fo, fd, bn):
-    "Read a C-source file and spit out code that outputs it with VSB_cat()"
-    fn = join(fd, bn)
+def emit_strings(fo, name, fc):
+    "spit out code that outputs the fc iterable with VSB_cat()"
 
-    fi = open(fn)
-    fc = fi.read()
-    fi.close()
-
-    w = 66      # Width of lines, after white space prefix
+    w = 66          # Width of lines, after white space prefix
     maxlen = 10240  # Max length of string literal
 
     x = 0
     l = 0
-    fo.write("\n\t/* %s */\n\n" % fn)
-    fo.write('\tVSB_cat(sb, "/* ---===### %s ###===--- */\\n\\n");\n' % bn)
+
+    fo.write('\tVSB_cat(sb, "/* ---===### %s ###===--- */\\n\\n");\n' % name)
     for c in fc:
         if l == 0:
             fo.write("\tVSB_cat(sb, \"")
@@ -488,6 +483,18 @@ def emit_file(fo, fd, bn):
         fo.write("\t);\n")
     fo.write('\tVSB_cat(sb, "\\n");\n')
 
+
+def emit_file(fo, fd, bn):
+    "Read a C-source file and spit out code that outputs it with VSB_cat()"
+    fn = join(fd, bn)
+
+    fi = open(fn)
+    fc = fi.read()
+    fi.close()
+
+    fo.write("\n\t/* %s */\n\n" % fn)
+    emit_strings(fo, bn, fc)
+
 #######################################################################
 
 
@@ -625,6 +632,10 @@ for i in sorted(task.keys()):
     fo.write(tbl40("#define VCL_MET_TASK_%s" % i.upper(),
                    "( " + (" | \\\n\t\t\t\t\t  ").join(task[i]) + " )\n"))
 
+fo.write("\n")
+fo.write(tbl40("#define VCL_MET_TASK_ALL", "( VCL_MET_TASK_"))
+fo.write(" | \\\n\t\t\t\t\t  VCL_MET_TASK_".join(map(str.upper, task.keys())))
+fo.write(" )")
 
 fo.write("\n/* VCL Returns */\n")
 n = 1
@@ -635,6 +646,17 @@ for i in sorted(rets.keys()):
 fo.write("\n" + tbl40("#define VCL_RET_MAX", "%d\n" % n))
 
 fo.write("\n/* VCL Types */\n")
+fo.write('''
+struct vrt_type {
+\tunsigned\t\tmagic;
+#define VRT_TYPE_MAGIC\t\t0xa943bc32
+\tconst char\t\t*lname;
+\tconst char\t\t*uname;
+\tconst char\t\t*ctype;
+\tsize_t\t\t\tszof;
+};
+
+''')
 for vcltype in sorted(vcltypes.keys()):
     fo.write("extern const struct vrt_type VCL_TYPE_%s[1];\n" % vcltype)
 
@@ -644,7 +666,6 @@ fo.write("""
 typedef int vcl_event_f(VRT_CTX, enum vcl_event_e);
 typedef int vcl_init_f(VRT_CTX);
 typedef void vcl_fini_f(VRT_CTX);
-typedef void vcl_func_f(VRT_CTX);
 
 struct VCL_conf {
 	unsigned		magic;
@@ -657,6 +678,7 @@ struct VCL_conf {
 	const struct vpi_ref	*ref;
 
 	int			nsrc;
+	unsigned		nsub;
 	const char		**srcname;
 	const char		**srcbody;
 
@@ -784,6 +806,14 @@ emit_file(fo, srcroot, "include/vrt.h")
 emit_file(fo, buildroot, "include/vcl.h")
 emit_file(fo, buildroot, "include/vrt_obj.h")
 emit_file(fo, srcroot, "include/vcc_interface.h")
+emit_strings(fo, "vgc asserts (generate.py)",
+'''#define assert(e)							\\
+do {									\\
+	if (!(e)) {							\\
+		VPI_Fail(__func__, __FILE__, __LINE__, #e);		\\
+	}								\\
+} while (0)
+''')
 
 fo.write("\n}\n")
 fo.close()
