@@ -138,7 +138,7 @@ vcc_acl_chk(struct vcc *tl, const struct acl_e *ae, const int l,
 	AN(sa);
 	VTCP_name(sa, h, sizeof h, NULL, 0);
 	bprintf(t, "%s/%d", h, ae->mask);
-	VSB_printf(tl->sb, "Address/Netmask mismatch, ");
+	VSB_cat(tl->sb, "Address/Netmask mismatch, ");
 	if (tl->acl_pedantic != 0)
 		VSB_printf(tl->sb, "need be %s\n", t);
 	else
@@ -194,12 +194,15 @@ vcc_acl_add_entry(struct vcc *tl, const struct acl_e *ae, int l,
 			 * If the two rules agree, silently ignore it
 			 * XXX: is that counter intuitive ?
 			 */
-			if (aen->not == ae2->not)
+			if (aen->not == ae2->not) {
+				free(aen);
 				return;
+			}
 			VSB_cat(tl->sb, "Conflicting ACL entries:\n");
 			vcc_ErrWhere(tl, ae2->t_addr);
 			VSB_cat(tl->sb, "vs:\n");
 			vcc_ErrWhere(tl, aen->t_addr);
+			free(aen);
 			return;
 		}
 		/*
@@ -277,6 +280,7 @@ vcc_acl_try_getaddrinfo(struct vcc *tl, struct acl_e *ae)
 		    "Mask (/%u) specified, but string resolves to"
 		    " both IPv4 and IPv6 addresses.\n", ae->mask);
 		vcc_ErrWhere(tl, ae->t_mask);
+		freeaddrinfo(res0);
 		return;
 	}
 
@@ -386,12 +390,14 @@ vcc_acl_entry(struct vcc *tl)
 		if (*e != '\0') {
 			VSB_cat(tl->sb, ".../mask is not numeric.\n");
 			vcc_ErrWhere(tl, ae->t_addr);
+			free(ae);
 			return;
 		}
 		ae->t_mask = ae->t_addr;
 		if (tl->t->tok == '/') {
 			VSB_cat(tl->sb, "/mask only allowed once.\n");
 			vcc_ErrWhere(tl, tl->t);
+			free(ae);
 			return;
 		}
 	} else if (tl->t->tok == '/') {
@@ -439,7 +445,7 @@ vcc_acl_emit(struct vcc *tl, const char *name, const char *rname, int anon)
 	Fh(tl, 0, "\n");
 	Fh(tl, 0, "\tfam = VRT_VSA_GetPtr(ctx, p, &a);\n");
 	Fh(tl, 0, "\tif (fam < 0) {\n");
-	Fh(tl, 0, "\t\tVRT_acl_log(ctx, \"NO_FAM %s\");\n", name);
+	Fh(tl, 0, "\t\tVPI_acl_log(ctx, \"NO_FAM %s\");\n", name);
 	Fh(tl, 0, "\t\treturn(0);\n");
 	Fh(tl, 0, "\t}\n\n");
 	if (!tl->err_unref && !anon) {
@@ -493,7 +499,7 @@ vcc_acl_emit(struct vcc *tl, const char *name, const char *rname, int anon)
 		i = ((int)ae->mask + 7) / 8;
 
 		if (!anon) {
-			Fh(tl, 0, "\t%*sVRT_acl_log(ctx, \"%sMATCH %s \" ",
+			Fh(tl, 0, "\t%*sVPI_acl_log(ctx, \"%sMATCH %s \" ",
 			    -i, "", ae->not ? "NEG_" : "", name);
 			t = ae->t_addr;
 			do {
@@ -523,7 +529,7 @@ vcc_acl_emit(struct vcc *tl, const char *name, const char *rname, int anon)
 
 	/* Deny by default */
 	if (!anon)
-		Fh(tl, 0, "\tVRT_acl_log(ctx, \"NO_MATCH %s\");\n", name);
+		Fh(tl, 0, "\tVPI_acl_log(ctx, \"NO_MATCH %s\");\n", name);
 	Fh(tl, 0, "\treturn (0);\n}\n");
 
 	if (!anon) {
