@@ -45,17 +45,50 @@ static void v_matchproto_(sym_act_f)
 vcc_act_call(struct vcc *tl, struct token *t, struct symbol *sym)
 {
 	struct token *t0;
+	unsigned u;
 
 	(void)t;
 	ExpectErr(tl, ID);
 	t0 = tl->t;
-	sym = VCC_SymbolGet(tl, SYM_MAIN, SYM_SUB, SYMTAB_CREATE, XREF_REF);
-	if (sym != NULL) {
+	sym = VCC_SymbolGet(tl, SYM_MAIN, SYM_NONE, SYMTAB_PARTIAL_NOERR,
+	    XREF_REF);
+	if (sym == NULL)
+		sym = VCC_SymbolGet(tl, SYM_MAIN, SYM_SUB, SYMTAB_CREATE,
+		    XREF_REF);
+
+	if (sym == NULL)
+		return;
+
+	if (sym->kind == SYM_SUB) {
 		vcc_AddCall(tl, t0, sym);
 		VCC_GlobalSymbol(sym, SUB, "VGC_function");
-		Fb(tl, 1, "%s(ctx);\n", sym->rname);
+
+		Fb(tl, 1, "%s(ctx, VSUB_STATIC, NULL);\n", sym->lname);
 		SkipToken(tl, ';');
+		return;
 	}
+
+	tl->t = t0;
+	u = tl->unique++;
+
+	Fb(tl, 1, "{\n");
+	tl->indent += INDENT;
+	Fb(tl, 2, "VCL_SUB call_%u =\n", u);
+	tl->indent += INDENT;
+	vcc_Expr(tl, SUB);
+	Fb(tl, 2, ";\n\n");
+	SkipToken(tl, ';');
+	tl->indent -= INDENT;
+	Fb(tl, 2, "if (call_%u == NULL) {\n", u);
+	Fb(tl, 2, "  VRT_fail(ctx, \"Tried to call NULL SUB near"
+	   " source %%u line %%u\",\n");
+	Fb(tl, 2, "    VGC_ref[%u].source,\n", tl->cnt);
+	Fb(tl, 2, "    VGC_ref[%u].line);\n", tl->cnt);
+	Fb(tl, 2, "  END_;\n");
+	Fb(tl, 2, "}\n");
+	Fb(tl, 2, "call_%u->func(ctx, VSUB_STATIC, NULL);\n", u);
+	tl->indent -= INDENT;
+	Fb(tl, 1, "}\n");
 }
 
 /*--------------------------------------------------------------------*/
@@ -167,7 +200,7 @@ vcc_act_unset(struct vcc *tl, struct token *t, struct symbol *sym)
 	AN(sym);
 	if (sym->u_methods == 0) {
 		vcc_ErrWhere2(tl, t, tl->t);
-		VSB_printf(tl->sb, "Variable cannot be unset.\n");
+		VSB_cat(tl->sb, "Variable cannot be unset.\n");
 		return;
 	}
 	vcc_AddUses(tl, t, tl->t, sym, XREF_UNSET);
@@ -184,10 +217,9 @@ vcc_act_ban(struct vcc *tl, struct token *t, struct symbol *sym)
 	(void)t;
 	(void)sym;
 
-	ExpectErr(tl, '(');
-	vcc_NextToken(tl);
+	SkipToken(tl, '(');
 
-	Fb(tl, 1, "VRT_ban_string(ctx, \n");
+	Fb(tl, 1, "(void) VRT_ban_string(ctx, \n");
 	tl->indent += INDENT;
 	vcc_Expr(tl, STRING);
 	tl->indent -= INDENT;
@@ -315,7 +347,7 @@ vcc_act_return_vcl(struct vcc *tl)
 static void v_matchproto_(sym_act_f)
 vcc_act_return(struct vcc *tl, struct token *t, struct symbol *sym)
 {
-	unsigned hand;
+	unsigned hand, mask;
 	const char *h;
 
 	(void)t;
@@ -331,20 +363,23 @@ vcc_act_return(struct vcc *tl, struct token *t, struct symbol *sym)
 
 	hand = VCL_RET_MAX;
 	h = NULL;
+	mask = 0;
 #define VCL_RET_MAC(l, U, B)				\
 		if (vcc_IdIs(tl->t, #l)) {		\
 			hand = VCL_RET_ ## U;		\
 			h = #U;				\
+			mask = (B);			\
 		}
 #include "tbl/vcl_returns.h"
 	if (h == NULL) {
-		VSB_printf(tl->sb, "Expected return action name.\n");
+		VSB_cat(tl->sb, "Expected return action name.\n");
 		vcc_ErrWhere(tl, tl->t);
 		ERRCHK(tl);
 	}
 	assert(hand < VCL_RET_MAX);
+	AN(mask);
 
-	vcc_ProcAction(tl->curproc, hand, tl->t);
+	vcc_ProcAction(tl->curproc, hand, mask, tl->t);
 	vcc_NextToken(tl);
 	if (tl->t->tok == '(') {
 		if (hand == VCL_RET_SYNTH || hand == VCL_RET_ERROR)
@@ -356,12 +391,12 @@ vcc_act_return(struct vcc *tl, struct token *t, struct symbol *sym)
 		else if (hand == VCL_RET_FAIL)
 			vcc_act_return_fail(tl);
 		else {
-			VSB_printf(tl->sb, "Arguments not allowed.\n");
+			VSB_cat(tl->sb, "Arguments not allowed.\n");
 			vcc_ErrWhere(tl, tl->t);
 		}
 	} else {
 		if (hand == VCL_RET_SYNTH || hand == VCL_RET_VCL) {
-			VSB_printf(tl->sb, "Missing argument.\n");
+			VSB_cat(tl->sb, "Missing argument.\n");
 			vcc_ErrWhere(tl, tl->t);
 		}
 	}
@@ -387,7 +422,7 @@ vcc_act_synthetic(struct vcc *tl, struct token *t, struct symbol *sym)
 	ERRCHK(tl);
 	vcc_NextToken(tl);
 
-	Fb(tl, 1, "VRT_synth_page(ctx, ");
+	Fb(tl, 1, "VRT_synth_strands(ctx, ");
 	vcc_Expr(tl, STRANDS);
 	ERRCHK(tl);
 	Fb(tl, 1, ");\n");
