@@ -72,6 +72,28 @@ static const struct method method_tab[] = {
 	{ NULL, 0U, 0}
 };
 
+struct vcc *vcc_builtin;
+
+/*--------------------------------------------------------------------*/
+
+static void
+vcc_vcl_met2c(struct vsb *vsb, unsigned method)
+{
+	int d = 0;
+
+	//lint -e{774} Boolean within 'if' always evaluates to False
+#define VCL_MET_MAC(l,U,t,b)			\
+	if (method & VCL_MET_##U) {		\
+		if (d)				\
+			VSB_putc(vsb, '|');	\
+		VSB_cat(vsb, "VCL_MET_" #U);	\
+		d = 1;				\
+	}
+#include "tbl/vcl_returns.h"
+	AN(d);
+}
+
+
 /*--------------------------------------------------------------------*/
 
 void * v_matchproto_(TlAlloc)
@@ -140,31 +162,106 @@ vcc_NewProc(struct vcc *tl, struct symbol *sym)
 	AN(p->body);
 	p->cname = VSB_new_auto();
 	AN(p->cname);
+	p->okmask = VCL_MET_TASK_ALL;
 	sym->proc = p;
+	p->sym = sym;
 	return (p);
 }
 
 static void
 vcc_EmitProc(struct vcc *tl, struct proc *p)
 {
+	struct vsb *vsbm;
+	unsigned mask, nsub;
+	const char *maskcmp;
+	const char *cc_adv;
+	int dyn = (p->sym->nref > p->called);
+
+	AN(p->okmask);
 	AZ(VSB_finish(p->cname));
 	AZ(VSB_finish(p->prologue));
 	AZ(VSB_finish(p->body));
+	AN(p->sym);
+
+	if (p->method) {
+		mask = p->method->bitval;
+		maskcmp = "==";
+	} else {
+		mask = p->okmask;
+		maskcmp = "&";
+	}
+
+	if ((mask & VCL_MET_TASK_H) && (mask & ~VCL_MET_TASK_H) == 0)
+		cc_adv = "v_dont_optimize ";
+	else
+		cc_adv = "";
+
+	nsub = tl->nsub++;
+
 	Fh(tl, 1, "vcl_func_f %s;\n", VSB_data(p->cname));
-	/*
-	 * TODO: v_dont_optimize for custom subs called from vcl_init/fini only
-	 *
-	 * Needs infrastructure similar to that in #3163 : custom subs need a
-	 * mask containing the builtin subs calling the custom sub. If that is
-	 * no larger than VCL_MET_TASK_H, we can enable v_dont_optimize
-	 */
-	Fc(tl, 1, "\nvoid %sv_matchproto_(vcl_func_f)\n",
-	   p->method && p->method->bitval & VCL_MET_TASK_H ?
-	   "v_dont_optimize " : "");
-	Fc(tl, 1, "%s(VRT_CTX)\n", VSB_data(p->cname));
-	Fc(tl, 1, "{\n%s\n%s}\n", VSB_data(p->prologue), VSB_data(p->body));
+	Fh(tl, 1, "const struct vcl_sub sub_%s[1] = {{\n",
+	   VSB_data(p->cname));
+	Fh(tl, 1, "\t.magic\t\t= VCL_SUB_MAGIC,\n");
+	Fh(tl, 1, "\t.methods\t= 0x%x,\n", p->okmask);
+	Fh(tl, 1, "\t.name\t\t= \"%.*s\",\n", PF(p->name));
+	Fh(tl, 1, "\t.vcl_conf\t= &VCL_conf,\n");
+	Fh(tl, 1, "\t.func\t\t= %s,\n", VSB_data(p->cname));
+	Fh(tl, 1, "\t.n\t\t= %d,\n", nsub);
+	Fh(tl, 1, "\t.nref\t\t= %d,\n", p->sym->nref);
+	Fh(tl, 1, "\t.called\t\t= %d\n", p->called);
+	Fh(tl, 1, "}};\n");
+
+	if (dyn) {
+		Fc(tl, 1, "\nstatic inline void %s\n", cc_adv);
+		Fc(tl, 1, "%s_checked(VRT_CTX)\n{\n", VSB_data(p->cname));
+	} else {
+		Fc(tl, 1, "\nvoid %sv_matchproto_(vcl_func_f)\n", cc_adv);
+		Fc(tl, 1, "%s(VRT_CTX, enum vcl_func_call_e call,\n",
+		    VSB_data(p->cname));
+		Fc(tl, 1, "    enum vcl_func_fail_e *failp)\n{\n");
+		Fc(tl, 1, "  assert(call == VSUB_STATIC);\n");
+		Fc(tl, 1, "  assert(failp == NULL);\n");
+	}
+
+	vsbm = VSB_new_auto();
+	AN(vsbm);
+	vcc_vcl_met2c(vsbm, mask);
+	AZ(VSB_finish(vsbm));
+	Fc(tl, 1, "  assert(ctx->method %s (%s));\n", maskcmp, VSB_data(vsbm));
+	VSB_destroy(&vsbm);
+	Fc(tl, 1, "%s\n%s}\n", VSB_data(p->prologue), VSB_data(p->body));
 	VSB_destroy(&p->body);
 	VSB_destroy(&p->prologue);
+
+	if (! dyn) {
+		VSB_destroy(&p->cname);
+		return;
+	}
+
+	/* wrapper to call the actual (_checked) function */
+	Fc(tl, 1, "\nvoid v_matchproto_(vcl_func_f)\n");
+	Fc(tl, 1, "%s(VRT_CTX, enum vcl_func_call_e call,\n",
+	    VSB_data(p->cname));
+	Fc(tl, 1, "    enum vcl_func_fail_e *failp)\n{\n");
+	Fc(tl, 1, "  enum vcl_func_fail_e fail;\n\n");
+	Fc(tl, 1, "  fail = VPI_Call_Check(ctx, &VCL_conf, 0x%x, %d);\n",
+	    mask, nsub);
+	Fc(tl, 1, "  if (failp)\n");
+	Fc(tl, 1, "    *failp = fail;\n");
+	Fc(tl, 1, "  else if (fail == VSUB_E_METHOD)\n");
+	Fc(tl, 1, "    VRT_fail(ctx, \"call to \\\"sub %.*s{}\\\""
+	    " not allowed from here\");\n", PF(p->name));
+	Fc(tl, 1, "  else if (fail == VSUB_E_RECURSE)\n");
+	Fc(tl, 1, "    VRT_fail(ctx, \"Recursive call to "
+	    "\\\"sub %.*s{}\\\"\");\n", PF(p->name));
+	Fc(tl, 1, "  else\n");
+	Fc(tl, 1, "    assert (fail == VSUB_E_OK);\n");
+	Fc(tl, 1, "  if (fail != VSUB_E_OK || call == VSUB_CHECK)\n");
+	Fc(tl, 1, "    return;\n");
+	Fc(tl, 1, "  VPI_Call_Begin(ctx, %d);\n", nsub);
+	Fc(tl, 1, "  %s_checked(ctx);\n", VSB_data(p->cname));
+	Fc(tl, 1, "  VPI_Call_End(ctx, %d);\n", nsub);
+	Fc(tl, 1, "}\n");
 	VSB_destroy(&p->cname);
 }
 
@@ -364,10 +461,10 @@ EmitInitFini(const struct vcc *tl)
 	Fc(tl, 0, "\n");
 	Fc(tl, 0, "\tif (*ctx->handling != VCL_RET_OK)\n");
 	Fc(tl, 0, "\t\treturn(1);\n");
+	Fc(tl, 0, "\t*ctx->handling = 0;\n");
 
 	VTAILQ_FOREACH(sy, &tl->sym_objects, sideways) {
 		Fc(tl, 0, "\tif (!%s) {\n", sy->rname);
-		Fc(tl, 0, "\t\t*ctx->handling = 0;\n");
 		Fc(tl, 0, "\t\tVRT_fail(ctx, "
 		    "\"Object %s not initialized\");\n" , sy->name);
 		Fc(tl, 0, "\t\treturn(1);\n");
@@ -493,6 +590,7 @@ EmitStruct(const struct vcc *tl)
 	Fc(tl, 0, "\t.ref = VGC_ref,\n");
 	Fc(tl, 0, "\t.nref = VGC_NREFS,\n");
 	Fc(tl, 0, "\t.nsrc = VGC_NSRCS,\n");
+	Fc(tl, 0, "\t.nsub = %d,\n", tl->subref > 0 ? tl->nsub : 0);
 	Fc(tl, 0, "\t.srcname = srcname,\n");
 	Fc(tl, 0, "\t.srcbody = srcbody,\n");
 	Fc(tl, 0, "\t.nvmod = %u,\n", tl->vmod_count);
@@ -506,18 +604,17 @@ EmitStruct(const struct vcc *tl)
 /*--------------------------------------------------------------------*/
 
 static struct source *
-vcc_new_source(const char *b, const char *e, const char *name)
+vcc_new_source(const char *src, const char *name)
 {
 	struct source *sp;
 
-	if (e == NULL)
-		e = strchr(b, '\0');
+	AN(src);
+	AN(name);
 	sp = calloc(1, sizeof *sp);
-	assert(sp != NULL);
-	sp->name = strdup(name);
-	AN(sp->name);
-	sp->b = b;
-	sp->e = e;
+	AN(sp);
+	REPLACE(sp->name, name);
+	sp->b = src;
+	sp->e = strchr(src, '\0');
 	return (sp);
 }
 
@@ -557,7 +654,7 @@ vcc_file_source(const struct vcc *tl, const char *fn)
 		free(fnp);
 		return (NULL);
 	}
-	sp = vcc_new_source(f, NULL, fnp);
+	sp = vcc_new_source(f, fnp);
 	free(fnp);
 	sp->freeit = f;
 	return (sp);
@@ -687,14 +784,17 @@ vcc_CompileSource(struct vcc *tl, struct source *sp, const char *jfile)
 	Fh(tl, 0, "\nextern const struct VCL_conf VCL_conf;\n");
 
 	/* Register and lex the main source */
-	VTAILQ_INSERT_TAIL(&tl->sources, sp, list);
-	sp->idx = tl->nsources++;
-	vcc_Lexer(tl, sp, 0);
-	if (tl->err)
-		return (NULL);
+	if (sp != NULL) {
+		AN(vcc_builtin);
+		VTAILQ_INSERT_TAIL(&tl->sources, sp, list);
+		sp->idx = tl->nsources++;
+		vcc_Lexer(tl, sp, 0);
+		if (tl->err)
+			return (NULL);
+	}
 
 	/* Register and lex the builtin VCL */
-	sp = vcc_new_source(tl->builtin_vcl, NULL, "Builtin");
+	sp = vcc_new_source(tl->builtin_vcl, "Builtin");
 	assert(sp != NULL);
 	VTAILQ_INSERT_TAIL(&tl->sources, sp, list);
 	sp->idx = tl->nsources++;
@@ -713,15 +813,6 @@ vcc_CompileSource(struct vcc *tl, struct source *sp, const char *jfile)
 	if (tl->err)
 		return (NULL);
 
-	/* Check if we have any backends at all */
-	if (tl->default_director == NULL) {
-		VSB_cat(tl->sb,
-		    "No backends or directors found in VCL program, "
-		    "at least one is necessary.\n");
-		tl->err = 1;
-		return (NULL);
-	}
-
 	/* Check for orphans */
 	if (vcc_CheckReferences(tl))
 		return (NULL);
@@ -734,15 +825,28 @@ vcc_CompileSource(struct vcc *tl, struct source *sp, const char *jfile)
 	if (vcc_CheckUses(tl) || tl->err)
 		return (NULL);
 
+	if (vcc_builtin == NULL)
+		return (NULL);
+
+	/* Check if we have any backends at all */
+	if (tl->default_director == NULL) {
+		VSB_cat(tl->sb,
+		    "No backends or directors found in VCL program, "
+		    "at least one is necessary.\n");
+		tl->err = 1;
+		return (NULL);
+	}
+
 	/* Tie vcl_init/fini in */
 	ifp = New_IniFin(tl);
-	VSB_cat(ifp->ini, "\tVGC_function_vcl_init(ctx);\n");
+	VSB_cat(ifp->ini, "\tVGC_function_vcl_init(ctx, VSUB_STATIC, NULL);\n");
 	/*
 	 * Because the failure could be half way into vcl_init{} so vcl_fini{}
 	 * must always be called, also on failure.
 	 */
 	ifp->ignore_errors = 1;
-	VSB_cat(ifp->fin, "\t\tVGC_function_vcl_fini(ctx);");
+	VSB_cat(ifp->fin, "\t\tVGC_function_vcl_fini(ctx, VSUB_STATIC, NULL);\n");
+	VSB_cat(ifp->fin, "\t\t\tVPI_vcl_fini(ctx);");
 
 	/* Emit method functions */
 	Fh(tl, 1, "\n");
@@ -785,6 +889,19 @@ vcc_CompileSource(struct vcc *tl, struct source *sp, const char *jfile)
 	return (vsb);
 }
 
+static struct vcc *
+vcc_ParseBuiltin(struct vcc *tl)
+{
+	struct vcc *tl_builtin;
+
+	CHECK_OBJ_NOTNULL(tl, VCC_MAGIC);
+	tl_builtin = VCC_New();
+	AN(tl_builtin);
+	VCC_Builtin_VCL(tl_builtin, tl->builtin_vcl);
+	AZ(vcc_CompileSource(tl_builtin, NULL, NULL));
+	return (tl_builtin);
+}
+
 /*--------------------------------------------------------------------
  * Report the range of VCL language we support
  */
@@ -817,8 +934,18 @@ VCC_Compile(struct vcc *tl, struct vsb **sb,
 	AN(vclsrcfile);
 	AN(ofile);
 	AN(jfile);
+
+	AZ(vcc_builtin);
+	vcc_builtin = vcc_ParseBuiltin(tl);
+	AN(vcc_builtin);
+	if (vcc_builtin->err) {
+		AZ(VSB_finish(vcc_builtin->sb));
+		*sb = vcc_builtin->sb;
+		return (-1);
+	}
+
 	if (vclsrc != NULL)
-		sp = vcc_new_source(vclsrc, NULL, vclsrcfile);
+		sp = vcc_new_source(vclsrc, vclsrcfile);
 	else
 		sp = vcc_file_source(tl, vclsrcfile);
 
