@@ -211,10 +211,12 @@ struct proc {
 	unsigned		ret_bitmap;
 	unsigned		called;
 	unsigned		active;
+	unsigned		okmask;
 	struct token		*return_tok[VCL_RET_MAX];
 	struct vsb		*cname;
 	struct vsb		*prologue;
 	struct vsb		*body;
+	struct symbol		*sym;
 };
 
 struct inifin {
@@ -264,6 +266,8 @@ struct vcc {
 						 */
 	struct vsb		*sb;
 	int			err;
+	unsigned		nsub;
+	unsigned		subref;	// SUB arguments present
 	struct proc		*curproc;
 	VTAILQ_HEAD(, proc)	procs;
 
@@ -284,6 +288,8 @@ struct vcc {
 	unsigned		vmod_count;
 };
 
+extern struct vcc *vcc_builtin;
+
 struct method {
 	const char		*name;
 	unsigned		ret_bitmap;
@@ -307,7 +313,7 @@ void vcc_Backend_Init(struct vcc *tl);
 void vcc_ParseProbe(struct vcc *tl);
 void vcc_ParseBackend(struct vcc *tl);
 struct fld_spec * vcc_FldSpec(struct vcc *tl, const char *first, ...);
-void vcc_IsField(struct vcc *tl, struct token **t, struct fld_spec *fs);
+void vcc_IsField(struct vcc *tl, const struct token **t, struct fld_spec *fs);
 void vcc_FieldsOk(struct vcc *tl, const struct fld_spec *fs);
 
 /* vcc_compile.c */
@@ -341,7 +347,7 @@ sym_expr_t vcc_Eval_Handle;
 sym_expr_t vcc_Eval_SymFunc;
 sym_expr_t vcc_Eval_TypeMethod;
 void vcc_Eval_Func(struct vcc *, const struct vjsn_val *,
-    const char *, const struct symbol *);
+    const char *, struct symbol *);
 void VCC_GlobalSymbol(struct symbol *, vcc_type_t fmt, const char *pfx);
 struct symbol *VCC_HandleSymbol(struct vcc *, vcc_type_t , const char *);
 void VCC_SymName(struct vsb *, const struct symbol *);
@@ -360,8 +366,6 @@ void Resolve_Sockaddr(struct vcc *tl, const char *host, const char *defport,
     const char **ipv4, const char **ipv4_ascii, const char **ipv6,
     const char **ipv6_ascii, const char **p_ascii, int maxips,
     const struct token *t_err, const char *errid);
-void Emit_UDS_Path(struct vcc *tl, const struct token *t_path,
-    const char *errid);
 double vcc_DurationUnit(struct vcc *);
 void vcc_ByteVal(struct vcc *, double *);
 void vcc_Duration(struct vcc *tl, double *);
@@ -380,11 +384,16 @@ extern const struct symxref XREF_NONE[1];
 extern const struct symxref XREF_DEF[1];
 extern const struct symxref XREF_REF[1];
 
-struct symmode { const char *name; };
+struct symmode {
+	const char	*name;
+	unsigned	noerr;
+	unsigned	partial;
+};
 extern const struct symmode SYMTAB_NOERR[1];
 extern const struct symmode SYMTAB_CREATE[1];
 extern const struct symmode SYMTAB_EXISTING[1];
 extern const struct symmode SYMTAB_PARTIAL[1];
+extern const struct symmode SYMTAB_PARTIAL_NOERR[1];
 
 struct symbol *VCC_SymbolGet(struct vcc *, vcc_ns_t, vcc_kind_t,
     const struct symmode *, const struct symxref *);
@@ -428,7 +437,7 @@ void VCC_InstanceInfo(struct vcc *tl);
 void VCC_XrefTable(struct vcc *);
 
 void vcc_AddCall(struct vcc *, struct token *, struct symbol *);
-void vcc_ProcAction(struct proc *p, unsigned action, struct token *t);
+void vcc_ProcAction(struct proc *, unsigned, unsigned, struct token *);
 int vcc_CheckAction(struct vcc *tl);
 
 
