@@ -153,6 +153,8 @@ vcc_expr_edit(struct vcc *tl, vcc_type_t fmt, const char *p, struct expr *e1,
 	struct expr *e, *e3;
 	int nl = 1;
 
+	(void) tl;
+
 	AN(e1);
 	e = vcc_new_expr(fmt);
 	while (*p != '\0') {
@@ -184,16 +186,10 @@ vcc_expr_edit(struct vcc *tl, vcc_type_t fmt, const char *p, struct expr *e1,
 		case 't':
 			e3 = (*p == 'T' ? e1 : e2);
 			AN(e3);
-			VSB_printf(tl->curproc->prologue,
-			    "  struct strands strs_%u_a;\n"
-			    "  const char * strs_%u_s[%d];\n",
-			    tl->unique, tl->unique, e3->nstr);
 			VSB_printf(e->vsb,
-			    "VPI_BundleStrands(%d, &strs_%u_a, strs_%u_s,"
-			    "\v+\n%s,\nvrt_magic_string_end\v-\n)",
-			    e3->nstr, tl->unique, tl->unique,
-			VSB_data(e3->vsb));
-			tl->unique++;
+			    "&(struct strands){.n = %d, .p = "
+			    "(const char *[%d]){\n%s\n}}",
+			    e3->nstr, e3->nstr, VSB_data(e3->vsb));
 			break;
 		case '1':
 			VSB_cat(e->vsb, VSB_data(e1->vsb));
@@ -355,7 +351,7 @@ vcc_Eval_Var(struct vcc *tl, struct expr **e, struct token *t,
  */
 
 static struct expr *
-vcc_priv_arg(struct vcc *tl, const char *p, const struct symbol *sym)
+vcc_priv_arg(struct vcc *tl, const char *p, struct symbol *sym)
 {
 	char buf[64];
 	struct inifin *ifp;
@@ -372,7 +368,7 @@ vcc_priv_arg(struct vcc *tl, const char *p, const struct symbol *sym)
 		bprintf(buf, "vmod_priv_%u", tl->unique++);
 		ifp = New_IniFin(tl);
 		Fh(tl, 0, "static struct vmod_priv %s;\n", buf);
-		VSB_printf(ifp->fin, "\tVRT_priv_fini(&%s);", buf);
+		VSB_printf(ifp->fin, "\tVRT_priv_fini(ctx, &%s);", buf);
 		return (vcc_mk_expr(VOID, "&%s", buf));
 	}
 
@@ -381,6 +377,7 @@ vcc_priv_arg(struct vcc *tl, const char *p, const struct symbol *sym)
 		marklist = &tl->curproc->priv_tasks;
 	} else if (!strcmp(p, "PRIV_TOP")) {
 		f = "top";
+		sym->r_methods &= VCL_MET_TASK_C;
 		marklist = &tl->curproc->priv_tops;
 	} else {
 		WRONG("Wrong PRIV_ type");
@@ -449,6 +446,8 @@ vcc_do_arg(struct vcc *tl, struct func_arg *fa)
 		vcc_do_enum(tl, fa, PF(tl->t));
 		SkipToken(tl, ID);
 	} else {
+		if (fa->type == SUB)
+			tl->subref++;
 		vcc_expr0(tl, &e2, fa->type);
 		ERRCHK(tl);
 		assert(e2->fmt == fa->type);
@@ -459,14 +458,14 @@ vcc_do_arg(struct vcc *tl, struct func_arg *fa)
 
 static void
 vcc_func(struct vcc *tl, struct expr **e, const void *priv,
-    const char *extra, const struct symbol *sym)
+    const char *extra, struct symbol *sym)
 {
 	vcc_type_t rfmt;
 	const char *cfunc;
 	struct expr *e1;
 	struct func_arg *fa, *fa2;
 	VTAILQ_HEAD(,func_arg) head;
-	struct token *t1;
+	struct token *tf, *t1;
 	const struct vjsn_val *vv, *vvp;
 	const char *sa, *extra_sep;
 	char ssa[64];
@@ -496,6 +495,7 @@ vcc_func(struct vcc *tl, struct expr **e, const void *priv,
 		AN((*e)->instance);
 		extra = (*e)->instance->rname;
 	}
+	tf = VTAILQ_PREV(tl->t, tokenhead, list);
 	SkipToken(tl, '(');
 	if (extra == NULL) {
 		extra = "";
@@ -635,6 +635,7 @@ vcc_func(struct vcc *tl, struct expr **e, const void *priv,
 		*e = vcc_expr_edit(tl, e1->fmt, "\v1\v-\n)", e1, NULL);
 	}
 	SkipToken(tl, ')');
+	vcc_AddUses(tl, tf, NULL, sym, XREF_READ);
 }
 
 
@@ -643,7 +644,7 @@ vcc_func(struct vcc *tl, struct expr **e, const void *priv,
 
 void
 vcc_Eval_Func(struct vcc *tl, const struct vjsn_val *spec,
-    const char *extra, const struct symbol *sym)
+    const char *extra, struct symbol *sym)
 {
 	struct expr *e = NULL;
 
@@ -775,6 +776,10 @@ vcc_expr5(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 			ERRCHK(tl);
 			e1 = vcc_mk_expr(IP, "%s", ip);
 			ERRCHK(tl);
+		} else if (fmt == REGEX) {
+			e1 = vcc_new_expr(REGEX);
+			vcc_regexp(tl, e1->vsb);
+			AZ(VSB_finish(e1->vsb));
 		} else {
 			e1 = vcc_new_expr(STRINGS);
 			EncToken(e1->vsb, tl->t);
@@ -897,7 +902,7 @@ vcc_expr4(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 /*--------------------------------------------------------------------
  * SYNTAX:
  *    ExprMul:
- *      Expr4 { {'*'|'/'} Expr4 } *
+ *      Expr4 { {'*'|'/'|'%'} Expr4 } *
  */
 
 static void
@@ -906,13 +911,20 @@ vcc_expr_mul(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 	struct expr *e2;
 	vcc_type_t f2;
 	struct token *tk;
+	char buf[24];
 
 	*e = NULL;
 	vcc_expr4(tl, e, fmt);
 	ERRCHK(tl);
 	AN(*e);
 
-	while (tl->t->tok == '*' || tl->t->tok == '/') {
+	while (tl->t->tok == '*' || tl->t->tok == '/' || tl->t->tok == '%') {
+		if (tl->t->tok == '%' && ((*e)->fmt != INT)) {
+			VSB_printf(tl->sb,
+			    "Operator %% only possible on INT.\n");
+			vcc_ErrWhere(tl, tl->t);
+			return;
+		}
 		f2 = (*e)->fmt->multype;
 		if (f2 == NULL) {
 			VSB_printf(tl->sb,
@@ -931,10 +943,8 @@ vcc_expr_mul(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 			vcc_ErrWhere(tl, tk);
 			return;
 		}
-		if (tk->tok == '*')
-			*e = vcc_expr_edit(tl, (*e)->fmt, "(\v1*\v2)", *e, e2);
-		else
-			*e = vcc_expr_edit(tl, (*e)->fmt, "(\v1/\v2)", *e, e2);
+		bprintf(buf, "(\v1%c\v2)", tk->tok);
+		*e = vcc_expr_edit(tl, (*e)->fmt, buf, *e, e2);
 	}
 }
 
@@ -1079,20 +1089,15 @@ cmp_simple(struct vcc *tl, struct expr **e, const struct cmps *cp)
 static void v_matchproto_(cmp_f)
 cmp_regexp(struct vcc *tl, struct expr **e, const struct cmps *cp)
 {
+	struct expr *e2;
 	char buf[128];
-	struct vsb vsb;
 
 	*e = vcc_expr_edit(tl, STRING, "\vS", *e, NULL);
 	vcc_NextToken(tl);
-	ExpectErr(tl, CSTR);
-	AN(VSB_init(&vsb, buf, sizeof buf));
-	VSB_printf(&vsb, "%sVRT_re_match(ctx, \v1, ", cp->emit);
-	vcc_regexp(tl, &vsb);
+	vcc_expr0(tl, &e2, REGEX);
 	ERRCHK(tl);
-	VSB_cat(&vsb, ")");
-	AZ(VSB_finish(&vsb));
-	*e = vcc_expr_edit(tl, BOOL, VSB_data(&vsb), *e, NULL);
-	VSB_fini(&vsb);
+	bprintf(buf, "%sVRT_re_match(ctx, \v1, \v2)", cp->emit);
+	*e = vcc_expr_edit(tl, BOOL, buf, *e, e2);
 }
 
 static void v_matchproto_(cmp_f)
@@ -1346,6 +1351,8 @@ vcc_expr0(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 	t1 = tl->t;
 	if (fmt->stringform)
 		vcc_expr_cor(tl, e, STRINGS);
+	else if (fmt == REGEX)
+		vcc_expr4(tl, e, REGEX);
 	else
 		vcc_expr_cor(tl, e, fmt);
 	ERRCHK(tl);
@@ -1431,7 +1438,7 @@ vcc_Act_Call(struct vcc *tl, struct token *t, struct symbol *sym)
 		SkipToken(tl, ';');
 		VSB_cat(tl->fb, ";\n");
 	} else if (t != tl->t) {
-		VSB_printf(tl->sb, "While compiling function call:\n\n");
+		VSB_cat(tl->sb, "While compiling function call:\n\n");
 		vcc_ErrWhere2(tl, t, tl->t);
 	}
 	vcc_delete_expr(e);
@@ -1461,10 +1468,9 @@ static void v_matchproto_(sym_expr_t)
 vcc_Eval_Regsub(struct vcc *tl, struct expr **e, struct token *t,
     struct symbol *sym, vcc_type_t fmt)
 {
-	struct expr *e2;
+	struct expr *e2, *e3;
 	int all = sym->eval_priv == NULL ? 0 : 1;
 	char buf[128];
-	struct vsb vsb;
 
 	(void)t;
 	(void)fmt;
@@ -1472,15 +1478,11 @@ vcc_Eval_Regsub(struct vcc *tl, struct expr **e, struct token *t,
 	vcc_expr0(tl, &e2, STRING);
 	ERRCHK(tl);
 	SkipToken(tl, ',');
-	ExpectErr(tl, CSTR);
-
-	AN(VSB_init(&vsb, buf, sizeof buf));
-	VSB_printf(&vsb, "VRT_regsub(ctx, %d,\v+\n\v1,\n", all);
-	vcc_regexp(tl, &vsb);
+	vcc_expr0(tl, &e3, REGEX);
 	ERRCHK(tl);
-	AZ(VSB_finish(&vsb));
-	*e = vcc_expr_edit(tl, STRING, VSB_data(&vsb), e2, NULL);
-	VSB_fini(&vsb);
+
+	bprintf(buf, "VRT_regsub(ctx, %d,\v+\n\v1,\n\v2", all);
+	*e = vcc_expr_edit(tl, STRING, buf, e2, e3);
 	SkipToken(tl, ',');
 	vcc_expr0(tl, &e2, STRING);
 	ERRCHK(tl);
@@ -1521,8 +1523,7 @@ vcc_Eval_Default(struct vcc *tl, struct expr **e, struct token *t,
 	else if (fmt == BACKEND)
 		*e = vcc_mk_expr(BACKEND, "*(VCL_conf.default_director)");
 	else {
-		VSB_printf(tl->sb,
-		    "Symbol 'default' is a reserved word.\n");
+		VSB_cat(tl->sb, "Symbol 'default' is a reserved word.\n");
 		vcc_ErrWhere(tl, t);
 	}
 }
