@@ -27,6 +27,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
+/*lint -save -esym(759, SYMTAB_NOERR) -esym(765, SYMTAB_NOERR)*/
 
 #include "config.h"
 
@@ -231,17 +232,17 @@ vcc_sym_in_tab(struct vcc *tl, struct symtab *st,
 	}
 	pst = st->parent;
 	if (pst == NULL)
-		return(sym);
+		return (sym);
 	psym = VTAILQ_FIRST(&pst->symbols);
 	if (psym == NULL)
-		return(sym);
+		return (sym);
 	if (psym->wildcard == NULL)
-		return(sym);
+		return (sym);
 
 	sym = vcc_new_symbol(tl, st, kind, vlo, vhi);
 	psym->wildcard(tl, psym, sym);
 	if (tl->err)
-		return(NULL);
+		return (NULL);
 	return (sym);
 }
 
@@ -249,10 +250,26 @@ vcc_sym_in_tab(struct vcc *tl, struct symtab *st,
 const struct symxref XREF_NONE[1] = {{"xref_none"}};
 const struct symxref XREF_DEF[1] = {{"xref_def"}};
 const struct symxref XREF_REF[1] = {{"xref_ref"}};
-const struct symmode SYMTAB_NOERR[1] = {{"sym_noerror"}};
-const struct symmode SYMTAB_CREATE[1] = {{"sym_create"}};
-const struct symmode SYMTAB_EXISTING[1] = {{"Symbol not found"}};
-const struct symmode SYMTAB_PARTIAL[1] = {{"Symbol not found"}};
+
+const struct symmode SYMTAB_NOERR[1] = {{
+		.name = "sym_noerror",
+		.noerr = 1
+	}};
+const struct symmode SYMTAB_CREATE[1] = {{
+		.name = "sym_create"
+	}};
+const struct symmode SYMTAB_EXISTING[1] = {{
+		.name = "Symbol not found"
+	}};
+const struct symmode SYMTAB_PARTIAL[1] = {{
+		.name = "Symbol not found",
+		.partial = 1
+	}};
+const struct symmode SYMTAB_PARTIAL_NOERR[1] = {{
+		.name = "Symbol not found",
+		.partial = 1,
+		.noerr = 1
+	}};
 
 struct symbol *
 VCC_SymbolGet(struct vcc *tl, vcc_ns_t ns, vcc_kind_t kind,
@@ -264,6 +281,7 @@ VCC_SymbolGet(struct vcc *tl, vcc_ns_t ns, vcc_kind_t kind,
 
 	AN(tl);
 	CHECK_OBJ_NOTNULL(ns, VCC_NAMESPACE_MAGIC);
+	AN(ns->name);
 	CHECK_OBJ_NOTNULL(kind, KIND_MAGIC);
 	AN(e);
 	AN(x);
@@ -297,15 +315,15 @@ VCC_SymbolGet(struct vcc *tl, vcc_ns_t ns, vcc_kind_t kind,
 			break;
 		tn = tn1;
 	}
-	if (sym != NULL && sym->kind == SYM_VMOD && e == SYMTAB_PARTIAL)
+	if (sym != NULL && sym->kind == SYM_VMOD && e->partial)
 		e = SYMTAB_EXISTING;
-	if (sym != NULL && e == SYMTAB_PARTIAL) {
+	if (sym != NULL && e->partial) {
 		st = st2;
 		tn = tn2;
 	} else if (st != st2) {
 		sym = NULL;
 	}
-	if (sym == NULL && e == SYMTAB_NOERR)
+	if (sym == NULL && e->noerr)
 		return (sym);
 	AN(st);
 	AN(tn);
@@ -336,7 +354,8 @@ VCC_SymbolGet(struct vcc *tl, vcc_ns_t ns, vcc_kind_t kind,
 		VSB_cat(tl->sb, "Symbol '");
 		for (tn1 = t0; tn1 != tl->t; tn1 = VTAILQ_NEXT(tn1, list))
 			VSB_printf(tl->sb, "%.*s", PF(tn1));
-		VSB_printf(tl->sb, "' has wrong type (%s): ", sym->kind->name);
+		VSB_printf(tl->sb, "' has wrong type (%s), expected %s:",
+		    sym->kind->name, kind->name);
 		VSB_cat(tl->sb, "\nAt: ");
 		vcc_ErrWhere2(tl, t0, tl->t);
 		if (sym->def_b != NULL) {
@@ -443,6 +462,7 @@ vcc_walksymbols(struct vcc *tl, const struct symtab *root,
 			assert(strcasecmp(st1->name, st2->name) >= 0);
 		st2 = st1;
 		vcc_walksymbols(tl, st1, func, kind);
+		ERRCHK(tl);
 	}
 }
 
@@ -467,6 +487,15 @@ VCC_GlobalSymbol(struct symbol *sym, vcc_type_t type, const char *pfx)
 	VSB_printf(vsb, "%s_", pfx);
 	VCC_PrintCName(vsb, sym->name, NULL);
 	AZ(VSB_finish(vsb));
+	sym->lname = strdup(VSB_data(vsb));
+	AN(sym->lname);
+	if (type == SUB) {
+		VSB_destroy(&vsb);
+		vsb = VSB_new_auto();
+		AN(vsb);
+		VSB_printf(vsb, "sub_%s", sym->lname);
+		AZ(VSB_finish(vsb));
+	}
 	sym->rname = strdup(VSB_data(vsb));
 	AN(sym->rname);
 	VSB_destroy(&vsb);
@@ -533,3 +562,4 @@ VCC_HandleSymbol(struct vcc *tl, vcc_type_t fmt, const char *pfx)
 		sym->def_b = t;
 	return (sym);
 }
+/*lint -restore */
