@@ -36,7 +36,6 @@
 #include <stdlib.h>
 #include <sys/socket.h>
 #include <sys/types.h>
-#include <sys/stat.h>
 #include <unistd.h>
 
 #include "vcc_compile.h"
@@ -58,9 +57,8 @@ vcc_regexp(struct vcc *tl, struct vsb *vgc_name)
 	int erroroffset;
 	struct inifin *ifp;
 
-	Expect(tl, CSTR);
-	if (tl->err)
-		return;
+	assert(tl->t->tok == CSTR);
+
 	t = VRE_compile(tl->t->dec, 0, &error, &erroroffset);
 	if (t == NULL) {
 		VSB_printf(tl->sb,
@@ -73,13 +71,12 @@ vcc_regexp(struct vcc *tl, struct vsb *vgc_name)
 	if (vgc_name)
 		VSB_cat(vgc_name, buf);
 
-	Fh(tl, 0, "static void *%s;\n", buf);
+	Fh(tl, 0, "static struct vre *%s;\n", buf);
 	ifp = New_IniFin(tl);
-	VSB_printf(ifp->ini, "\tVRT_re_init(&%s, ",buf);
+	VSB_printf(ifp->ini, "\tVPI_re_init(&%s, ",buf);
 	EncToken(ifp->ini, tl->t);
 	VSB_cat(ifp->ini, ");");
-	VSB_printf(ifp->fin, "\t\tVRT_re_fini(%s);", buf);
-	vcc_NextToken(tl);
+	VSB_printf(ifp->fin, "\t\tVPI_re_fini(%s);", buf);
 }
 
 /*
@@ -257,45 +254,6 @@ Resolve_Sockaddr(struct vcc *tl,
 	ZERO_OBJ(rss, sizeof rss);
 }
 
-/*
- * For UDS, we do not create a VSA. Check if it's an absolute path, can
- * be accessed, and is a socket. If so, just emit the path field and set
- * the IP suckaddrs to NULL.
- */
-void
-Emit_UDS_Path(struct vcc *tl, const struct token *t_path, const char *errid)
-{
-	struct stat st;
-
-	AN(t_path);
-	AN(t_path->dec);
-
-	if (*t_path->dec != '/') {
-		VSB_printf(tl->sb,
-			   "%s: Must be an absolute path:\n", errid);
-		vcc_ErrWhere(tl, t_path);
-		return;
-	}
-	errno = 0;
-	if (stat(t_path->dec, &st) != 0) {
-		int err = errno;
-		VSB_printf(tl->sb, "%s: Cannot stat: %s\n", errid,
-			   strerror(errno));
-		vcc_ErrWhere(tl, t_path);
-		if (err == ENOENT || err == EACCES)
-			vcc_Warn(tl);
-		else
-			return;
-	} else if (!S_ISSOCK(st.st_mode)) {
-		VSB_printf(tl->sb, "%s: Not a socket:\n", errid);
-		vcc_ErrWhere(tl, t_path);
-		return;
-	}
-	Fb(tl, 0, "\t.path = \"%s\",\n", t_path->dec);
-	Fb(tl, 0, "\t.ipv4_suckaddr = (void *) 0,\n");
-	Fb(tl, 0, "\t.ipv6_suckaddr = (void *) 0,\n");
-}
-
 /*--------------------------------------------------------------------
  * Recognize and convert units of duration, return seconds.
  */
